#include <iostream>
#include <math.h>
#include <float.h>
#include <GL/gl.h>
#include <GL/glut.h>

#include "mesh.h"
using namespace std;

//-----------------------------------------------------------------------------------------------------------------
// CONSTRUCTOR(S) & DESTRUCTOR
//----------------------------
Mesh::Mesh() : TriMesh(),
	       geometryOK_(false),
	       radius_(0.),
	       center_(0.,0.,0.), boundingboxMin_(+FLT_MAX,+FLT_MAX,+FLT_MAX), boundingboxMax_(-FLT_MAX,-FLT_MAX,-FLT_MAX),
	       octreeOK_(false), octree_(NULL),
	       curvatureOK_(false)
{
  request_face_colors();
  request_face_normals();
  request_vertex_colors();
  request_vertex_normals();
  request_vertex_texcoords2D();
}

Mesh::~Mesh() {
  delete octree_;
}

//-----------------------------------------------------------------------------------------------------------------
// INITIALIZATION
//---------------
const bool Mesh::load(const char* fileName) {

  // Load mesh geometry from file
  //-----------------------------
  OpenMesh::IO::Options ropt;
  ropt += OpenMesh::IO::Options::FaceColor;
  geometryOK_ = OpenMesh::IO::read_mesh(*this,fileName,ropt);

  if (!geometryOK_){
    cerr<<"Mesh::load: Error loading mesh from file ["<<fileName<<"]!"<<endl;
  } 
  else {
    curvatureOK_=false;
    
    update_normals();
    updateBoundingVolume();
    smoothNormals();
    
    cout<<"Mesh::load: Mesh successfully imported from ["<<fileName<<"]"<<endl;
  }
  return geometryOK_;
}

//----------------------------------------------------------------------------------------
// Computes the limits of the bounding box of the mesh
//----------------------------------------------------
void Mesh::updateBoundingVolume() {

  float xmin,ymin,zmin,xmax,ymax,zmax;
  xmin=ymin=zmin= FLT_MAX;
  xmax=ymax=zmax=-FLT_MAX;

  int nbVertices=0;
  center_[0]=center_[1]=center_[2]=0.;

  TriMesh::Point p_i;
  TriMesh::VertexIter v_i, v_End = vertices_end();
  for (v_i=vertices_begin();v_i!=v_End;++v_i) {

    p_i = point(v_i);
    center_[0] += p_i[0]; 
    center_[1] += p_i[1]; 
    center_[2] += p_i[2];
    nbVertices++;

    if(p_i[0]<xmin) xmin=p_i[0];
    if(p_i[0]>xmax) xmax=p_i[0];
    
    if(p_i[1]<ymin) ymin=p_i[1];
    if(p_i[1]>ymax) ymax=p_i[1];
    
    if(p_i[2]<zmin) zmin=p_i[2];
    if(p_i[2]>zmax) zmax=p_i[2]; 
  }

  // BOUNDING BOX
  boundingboxMin_ = OpenMesh::Vec3f(xmin, ymin, zmin);
  boundingboxMax_ = OpenMesh::Vec3f(xmax, ymax, zmax);

  // BOUNDING SPHERE
  // Center  
  center_[0]/=nbVertices;
  center_[1]/=nbVertices;
  center_[2]/=nbVertices;  

  // Radius
  float x,y,z;
  if(fabs(xmin)>xmax) x=fabs(xmin);
  else                x=xmax;
  if(fabs(ymin)>ymax) y=fabs(ymin);
  else                y=ymax;
  if(fabs(zmin)>zmax) z=fabs(zmin);
  else                z=zmax;

  if(x<y && x<z)      x=z;
  else if(y<x && y<z) y=z;

  radius_ = sqrt(x*x+y*y);
}


//----------------------------------------------------------------------------------------
// Remplace each vertex's normal by the normal of the center of gravity of its 1-ring
//-----------------------------------------------------------------------------------
void Mesh::smoothNormals() {
  if(!geometryOK_) return;

  OpenMesh::VPropHandleT<TriMesh::Normal> smoothedNormals;
  add_property(smoothedNormals);
  //------------------
  TriMesh::VertexIter       v_i, v_End(vertices_end());
  TriMesh::VertexVertexIter v_ij;

  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) {
   
    if (valence(v_i)>0) {

      property(smoothedNormals,v_i).vectorize(0.0f);
      for (v_ij=vv_iter(v_i); v_ij; ++v_ij)
	property(smoothedNormals,v_i) += calc_vertex_normal(v_ij);

      property(smoothedNormals,v_i) /= valence(v_i);
      property(smoothedNormals,v_i).normalize();
    }
  }
    
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i)
    if (!is_boundary(v_i)) 
      set_normal(v_i, property(smoothedNormals,v_i));
  //--------------------
  remove_property(smoothedNormals);
}

//-----------------------------------------------------------------------------
// RENDERING
//----------

void Mesh::draw() const {
  if (!geometryOK_) return;

  const float K = 1./255.;

  glBegin(GL_TRIANGLES);
  TriMesh::ConstFaceIter f_i, f_End=faces_end();
  for (f_i=faces_begin(); f_i!=f_End; ++f_i) {

    glColor4f(K*color(f_i)[0], K*color(f_i)[1], K*color(f_i)[2], 1.);
    TriMesh::ConstFaceVertexIter v_ij;
    for (v_ij=cfv_iter(f_i); v_ij; ++v_ij) {
      
      // GL_TEXTURE0_ARB :: Texture Coordinates
      //-----------------
      // glMultiTexCoord4f(GL_TEXTURE0_ARB,texcoord2D(v_ij)[0],texcoord2D(v_ij)[1],0.,0.);
      glNormal3fv(normal(v_ij));
      glVertex3fv(point (v_ij));
    }
  }
  glEnd();
  glFinish();
}


void Mesh::draw(const int& R, const int& G, const int& B, const int& A) const {
  if (!geometryOK_) return;

  const float K = 1./255.;

  glBegin(GL_TRIANGLES);
  TriMesh::ConstFaceIter f_i, f_End=faces_end();
  for (f_i=faces_begin(); f_i!=f_End; ++f_i) {

    glColor4f(K*R, K*G, K*B, A);
    TriMesh::ConstFaceVertexIter v_ij;
    for (v_ij=cfv_iter(f_i); v_ij; ++v_ij) {
      
      // GL_TEXTURE0_ARB :: Texture Coordinates
      //-----------------
      // glMultiTexCoord4f(GL_TEXTURE0_ARB,texcoord2D(v_ij)[0],texcoord2D(v_ij)[1],0.,0.);
      glNormal3fv(normal(v_ij));
      glVertex3fv(point (v_ij));
    }
  }
  glEnd();
  glFinish();
}


void Mesh::smoothDraw() const {
  if (!geometryOK_) return;
  
  const float K = 1./255.;

  glBegin(GL_TRIANGLES);
  TriMesh::ConstFaceIter f_i, f_End=faces_end();
  for (f_i=faces_begin(); f_i!=f_End; ++f_i) {

    glColor4f(K*color(f_i)[0], K*color(f_i)[1], K*color(f_i)[2]/255., 1.);

    TriMesh::ConstFaceVertexIter v_ij;
    for (v_ij=cfv_iter(f_i); v_ij; ++v_ij) {
      glNormal3fv  (normal    (v_ij));
      glVertex3fv  (property  (vertices_,v_ij));
    }
  }
  glEnd(); 
  glFinish();
}

//--------------------------------------------------------------------------------
// CURVATURE MAGNITUDES AND DIRECTIONS
// -----------------------------------
void Mesh::computeCurvatures(const int&   geometrySmoothingIterations,
			     const float& scaleFactor1, 
			     const float& scaleFactor2,
			     const int&   curvatureSmoothingIterations, 
			     const float& smoothParameter) {
  if (!geometryOK_) return;

  curvatureOK_=false;

  // Removing previous properties
  //-----------------------------
  remove_property(vertices_);
  remove_property(minimalCurvatures_);
  remove_property(maximalCurvatures_);
  remove_property(minimalCurvatureDirections_);
  remove_property(maximalCurvatureDirections_);

    // Allocate new properties
  //------------------------
  add_property(vertices_);
  add_property(minimalCurvatures_);
  add_property(maximalCurvatures_);
  add_property(minimalCurvatureDirections_);
  add_property(maximalCurvatureDirections_);

  // Geometry initialization (to-be-smoothed version)
  //------------------------
  TriMesh::VertexIter v_i, v_End(vertices_end());
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) property(vertices_,v_i) = point(v_i);
   
  // Smooth the geometry [Gabriel TAUBIN 95]
  //----------------------------------------
  for (int iter=0; iter<geometrySmoothingIterations; iter++) {
    smoothGeometry(scaleFactor1, scaleFactor2);  
    cout<<"Mesh::computeCurvatures: Geometry smoothing iteration "<<iter<<" finished."<<endl;
  }

  // Compute the curvature tensor [Gabriel TAUBIN 95]
  //-------------------------------------------------
  computeCurvatureTensor();

  // Smooth the curvature tensor [Yutaka OHTAKE 01]
  //-----------------------------------------------
  for (int iter=0; iter<curvatureSmoothingIterations; iter++) {
    smoothCurvatures(smoothParameter);  
    cout<<"Mesh::computeCurvatures: Curvature smoothing iteration "<<iter<<" finished."<<endl;
  }
  cout<<endl;

  curvatureOK_=true;
}


void Mesh::smoothGeometry(const float& scaleFactor1, const float& scaleFactor2) {
  if (!geometryOK_) return;

  float factor1 = scaleFactor1;
  float factor2 = scaleFactor2;
  if (fabs(scaleFactor1)>fabs(scaleFactor1)) {
    cerr<<"Mesh::smoothGeometry: Swapped scale factors."<<endl;
    factor1 = scaleFactor2;
    factor2 = scaleFactor1;
  }
  if (factor1<0) {
    cerr<<"Mesh::smoothGeometry: Force scale factor 1 to be positive."<<endl;
    factor1 = -factor1;
  }
  if (factor2>0) {
    cerr<<"Mesh::smoothGeometry: Force scale factor 2 to be negative."<<endl;
    factor2 = -factor2;
  }

  OpenMesh::VPropHandleT<OpenMesh::Vec3f> displacements;
  add_property(displacements);

  TriMesh::VertexIter       v_i, v_End(vertices_end());
  TriMesh::VertexVertexIter v_ij;
  float w_ij;

  //----------------------------------------
  // Fisrt step
  //-----------
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) {
    property(displacements,v_i) = OpenMesh::Vec3f(0.,0.,0.);
    if (valence(v_i)>0) {
      w_ij = 1./(float)valence(v_i);
      for (v_ij=vv_iter(v_i); v_ij; ++v_ij)
	property(displacements,v_i) += w_ij*(property(vertices_,v_i)-property(vertices_,v_ij));
    }
  }
  //------------
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i)
    property(vertices_,v_i) += scaleFactor1 * property(displacements,v_i);
  //----------------------------------------
  // Second step
  //------------
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) {
    property(displacements,v_i) = OpenMesh::Vec3f(0.,0.,0.);
    if (valence(v_i)>0) {
      w_ij = 1./(float)valence(v_i);
      for (v_ij=vv_iter(v_i); v_ij; ++v_ij)
	property(displacements,v_i) += w_ij*(property(vertices_,v_i)-property(vertices_,v_ij));
    }
  }  
  //------------
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i)
    property(vertices_,v_i) += scaleFactor2 * property(displacements,v_i);
  //----------------------------------------
  remove_property(displacements);
}


void Mesh::computeCurvatureTensor() {
  if (!geometryOK_) return;

  float M_i[9];
  float N_i[9];
  float Q_i[9];
  float T_i[9]; 
  float area_ij, fullArea_i, k_ij, m_11, m_12, m_22, theta, curv1, curv2;
  OpenMesh::Vec3f e_1, n_i, v_ij, t_ij, curvDir1, curvDir2;

  e_1[0]=1.; e_1[1]=e_1[2]=0.;

  TriMesh::VertexIter   v_i, v_End=vertices_end();
  TriMesh::VertexHandle v_j;

  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) {
    
    // Computation of the normal weighted by the adjacent faces' area
    //---------------------------------------------------------------
    n_i[0]=n_i[1]=n_i[2]=0.;
    for (TriMesh::ConstVertexFaceIter vface=cvf_iter(v_i); vface; ++vface)
      n_i += faceArea(face_handle(vface)) * calc_face_normal(vface);

    if (isNan(n_i) || n_i.norm()==0) n_i=calc_vertex_normal(v_i);
    //-------------------
    n_i=n_i.normalize();

    // Computation of the vertices' curvatures
    //---------------------------------------- 
    fullArea_i=0.;
    for (int k=8; k>=0; --k) M_i[k]=0.;

    for (TriMesh::ConstVertexOHalfedgeIter e_ij=cvoh_iter(v_i); e_ij; ++e_ij) {
      v_j  = to_vertex_handle(e_ij);
      v_ij = property(vertices_,v_i)-property(vertices_,v_j);

      for (int k=2; k>=0; --k) 
	for (int l=2; l>=0; --l) 
	  N_i[3*k+l] = n_i[k]*n_i[l];

      t_ij[0]=t_ij[1]=t_ij[2]=0.;
      for (int k=2; k>=0; --k) 
	for (int l=2; l>=0; --l) 
	  if (k==l) t_ij[k] += (1-N_i[3*k+l]) * v_ij[l]; 
	  else      t_ij[k] +=   -N_i[3*k+l]  * v_ij[l];
      t_ij=t_ij.normalize();
      
      if (v_ij.norm()==0.) k_ij=0.;
      else                 k_ij=-2.*(n_i[0]*v_ij[0] + n_i[1]*v_ij[1] + n_i[2]*v_ij[2])/v_ij.norm();

      area_ij = 0.;
      if (face_handle(halfedge_handle(e_ij)).is_valid())
	area_ij += faceArea(face_handle(halfedge_handle(e_ij)));
      if (face_handle(opposite_halfedge_handle(e_ij)).is_valid())
	area_ij += faceArea(face_handle(opposite_halfedge_handle(halfedge_handle(e_ij))));

      fullArea_i += area_ij;
      
      for (int k=2; k>=0; --k) {
	for (int l=2; l>=0; --l) {
	  M_i[3*k+l] += area_ij * k_ij * t_ij[k] * t_ij[l];
	}
      }
    }
    
    for (int k=8; k>=0; --k) {
      if (!fullArea_i==0.) M_i[k] /= fullArea_i;
    }

    OpenMesh::Vec3f W_i;
    if ( (e_1+n_i).norm() > (e_1-n_i).norm() ) 
      W_i = (e_1+n_i).normalize();
    else 
      W_i = (e_1-n_i).normalize();

    for (int k=2; k>=0; --k) 
      for (int l=2; l>=0; --l)  
	Q_i[3*k+l] = -2*(W_i[k]*W_i[l]);
    Q_i[0]++; Q_i[4]++; Q_i[8]++;
    
    for (int k=8; k>=0; --k) 
      T_i[k]=0;
    // multiplication Qi^t Mi Qi
    for (int k=2; k>=0; --k)
      for (int l=2; l>=0; --l)
	for (int m=2; m>=0; --m)
	  T_i[3*k+l] += M_i[3*k+m]*Q_i[3*m+l];

    m_11 = Q_i[1]*T_i[1] + Q_i[4]*T_i[4] + Q_i[7]*T_i[7];
    m_12 = Q_i[2]*T_i[1] + Q_i[5]*T_i[4] + Q_i[8]*T_i[7];
    m_22 = Q_i[2]*T_i[2] + Q_i[5]*T_i[5] + Q_i[8]*T_i[8];
    
    // Givens Rotation angle
    theta = (m_22-m_11)/2*m_12;

    // Principal Curvature Directions
    curvDir1[0] = cos(theta)*Q_i[1] - sin(theta)*Q_i[2];
    curvDir1[1] = cos(theta)*Q_i[4] - sin(theta)*Q_i[5];
    curvDir1[2] = cos(theta)*Q_i[7] - sin(theta)*Q_i[8];

    curvDir2[0] = sin(theta)*Q_i[1] + cos(theta)*Q_i[2];
    curvDir2[1] = sin(theta)*Q_i[4] + cos(theta)*Q_i[5];
    curvDir2[2] = sin(theta)*Q_i[7] + cos(theta)*Q_i[8];
      
    // Associated Principal Curvatures
    curv1 = 3*m_11-m_22;
    curv2 = 3*m_22-m_11;

    // Normalizes curvature directions
    curvDir1 = curvDir1.normalize();
    curvDir2 = curvDir2.normalize(); 

    if (isNan(curvDir1)) 
      cerr<<"Mesh::computeCurvatures: Incorrect curvature direction at vertex "<<v_i<<"!"<<endl;
    if (isNan(curvDir2)) 
      cerr<<"Mesh::computeCurvatures: Incorrect curvature direction at vertex "<<v_i<<"!"<<endl;

    if (fabsf(curv1)<fabsf(curv2)) {
      property(maximalCurvatures_,v_i)          = curv1;
      property(maximalCurvatureDirections_,v_i) = curvDir2;

      property(minimalCurvatures_,v_i)          = curv2;   
      property(minimalCurvatureDirections_,v_i) = curvDir1;
    }
    else { 
      property(maximalCurvatures_,v_i)          = curv2;
      property(maximalCurvatureDirections_,v_i) = curvDir1;

      property(minimalCurvatures_,v_i)          = curv1;   
      property(minimalCurvatureDirections_,v_i) = curvDir2;
    }
  }
  curvatureOK_=true;
}

//-----------------------------------------------------------
// SMOOTH CURVATURE MAGNITUDES AND DIRECTIONS
//-------------------------------------------

void Mesh::smoothCurvatures(const float& smoothParameter) {
  if (!geometryOK_) return;

  if (!curvatureOK_) {
    cerr<<"Mesh::smoothCurvatures: Principal curvature directions not computed yet!"<<endl;
    return;
  }

  bool   chooseMinDir; 
  float  dist_ij, cos_ij, sin_ij, rho_ij;
  float* w_ij = new float[3];
  float* R_ij = new float[9]; 
  
  float           num_minCurv, den_minCurv, num_maxCurv, den_maxCurv;
  OpenMesh::Vec3f t_i, new_t_i, proj_t_i;
  OpenMesh::Vec3f n_i, n_ij, n_iXj, t_ij, Rt_ij; 
  n_i[0]=n_i[1]=n_i[2]=0.;

  TriMesh::VertexIter v_i, v_End=vertices_end(); 
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) {
    
    chooseMinDir = fabs( property(minimalCurvatures_,v_i) ) > fabs( property(maximalCurvatures_,v_i) );
    if (chooseMinDir) t_i = property(minimalCurvatureDirections_,v_i);
    else              t_i = property(maximalCurvatureDirections_,v_i);
    t_i.normalize();

    new_t_i[0] = new_t_i[1] = new_t_i[2] = 0.;
    num_minCurv = property(minimalCurvatures_, v_i); den_minCurv = 1.;
    num_maxCurv = property(maximalCurvatures_, v_i); den_maxCurv = 1.;
    for (TriMesh::ConstVertexVertexIter v_ij=cvv_iter(v_i); v_ij; ++v_ij) {
      
      if (chooseMinDir) t_ij = property(minimalCurvatureDirections_,v_ij);
      else              t_ij = property(maximalCurvatureDirections_,v_ij);
      
      // Rotation matrix (axis= n_iXj, angle= ang_ij)
      //---------------------------------------------
      n_i   = calc_vertex_normal(v_i);  n_i .normalize();
      n_ij  = calc_vertex_normal(v_ij); n_ij.normalize();
      n_iXj = cross( n_i,n_ij );
      
      cos_ij = dot( n_i,n_ij );
      sin_ij = sqrt(1 - cos_ij*cos_ij); if (isnan(sin_ij)) sin_ij=0.;
     
      
      // formule d'Olinde Rodrigues     
      //---------------------------
      R_ij[0]          = cos_ij + (1.-cos_ij) * n_iXj[0]*n_iXj[0];
      R_ij[4]          = cos_ij + (1.-cos_ij) * n_iXj[1]*n_iXj[1];
      R_ij[8]          = cos_ij + (1.-cos_ij) * n_iXj[2]*n_iXj[2];
      R_ij[1] = R_ij[3]=          (1.-cos_ij) * n_iXj[0]*n_iXj[1];
      R_ij[2] = R_ij[6]=          (1.-cos_ij) * n_iXj[0]*n_iXj[2]; 
      R_ij[5] = R_ij[7]=          (1.-cos_ij) * n_iXj[1]*n_iXj[2];
      
      R_ij[1]-= sin_ij*n_iXj[2]; R_ij[3]+= sin_ij*n_iXj[2];
      R_ij[2]+= sin_ij*n_iXj[1]; R_ij[6]-= sin_ij*n_iXj[1];
      R_ij[5]-= sin_ij*n_iXj[0]; R_ij[7]+= sin_ij*n_iXj[0];
      //---------------------------------------------
      
      // Rotated Direction
      //------------------
      Rt_ij[0] = R_ij[0]*t_ij[0] + R_ij[1]*t_ij[1] + R_ij[2]*t_ij[2];
      Rt_ij[1] = R_ij[3]*t_ij[0] + R_ij[4]*t_ij[1] + R_ij[5]*t_ij[2];
      Rt_ij[2] = R_ij[6]*t_ij[0] + R_ij[7]*t_ij[1] + R_ij[8]*t_ij[2];
      Rt_ij.normalize();
      //--------------------------------------------
      
      // Weight
      //-------
      dist_ij = (property(vertices_,v_i) - property(vertices_,v_ij)).norm();
      if (dist_ij==0.) {
	cerr<<"Mesh::smoothCurvatures : vertices "<<v_i<<" and "<<vertex_handle(v_ij)<<" at same position!"<<endl;
	w_ij[0] = w_ij[1] = w_ij[2] = 1.;
      }
      else {
	dist_ij *= smoothParameter;
	
	rho_ij  = acos( fabs(dot(t_i,Rt_ij)) ) / dist_ij; if (isnan(rho_ij)) rho_ij = 0.;
	rho_ij *= rho_ij;
	w_ij[0] = ( dot(t_i,Rt_ij)>0 )? exp(-rho_ij) : -exp(-rho_ij);

	rho_ij  = fabs(property(minimalCurvatures_,v_i)-property(minimalCurvatures_,v_ij)) / dist_ij;
	rho_ij *= rho_ij;
	w_ij[1] = exp(-rho_ij);

	rho_ij  = fabs(property(maximalCurvatures_,v_i)-property(maximalCurvatures_,v_ij)) / dist_ij;
	rho_ij *= rho_ij;
	w_ij[2] = exp(-rho_ij);	
      }
      new_t_i     += w_ij[0]*Rt_ij;
      num_minCurv += w_ij[1]*property(minimalCurvatures_,v_ij);
      den_minCurv += w_ij[1];
      num_maxCurv += w_ij[2]*property(maximalCurvatures_,v_ij);
      den_maxCurv += w_ij[2];
    }
    
    if (new_t_i[0]==0 && new_t_i[1]==0 && new_t_i[2]==0) new_t_i = t_i;
    new_t_i = new_t_i.normalize();
    
    // Linear interpolation with projection onto tangent plane
    //--------------------------------------------------------
    proj_t_i = cross(n_i, cross(new_t_i,n_i))                              .normalize();
    new_t_i  = ( (1-dot(new_t_i,n_i))*proj_t_i + dot(new_t_i,n_i)*new_t_i ).normalize();

    // Update curvature direction
    //---------------------------
    if (chooseMinDir) {
      property(minimalCurvatureDirections_,v_i) = new_t_i;
      property(maximalCurvatureDirections_,v_i) = cross(new_t_i,n_i).normalize();
    }
    else {
      property(maximalCurvatureDirections_,v_i) = new_t_i;
      property(minimalCurvatureDirections_,v_i) = cross(n_i,new_t_i).normalize();
    }

    // Update curvatures
    //------------------
    property(minimalCurvatures_,v_i) = num_minCurv/den_minCurv;
    property(maximalCurvatures_,v_i) = num_maxCurv/den_maxCurv;

  }
  delete [] R_ij;
  delete [] w_ij;
}

//----------------------------------------------------------------------------------------
float Mesh::minimalCurvature(const VertexHandle& vertex) const {
  if (!curvatureOK_) {
    cerr<<"Mesh::minimalCurvature: Curvature not computed yet!"<<endl;
    return 0.;
  }
  return property(minimalCurvatures_,vertex);
}
//----------------------------------------------------------------------------------------
float Mesh::maximalCurvature(const VertexHandle& vertex) const {
  if (!curvatureOK_) {
    cerr<<"Mesh::maximalCurvature: Curvature not computed yet!"<<endl;
    return 0.;
  }  
  return property(maximalCurvatures_,vertex);
}
//----------------------------------------------------------------------------------------
float Mesh::meanCurvature(const VertexHandle& vertex) const {
  if (!curvatureOK_) {
    cerr<<"Mesh::meanCurvature: Curvature not computed yet!"<<endl;
    return 0.;
  }  
  return .5*(property(minimalCurvatures_,vertex) + property(maximalCurvatures_,vertex));
}
//----------------------------------------------------------------------------------------
float Mesh::gaussianCurvature(const VertexHandle& vertex) const {
  if (!curvatureOK_) {
    cerr<<"Mesh::gaussianCurvature: Curvature not computed yet!"<<endl;
    return 0.;
  }  
  return property(minimalCurvatures_,vertex) * property(maximalCurvatures_,vertex);
}
//----------------------------------------------------------------------------------------
float Mesh::radialCurvature(const VertexHandle& vertex, 
			    const float& eyeX, const float& eyeY, const float& eyeZ) const {
  if (!curvatureOK_) {
    cerr<<"Mesh::gaussianCurvature: Curvature not computed yet!"<<endl;
    return 0.;
  }
  OpenMesh::Vec3f eye(eyeX, eyeY, eyeZ);
  OpenMesh::Vec3f V = (eye - property(vertices_,vertex)); V.normalize();
  OpenMesh::Vec3f N = normal(vertex);                     N.normalize();
  OpenMesh::Vec3f W = cross(N,cross(V,N));                W.normalize();

  float theta = angle(W,property(minimalCurvatureDirections_,vertex));
  float cos_theta_2 = cos(theta)*cos(theta);
  return cos_theta_2*property(minimalCurvatures_,vertex) + (1.-cos_theta_2)*property(maximalCurvatures_,vertex);
}
//----------------------------------------------------------------------------------------
OpenMesh::Vec3f Mesh::minimalCurvatureDirection(const VertexHandle& vertex) const {
  if (!curvatureOK_) {
    cerr<<"Mesh::minimalCurvatureDirection: Curvature directions not computed yet!"<<endl;
    return OpenMesh::Vec3f(0.,0.,0.);
  }
  return property(minimalCurvatureDirections_,vertex);
}
//----------------------------------------------------------------------------------------
OpenMesh::Vec3f Mesh::maximalCurvatureDirection(const VertexHandle& vertex) const {
  if (!curvatureOK_) {
    cerr<<"Mesh::maximalCurvatureDirection: Curvature directions not computed yet!"<<endl;
    return OpenMesh::Vec3f(0.,0.,0.);
  }
  return property(maximalCurvatureDirections_,vertex);
}


void Mesh::curvatureExtrema(float& min, float& max, 
			    const float& eyeX, const float& eyeY, const float& eyeZ) const {
  if (!curvatureOK_) {
    cerr<<"Mesh::curvatureExtrema: Curvature directions not computed yet!"<<endl;
    return;
  }

  min=+FLT_MAX; 
  max=-FLT_MAX;
  /*
  float curv;
  TriMesh::ConstVertexIter v_i, v_End=vertices_end();
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) {
    if (display_==MINIMAL_CURVATURE)
      curv = minimalCurvature(v_i);
    else if (display_==MAXIMAL_CURVATURE)
      curv = maximalCurvature(v_i);
    else if (display_==MEAN_CURVATURE)
      curv = meanCurvature(v_i);
    else if (display_==GAUSSIAN_CURVATURE)
      curv = gaussianCurvature(v_i);
    else if (display_==RADIAL_CURVATURE) {
      curv = radialCurvature(v_i,eyeX,eyeY,eyeZ);
    }
    else break;
    
    if (curv<min) min=curv;
    if (curv>max) max=curv;
  }
  */
}
//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------

// AMBIENT OCCLUSION
//------------------

void Mesh::computeAmbientOcclusion() {
  if (!geometryOK_) return;
  
  TriMesh::ConstFaceIter   f_i, f_End=faces_end();
  TriMesh::ConstVertexIter v_i, v_j, v_End=vertices_end(); 
  TriMesh::ConstVertexFaceIter f_ij;

  OpenMesh::FPropHandleT<float> faceAreas;
  add_property(faceAreas);
  //----------------------
  for (f_i=faces_begin(); f_i!=f_End; ++f_i) property(faceAreas, f_i)=faceArea(f_i);

  OpenMesh::VPropHandleT<Mesh_AmbientOcclusion_FormFactor*> formFactors;
  add_property(formFactors);
  //------------------------
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) {
    Mesh_AmbientOcclusion_FormFactor* factor = new Mesh_AmbientOcclusion_FormFactor();
    factor->setCenter(point(v_i));
    factor->setOrientation(calc_vertex_normal(v_i));

    float area=0.;
    for (f_ij=cvf_iter(v_i); f_ij; ++f_ij) area+=property(faceAreas, f_ij);
    factor->setArea(area);

    property(formFactors,v_i)=factor;
  }
  remove_property(faceAreas);

  //------------------------
  add_property(occlusions_);
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i) {
    Mesh_AmbientOcclusion_FormFactor* factor1 = property(formFactors, v_i);
    property(occlusions_,v_i)=0.;

    for (v_j=vertices_begin(); v_j!=v_End; ++v_j) {
      if (v_i==v_j) continue;

      Mesh_AmbientOcclusion_FormFactor* factor2 = property(formFactors, v_j);
      if (dot(factor1->orientation(),factor2->orientation())<=0) continue;

      OpenMesh::Vec3f D = factor2->center()-factor1->center();
      float d = D.norm();
      D.normalize();
      if (d>100) continue;

      float cosR = dot(D,factor1->orientation());
      float cosE = dot(D,factor2->orientation());
      property(occlusions_,v_i)+= factor2->area()*cosR*cosE/(factor2->area()+M_PI*d*d);
    }
  }
  remove_property(formFactors);
}



const float Mesh::sqrNorm(const OpenMesh::Vec3f& v) const {
  return sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
}

const float Mesh::faceArea(const OpenMesh::FaceHandle& face) const {
  TriMesh::Point  A,B,C;
  OpenMesh::Vec3f AB,AC;
  A = property(vertices_, from_vertex_handle                   (halfedge_handle(face)));
  B = property(vertices_, to_vertex_handle                     (halfedge_handle(face))); 
  C = property(vertices_, to_vertex_handle(next_halfedge_handle(halfedge_handle(face))));
  
  AB = B-A;
  AC = C-A;
  return sqrNorm(cross(AB,AC))/2.;
}

const float Mesh::angle(const OpenMesh::Vec3f& v1, const OpenMesh::Vec3f& v2) const {
  if (v1.norm()==0. || v2.norm()==0.) {
    cerr<<"Mesh::angle: At least one of the vectors has a null norm!"<<endl;
    return 0.;
  }
  float ang = acos(dot(v1,v2)/(v1.norm()*v2.norm()));
  if (isnan(ang)) return 0.;
  else            return ang;
}


//-------------------------------------------------------------------------------------
// INVERSION MATRIX 4x4
//---------------------
double Mesh::M3x3_determinant(double matrix3x3[9]) const {
  return (matrix3x3[0]*(matrix3x3[4]*matrix3x3[8] - matrix3x3[7]*matrix3x3[5])
	  - matrix3x3[1]*( matrix3x3[3]*matrix3x3[8] - matrix3x3[6]*matrix3x3[5])
	  + matrix3x3[2]*( matrix3x3[3]*matrix3x3[7] - matrix3x3[6]*matrix3x3[4]));
}

//---------------------
void Mesh::M4x4_subMatrix(double matrix4x4[16],double matrix3x3[9],
			  int i, int j) const {
  int di, dj, si, sj;
  for( di=0; di<3; di++ ) {
    for( dj=0; dj<3; dj++ ) {
      si=di+((di>=i)? 1:0 );
      sj=dj+((dj>=j)? 1:0 );
      matrix3x3[3*di+dj]=matrix4x4[4*si+sj];
    }
  }
}

//---------------------
double Mesh::M4x4_determinant(double matrix4x4[16]) const {

  double i = 1.;
  double matrix3x3[9];
  double det;

  double result = 0.;
  for (int n=0; n<4; n++){
    M4x4_subMatrix(matrix4x4,matrix3x3,0,n);
    det    = M3x3_determinant(matrix3x3);
    result += matrix4x4[n]*det*i;
    i *= -1.;
  }
  return result;
}

//---------------------
void Mesh::M4x4_inverse(double matrix4x4[16],double inverse4x4[16]) const {
  double mdet = M4x4_determinant( matrix4x4 );
  if ( fabs(mdet) <= FLT_EPSILON ) {
    for (int i=15; i>=0; i-- ) 
      inverse4x4[i] = 0.;
    inverse4x4[0] = inverse4x4[5] = inverse4x4[10] = inverse4x4[15] = 1.;
    return;
  }
  int sign;
  double temp3x3[9];
  for (int i=0; i<4; i++)
    for (int j=0; j<4; j++) {
      sign = 1-((i+j)%2)*2;
      M4x4_subMatrix(matrix4x4,temp3x3,i,j);
      inverse4x4[4*j+i] = ( M3x3_determinant(temp3x3)*sign )/mdet;
    }
}

//--------------------------------------------------------------------------------------
// EYE POSITION FROM MODELVIEW_MATRIX
//-----------------------------------
OpenMesh::Vec3f Mesh::getEyePosition() const { 
  double modelview[16];
  double modelview_inv[16];
  glGetDoublev(GL_MODELVIEW_MATRIX,modelview);
  cout<<"modelView_matrix = ["<<endl;
  cout<<modelview [0]<<" "<<modelview [1]<<" "<<modelview [2]<<" "<<modelview [3]<<endl;
  cout<<modelview [4]<<" "<<modelview [5]<<" "<<modelview [6]<<" "<<modelview [7]<<endl;
  cout<<modelview [8]<<" "<<modelview [9]<<" "<<modelview[10]<<" "<<modelview[11]<<endl;
  cout<<modelview[12]<<" "<<modelview[13]<<" "<<modelview[14]<<" "<<modelview[15]<<"]"<<endl;
  M4x4_inverse(modelview,modelview_inv);
  return OpenMesh::Vec3f((float)modelview_inv[12],(float)modelview_inv[13],(float)modelview_inv[14]);
}









// MISCELLANEOUS
void Mesh_AmbientOcclusion_FormFactor::draw() const{
  if (radius_<=0.) return;

  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);
  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);

  OpenMesh::Vec3f c = center_;
  OpenMesh::Vec3f z = orientation_;
  OpenMesh::Vec3f x(orientation_[1],-orientation_[0],0.); x.normalize();
  OpenMesh::Vec3f y = cross(z,x); y.normalize();
  
  float matrix[16];
  matrix [0]=x[0]; matrix [1]=x[1]; matrix [2]=x[2]; matrix [3]=0.;
  matrix [4]=y[0]; matrix [5]=y[1]; matrix [6]=y[2]; matrix [7]=0.;
  matrix [8]=z[0]; matrix [9]=z[1]; matrix[10]=z[2]; matrix[11]=0.;
  matrix[12]=c[0]; matrix[13]=c[1]; matrix[14]=c[2]; matrix[15]=1.;

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glMultMatrixf(matrix);

  GLUquadric* q = gluNewQuadric();
  gluDisk(q,0.,radius_,24,1);

  glPopMatrix();
  glPopAttrib();
}

//--------------------------------
// OCTREE
//-------
const bool Mesh::buildOctree(const unsigned int& size_min,
			     const unsigned int& depth_max) {
  if (!geometryOK_) return false;
  
  Octree_Point<VertexHandle>* points = new Octree_Point<VertexHandle>[n_vertices()];
  TriMesh::ConstVertexIter v_i, v_End = vertices_end();
  int i=0;
  for (v_i=vertices_begin(); v_i!=v_End; ++v_i,i++) {
    points[i].setXYZ(point(v_i)[0],point(v_i)[1],point(v_i)[2]);
    points[i].setData(v_i);
  }
  octree_ = new Octree<VertexHandle>();
  octreeOK_ = octree_->buildTree(points,
				    n_vertices(),
				    size_min,
				    depth_max);
  return octreeOK_;
  
}
