
#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>

using namespace std;


////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint	filter = GL_NEAREST;
GLuint	textureCube, textureSphere, textureCylinder, textureCone;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;


int loadImage(char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;		
    if ((file = fopen(filename, "rb"))==NULL) {
				cout << "File Not Found : " << filename << endl;
				return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
				cout << "Error reading width from " << filename << endl;
				return 0;
    }    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
				cout << "Error reading height from " << filename << endl;
				return 0;
    }    
		unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
				cout << "Error reading planes from " << filename << endl;
				return 0;
    }
    if (planes != 1) {
				cout << "Planes from " << filename << "is not 1: " << planes << endl;
				return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) { 
				cout << "Error reading bpp from " << filename << endl;
				return 0;
    }
    if (bpp != 24) {
				cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
				return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
				cout << "Error allocating memory for color-corrected image data" << endl;
				return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
				cout << "Error reading image data from " << filename << endl;
				return 0;
    }

		// reverse all of the colors. (bgr -> rgb) 
		char temp;                         
		for (i=0;i<size;i+=3) { 
				temp = image->data[i];
				image->data[i] = image->data[i+2];
				image->data[i+2] = temp;
    }
		
    return 1;
}


GLvoid loadGLTexture(char* textureName, GLuint* textureIdx) {	
    // Load Texture
    Image *image1;
    
    // allocate space for cube texture
    image1 = (Image *) malloc(sizeof(Image));
    if (image1 == NULL) {
				printf("Error allocating space for image");
				exit(0);
    }
    if (!loadImage(textureName, image1)) {
				exit(1);
    }        

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT); 
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image1->sizeX, image1->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image1->data);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, image1->sizeX, image1->sizeY, GL_RGB, GL_UNSIGNED_BYTE, image1->data); 

}

////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 0.2f, 0.2f, 0.2f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;

void setLight(){

                GLfloat lightPosition[]= { 4.0*cos(lightAngle), 3.0f, 4.0*sin(lightAngle), 1.0f };
                glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
                glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
                glLightfv(GL_LIGHT0, GL_POSITION,lightPosition);
                glEnable(GL_LIGHT0);
}




/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 3.0f;

void setCamera() {

                float x = distToOrigin*cos(cameraAngle);
                float y = cameraHeight;
                float z = distToOrigin*sin(cameraAngle);

                glLoadIdentity();
                gluLookAt(x, y, z,
                                                        -x, y, -z,
                                                        0.0f,1.0f,0.0f);
                setLight();
}




/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0, deltaT = 0.005;
float animRotation = 0.0, animRotationSign = 1.0;

void updateAnimationParam(){
		// compute a time parameter based on a 
		float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
		float t = (newTime-oldTime)/deltaT;
		oldTime = newTime;

		// update a rotation parameter (in degrees)
		animRotation += t*animRotationSign;
		if (animRotation>70.0f){
				animRotation = 70.0f;
				animRotationSign = -animRotationSign;
		}
		if (animRotation<-25.0f){
				animRotation = -25.0f;
				animRotationSign = -animRotationSign;
		}
}




////////////////////////////////////////
// Initialization methods
////////////////////////////////////////



void initScene() {
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);				// This Will Clear The Background Color To Black
		glClearDepth(1.0);													// Enables Clearing Of The Depth Buffer
		glDepthFunc(GL_LESS);												// The Type Of Depth Test To Do
		glEnable(GL_DEPTH_TEST);										// Enables Depth Testing
		glShadeModel(GL_SMOOTH);										// Enables Smooth Color Shading
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	// Controls the rendering style (fill, line or point)
		glLineWidth(2.0f);													
		glPointSize(4.0f);

		// init lighting
		setLight();
		glEnable(GL_LIGHTING);

		// init texturing
		loadGLTexture("textureCube.bmp", &textureCube);
		loadGLTexture("textureSphere.bmp", &textureSphere);
		loadGLTexture("textureCylinder.bmp", &textureCylinder);
		loadGLTexture("textureCone.bmp", &textureCone);
    glEnable(GL_TEXTURE_2D);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE); 
}




void changeSize(int w, int h)   {

                // Prevent a divide by zero, when window is too short
                // (you cant make a window of zero width).
                if(h == 0)
                                h = 1;

                float ratio = 1.0f * w / h;
                // Reset the coordinate system before modifying
                glMatrixMode(GL_PROJECTION);
                glLoadIdentity();

                // Set the viewport to be the entire window
                glViewport(0, 0, w, h);

                // Set the clipping volume
                gluPerspective(45,ratio,1,1000);
                glMatrixMode(GL_MODELVIEW);
                setCamera();
}










///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////

void drawGround(){

                float backColor[] = {1.0, 0.0, 0.0};
                float frontColor[] = {0.0, 0.0, 1.0};

                glBegin(GL_QUADS);

        glColor3fv(frontColor);
                glVertex3f(-10.0f, 0.0f, -10.0f);
        glColor3fv(backColor);
                glVertex3f(-10.0f, 0.0f,  10.0f);
        glColor3fv(backColor);
                glVertex3f( 10.0f, 0.0f,  10.0f);
        glColor3fv(frontColor);
                glVertex3f( 10.0f, 0.0f, -10.0f);

                glEnd();
}


void drawCube(){
		
    glBindTexture(GL_TEXTURE_2D, textureCube);
		
		glBegin(GL_QUADS);

		// Face 1
		glNormal3f(0,0,1);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-1.0,-1.0,1.0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(2.0, 0.0);
		glVertex3f(1.0,-1.0,1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(2.0, 2.0);
		glVertex3f(1.0,1.0,1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 2.0);
		glVertex3f(-1.0,1.0,1.0);


		// Face 2	 
		glNormal3f(0,0,-1);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-1.0,1.0,-1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(2.0, 0.0);
		glVertex3f(1.0,1.0,-1.0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(2.0, 2.0);
		glVertex3f(1.0,-1.0,-1.0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 2.0);
		glVertex3f(-1.0,-1.0,-1.0);
	 
		// Face 3	 
		glNormal3f(-1,0,0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-1.0,-1.0,1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(2.0, 0.0);
		glVertex3f(-1.0,1.0,1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(2.0, 2.0);
		glVertex3f(-1.0,1.0,-1.0);
	 
		glColor3f(0.0, 0.0, 1.0);	 
		glTexCoord2f(0.0, 2.0);
		glVertex3f(-1.0,-1.0,-1.0);
   
		// Face 4	 
		glNormal3f(1,0,0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(1.0,-1.0,-1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(2.0, 0.0);
		glVertex3f(1.0,1.0,-1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(2.0, 2.0);
		glVertex3f(1.0,1.0,1.0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 2.0);
		glVertex3f(1.0,-1.0,1.0);
	 
		// Face 5	 
		glNormal3f(0,-1,0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-1.0,-1.0,-1.0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(2.0, 0.0);
		glVertex3f(1.0,-1.0,-1.0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(2.0, 2.0);
		glVertex3f(1.0,-1.0,1.0);
	 
		glColor3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 2.0);
		glVertex3f(-1.0,-1.0,1.0);

		// Face 6
		glNormal3f(0,1,0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-1.0,1.0,1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(2.0, 0.0);
		glVertex3f(1.0,1.0,1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(2.0, 2.0);
		glVertex3f(1.0,1.0,-1.0);
	 
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 2.0);
		glVertex3f(-1.0,1.0,-1.0);
	 
		glEnd();
}



void drawCone(){
		glBindTexture(GL_TEXTURE_2D, textureCone);
		
		int nbFaces = 50;
		float stepAngle = 2*M_PI/nbFaces;

 		// draw base 
		glBegin(GL_POLYGON);
		glColor3f(1.0, 0.0, 0.0);
		glNormal3f(0.0, -1.0, 0.0);
		for (int i=0 ; i<nbFaces ; i++){
				float angle = i*stepAngle;
				glTexCoord2f(0.0, 0.0);
				glVertex3f(cos(angle), 0.0, sin(angle));
		}
		glEnd();

		
		// draw apex
		float norm = 0.5*sqrt(2.0);
		glBegin(GL_TRIANGLE_FAN);
		glNormal3f(0.0, 1.0, 0.0);
		glColor3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(0.0, 1.0, 0.0);
		for (int i=0 ; i<=nbFaces ; i++){
				float angle = i*stepAngle;
				
				glNormal3f(norm*cos(angle), norm, norm*sin(angle));
				glColor3f(0.0, 0.0, 1.0);
				glTexCoord2f(angle/(2.0*M_PI), 0.0);
				glVertex3f(cos(angle), 0.0, sin(angle));
		}
		glEnd();
		
}

void drawCylinder(){
		glBindTexture(GL_TEXTURE_2D, textureCylinder);
		
		int nbStrips = 20;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		// draw top 
		glBegin(GL_POLYGON);
		glColor3f(1.0, 0.0, 0.0);
		glNormal3f(0.0, 1.0, 0.0);
		for (int i=0 ; i<nbFacesPerStrip ; i++){
				float angle = i*stepHAngle;
				glTexCoord2f(0.0, 0.0);
				glVertex3f(cos(angle), 1.0, sin(angle));
		}
		glEnd();
		
		// draw base
		glBegin(GL_POLYGON);
		glColor3f(0.0, 0.0, 1.0);
		glNormal3f(0.0, -1.0, 0.0);
		for (int i=0 ; i<nbFacesPerStrip ; i++){
				float angle = i*stepHAngle;
				glTexCoord2f(0.0, 0.0);
				glVertex3f(cos(angle), -1.0, sin(angle));
		}
		glEnd();
		
		// draw cylinder
		for(int i=0 ; i<nbStrips ; i++) {
				float y1 = 2.0*i/nbStrips - 1.0;
				float y2 = 2.0*(i+1.0)/nbStrips - 1.0;
				
				// draw one strip
				float intensity1 = (float)i/nbStrips;
				float intensity2 = (float)(i+1)/nbStrips;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;						
						glNormal3f(cos(hAngle), 0.0, sin(hAngle));
						
						glColor3f(intensity1, 0.0, 1.0-intensity1);
						glTexCoord2f(hAngle/(2.0*M_PI), i/(float)nbStrips);
						glVertex3f(cos(hAngle), y1, sin(hAngle));
						
						glColor3f(intensity2, 0.0, 1.0-intensity2);
						glTexCoord2f(hAngle/(2.0*M_PI), (i+1.0)/(float)nbStrips);
						glVertex3f(cos(hAngle), y2, sin(hAngle));
				}
				glEnd();
		}

}

void drawSphere(){
		glBindTexture(GL_TEXTURE_2D, textureSphere);
		
		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;
				
				// draw one strip
				float intensity1 = (float)i/nbStrips;
				float intensity2 = (float)(i+1)/nbStrips;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;						
						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						
						glNormal3fv(pos1);
						glColor3f(intensity1, 0.0, 1.0-intensity1);
						glTexCoord2f(hAngle/(2.0*M_PI), i/(float)nbStrips);
						glVertex3fv(pos1);
						
						glNormal3fv(pos2);
						glColor3f(intensity2, 0.0, 1.0-intensity2);
						glTexCoord2f(hAngle/(2.0*M_PI), (i+1.0)/(float)nbStrips);
						glVertex3fv(pos2);
				}
				glEnd();
		}

}


void drawRobot() {

                glPushMatrix();
                // TODO
                glPopMatrix();
}


void drawPrimitives(){
                glPushMatrix();

                glTranslatef(-4.0f ,1.0f, 0.0f);
                drawCube();
                glTranslatef(2.25f ,0.0f, 0.0f);
                drawCone();
                glTranslatef(2.25f ,0.0f, 0.0f);
                drawSphere();
                glTranslatef(2.25f ,0.0f, 0.0f);
                drawCylinder();

                glPopMatrix();
}


void renderScene(void) {
                // clear buffers
                glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

                // draw objects
                drawGround();
                drawPrimitives();

								// updateAnimationParameters
								updateAnimationParameters();
								
                // display rendering
                glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////


void specialKey(int key, int x_, int y_) {

                switch (key) {
                case GLUT_KEY_LEFT :
                                cameraAngle += 0.1f;
                                setCamera();  break;
                case GLUT_KEY_RIGHT :
                                cameraAngle -=0.1f;
                                setCamera();  break;
                case GLUT_KEY_UP :
                                distToOrigin -= 0.2f;
                                if (distToOrigin<1.5f) distToOrigin = 1.5f;
                                setCamera();  break;
                case GLUT_KEY_DOWN :
                                distToOrigin += 0.2f;
                                setCamera();  break;
                case GLUT_KEY_PAGE_UP :
                                cameraHeight += 0.1f;
                                setCamera();  break;
                case GLUT_KEY_PAGE_DOWN :
                                cameraHeight -= 0.1f;
                                setCamera();  break;
                case GLUT_KEY_F1 :
                                lightAngle += 0.1f;
                                setLight(); break;
                case GLUT_KEY_F2 :
                                lightAngle -= 0.1f;
                                setLight(); break;
                }
}




void commonKey(unsigned char key, int x_, int y_) {

                switch (key) {
                default:
                                cout << key << endl; break;
                }
}






////////////////////////////////////
//  Main method
////////////////////////////////////

int main(int argc, char **argv) {

                // create window
                glutInit(&argc, argv);
                glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
                glutInitWindowPosition(100,100);
                glutInitWindowSize(800, 600);
                glutCreateWindow("Humanoid");

                // initialize OpenGL states
                initScene();

                // Bind Key events
                glutSpecialFunc(specialKey);
                glutKeyboardFunc(commonKey);

                // Bind rendering methods
                glutDisplayFunc(renderScene);
                glutIdleFunc(renderScene);

                glutReshapeFunc(changeSize);

                // run program
                glutMainLoop();

                return(0);
}
