#include <math.h>
#ifdef __APPLE__
#include <glut.h>
#else
#include <GL/glut.h>
#endif
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <float.h>
#include <string.h>
#include "Vec3.h"

#define CLIP(a) if(a<0.0)a=0.0; if(a>1.0)a=1.0;

using namespace std;

////////////////////////////////
// Object mesh methods
////////////////////////////////

float znear=1.0, zfar=100.0;
vector<Vec3> vertices, normals;
vector< pair<int, int> > faceIdx[3];
bool objectLoaded = false;


bool loadObj(const char * filename, bool normalize) {

  ifstream in(filename);
    char type;
    char s[256];

  if (!in.is_open()) {
            cout << "Using default object: unit sphere"  << endl;
    return false;
    }

    vertices.clear();
    normals.clear();
    faceIdx[0].clear();
    faceIdx[1].clear();
    faceIdx[2].clear();

    while (!in.eof()) {
        if (in.peek() == 'v') {
            in >> type;

            if (in.peek() == 'n'){
                in >> type;

                float x, y, z;
                in>> x; in>> y; in>> z;
                normals.push_back(Vec3(x,y,z));

            }   else if (in.peek() != 't') {
                float x, y, z;
                in>> x; in>> y; in>> z;
                vertices.push_back(Vec3(x,y,z));
            }
            in.getline(s, 256);

        }   else if (in.peek() == 'f') {
            in >> type;

            int vertex1, vertex2, vertex3;
            int normal1, normal2, normal3;
            char tempByte;

            // vertex 1
            in>> vertex1; --vertex1;
            // separator
            in >> tempByte; in >> tempByte;
            // normal 1
            in>> normal1; --normal1;

            // vertex 2
            in>> vertex2; --vertex2;
            // separator
            in >> tempByte; in >> tempByte;
            // normal 2
            in>> normal2; --normal2;

            // vertex 3
            in>> vertex3; --vertex3;
            // separator
            in >> tempByte; in >> tempByte;
            // normal 3
            in>> normal3; --normal3;

            // Store face
            pair<int, int> idx1 (vertex1, normal1);
            pair<int, int> idx2 (vertex2, normal2);
            pair<int, int> idx3 (vertex3, normal3);

            faceIdx[0].push_back(idx1);
            faceIdx[1].push_back(idx2);
            faceIdx[2].push_back(idx3);

            in.getline(s, 256);
        } else {
            in.getline(s, 256);
        }
    }
    in.close();


    unsigned int nbVertices = vertices.size();
    cout << "Object loaded: " << endl;
    cout << "\t Filename = " << filename << endl;
    cout << "\t #vertices = " << nbVertices << endl;

    // normalize object scale.
    if (normalize) {
        float minX=FLT_MAX;
        float maxX=-FLT_MAX;
        float minY=FLT_MAX;;
        float maxY=-FLT_MAX;
        float minZ=FLT_MAX;
        float maxZ=-FLT_MAX;

        // find bounding box coords
        for (unsigned int i=0 ; i<nbVertices ; i++){
            minX=(minX>vertices[i].x)?vertices[i].x:minX;
            maxX=(maxX<vertices[i].x)?vertices[i].x:maxX;
            minY=(minY>vertices[i].y)?vertices[i].y:minY;
            maxY=(maxY<vertices[i].y)?vertices[i].y:maxY;
            minZ=(minZ>vertices[i].z)?vertices[i].z:minZ;
            maxZ=(maxZ<vertices[i].z)?vertices[i].z:maxZ;
        }

        // modify object vertices so that the maximum distance is unit size...
        Vec3 center (0.5*(maxX+minX), 0.5*(maxY+minY), 0.5*(maxZ+minZ));
        float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
      float scaling=1.0/norm;

      for (unsigned int i=0 ; i<nbVertices ; i++) {
        vertices[i].x =(vertices[i].x - center.x)*scaling;
        vertices[i].y =(vertices[i].y - center.y)*scaling;
        vertices[i].z =(vertices[i].z - center.z)*scaling;
      }
    }


    return true;
}



////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint  texture;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;

static short LE2( short val )
{
#if defined(__APPLE__) && defined(__BIG_ENDIAN__)
    return ((val&0xFF)<<8)|((val&0xFF00)>>8);
#else
    return val;
#endif
}

static int LE4( int val )
{
#if defined(__APPLE__) && defined(__BIG_ENDIAN__)
    return ((val&0xFF)<<24)|((val&0xFF00)<<8)|((val&0xFF0000)>>8)|((val&0xFF000000)>>24);
#else
    return val;
#endif
}

int loadImage(const char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;
    if ((file = fopen(filename, "rb"))==NULL) {
                return 0;
    }

    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
                cout << "Error reading width from " << filename << endl;
                return 0;
    }
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
                cout << "Error reading height from " << filename << endl;
                return 0;
    }
    image->sizeX = LE4(image->sizeX);
    image->sizeY = LE4(image->sizeY);
    unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
                cout << "Error reading planes from " << filename << endl;
                return 0;
    }
    planes = LE2(planes);
    if (planes != 1) {
                cout << "Planes from " << filename << "is not 1: " << planes << endl;
                return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) {
                cout << "Error reading bpp from " << filename << endl;
                return 0;
    }
    bpp = LE2(bpp);
    if (bpp != 24) {
                cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
                return 0;
    }

    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data.
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
                cout << "Error allocating memory for color-corrected image data" << endl;
                return 0;
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
                cout << "Error reading image data from " << filename << endl;
                return 0;
    }

        // reverse all of the colors. (bgr -> rgb)
        char temp;
        for (i=0;i<size;i+=3) {
                temp = image->data[i];
                image->data[i] = image->data[i+2];
                image->data[i+2] = temp;
    }

    return 1;
}


GLvoid loadGLTexture(const char* textureName, GLuint* textureIdx) {
    // Load Texture
    Image *image;

    // allocate space for cube texture
    image = (Image *) malloc(sizeof(Image));
    if (image == NULL) {
                printf("Error allocating space for image");
                exit(0);
    }
    if (!loadImage(textureName, image)){
                if (!loadImage("defaultToonTex.bmp", image)) {
                        cerr << "Default texture fot found : defaultToonTex.bmp" << endl;
                        exit(1);
                } else {
                        cout << "Using default texture: defaultToonTex.bmp" << endl;
                }
    } else {
                cout << "Texture Loaded: " << endl;
                cout << "\t Filename: " << textureName << endl;
                cout << "\t Size: " << image->sizeX << endl;
        }

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
};


////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 0.2f, 0.2f, 0.2f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;
float lightRadius = 5.0;
float lightHeight = 1.0;
Vec3 light (lightRadius, lightRadius, 0.0);

void setLight()
{
    light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));
    GLfloat lightPosition[]= {light.x, light.y, light.z, 1.0f};
    glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
    glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
    glEnable(GL_LIGHT0);
}

/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 0.0f;
Vec3 camera(0.0, cameraHeight, distToOrigin);

void setCamera() {

        float x = distToOrigin*cos(cameraAngle);
        float y = cameraHeight;
        float z = distToOrigin*sin(cameraAngle);
        camera = Vec3(x, y, z);

        glLoadIdentity();
        gluLookAt(x, y, z,
                            -x, y, -z,
                            0.0f,1.0f,0.0f);
        setLight();
}

////////////////////////////////////////
//  Material methods
////////////////////////////////////////
int PHONG=1, PHONG_TOON=2, FULL_TOON=3, X_TOON=4;
int materialModel = PHONG;

#define PLASTIC 0
#define METAL 1
int material = PLASTIC;

#define NONE 0
#define DEPTH 1
#define ORIENT 2
#define REFLECT 3
int detailMap = NONE;

// plastic material parameters
GLfloat plasticDiffuse[] = { 0.1, 0.5, 0.8, 1.0 };
GLfloat plasticSpecular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat plasticShininess = 100.0;

// metallic material parameters
GLfloat metalDiffuse[] = { 0.6, 0.5, 0.4, 1.0 };
GLfloat metalSpecular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat metalShininess = 5.0;

// toon shading parameters
int intervalles = 5;

// X-Toon shading parameters
float xtoonDistance = 5.0;
float xtoonOrient = 1.0;
float xtoonShiny = 1.0;

void setMaterial()
{
    if( materialModel == PHONG || materialModel == PHONG_TOON )
    {
        glEnable( GL_LIGHTING );
        if( material == PLASTIC )
        {
            glMaterialfv( GL_FRONT, GL_DIFFUSE, plasticDiffuse );
            glMaterialfv( GL_FRONT, GL_SPECULAR, plasticSpecular );
            glMaterialf( GL_FRONT, GL_SHININESS, plasticShininess );
        }
        else if( material == METAL )
        {
            glMaterialfv( GL_FRONT, GL_DIFFUSE, metalDiffuse );
            glMaterialfv( GL_FRONT, GL_SPECULAR, metalSpecular );
            glMaterialf( GL_FRONT, GL_SHININESS, metalShininess );
        }
    }
    if( materialModel == PHONG_TOON )
    {
        glEnable( GL_COLOR_MATERIAL );
        glColorMaterial( GL_FRONT, GL_DIFFUSE );
    }
    if( materialModel == X_TOON )
    {
        glEnable( GL_TEXTURE_2D );
    }
}

void unsetMaterial()
{
    glDisable( GL_COLOR_MATERIAL );
    glDisable( GL_LIGHTING );
    glDisable( GL_TEXTURE_2D );
}

float toon( float val )
{
    for( int i = 0; i < intervalles; i++ )
    {
        if( val < (float) ( i + 1 ) / intervalles )
        {
            return (float) ( 2 * i + 1 ) / intervalles / 2;
        }
    }

    return 1.0;
}

void getColorNoToon( const Vec3& vertex, const Vec3& normal, float& spec, float &diff )
{
    float shininess;
    if( material == PLASTIC )
        shininess = plasticShininess;
    else if( material == METAL )
        shininess = metalShininess;
    else
        return;

    Vec3 lumiere = light - vertex;
    diff = normal.normalized() | lumiere.normalized();

    spec = 0.0;
    if( materialModel != PHONG_TOON )
    {
        Vec3 vue = camera - vertex;
        Vec3 half = lumiere.normalized() + vue.normalized();
        spec = normal.normalized() | half.normalized();
        spec = pow( spec, shininess );
    }
}

void getColor(const Vec3& vertex, const Vec3& normal, Vec3& color)
{
    float * diffmat;
    if( material == PLASTIC )
        diffmat = plasticDiffuse;
    else if( material == METAL )
        diffmat = metalDiffuse;
    else
        return;

    float spec, diff;
    getColorNoToon( vertex, normal, spec, diff );

    diff = toon( diff );
    if( spec < 0.5 ) spec = 0.0; else spec = 1.0;

    color.x = diff * diffmat[0] + spec; CLIP(color.x);
    color.y = diff * diffmat[1] + spec; CLIP(color.y);
    color.z = diff * diffmat[2] + spec; CLIP(color.z);
}

void getTexCoord(const Vec3& vertex, const Vec3& normal, float& toneCoord, float& detailCoord)
{
    Vec3 color;
    float diff, spec;
    getColorNoToon( vertex, normal, spec, diff );
    toneCoord = diff;

    switch( detailMap )
    {
        case DEPTH:
        {
            detailCoord = dist( camera, vertex ) - xtoonDistance; CLIP( detailCoord );
            break;
        }
        case ORIENT:
        {
            Vec3 vue = camera - vertex;
            detailCoord = fabsf( ( normal | vue ) / normal.norm() / vue.norm() );
            detailCoord = pow( detailCoord, xtoonOrient );
            break;
        }
        case REFLECT:
        {
            CLIP( spec );
            detailCoord = spec;
            break;
        }
        default:
            detailCoord = 1.0;
    }
}

/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
float rotationSpeed = 1.0;
bool lightRotation = true;

void updateAnimationParam()
{
    // compute a time parameter based on a
    float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
    float t = newTime-oldTime;
    oldTime = newTime;

    if( lightRotation )
    {
        lightAngle += t * rotationSpeed;
        setLight();
    }
}

////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene(const char* objFile, const char* toonTexFile) {
        // init scanline
        glClearColor(0.3, 0.3, 0.3, 1.0f);
        glClearDepth(1.0);
        glDepthFunc(GL_LESS);
        glEnable(GL_DEPTH_TEST);
        glShadeModel(GL_SMOOTH);
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
        glLineWidth(2.0f);
        glPointSize(4.0f);

        // init lighting
        setLight();
        glEnable(GL_LIGHTING);

        // init textures
        loadGLTexture(toonTexFile, &texture);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

        // init object
        glEnable(GL_NORMALIZE);
        objectLoaded = loadObj(objFile, true);
}


void changeSize(int w, int h)   {

        // Prevent a divide by zero, when window is too short
        // (you cant make a window of zero width).
        if(h == 0)
                h = 1;

        float ratio = 1.0f * w / h;
        // Reset the coordinate system before modifying
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();

        // Set the viewport to be the entire window
        glViewport(0, 0, w, h);

        // Set the clipping volume
        gluPerspective(45,ratio,znear,zfar);
        glMatrixMode(GL_MODELVIEW);
        setCamera();
}










///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////


void drawSphere(){

        setMaterial();

        glPushMatrix();

        int nbStrips = 20;
        float stepVAngle = M_PI/nbStrips;

        int nbFacesPerStrip = 50;
        float stepHAngle = 2*M_PI/nbFacesPerStrip;

        for(int i=0 ; i<nbStrips ; i++) {
                float vAngle1 = i*stepVAngle - 0.5*M_PI;
                float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;

                // draw one strip
                Vec3 color1, color2;
                glBegin(GL_QUAD_STRIP);
                for (int j=0 ; j<=nbFacesPerStrip ; j++){
                        float hAngle = j*stepHAngle;

                        float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
                        glColor3fv(color1);
                        glNormal3fv(pos1);
                        glVertex3fv(pos1);

                        float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
                        glColor3fv(color2);
                        glNormal3fv(pos2);
                        glVertex3fv(pos2);
                }
                glEnd();
        }
        glPopMatrix();

        unsetMaterial();

}

void drawObject()
{
    setMaterial();

    glPushMatrix();
    glScalef( 10.0, 10.0, 10.0 );
    glBegin( GL_TRIANGLES );

    Vec3 color;
    float tone, detail;

    for( unsigned i = 0; i < faceIdx[0].size(); i++ )
    {
        /* Sommets */
        for( int j = 0; j < 3; j++ )
        {
            if( materialModel == X_TOON )
            {
                getTexCoord( vertices[faceIdx[j][i].first], normals[faceIdx[j][i].second], tone, detail );
                glTexCoord2f( tone, detail );
            }
            else
            {
                getColor( vertices[faceIdx[j][i].first], normals[faceIdx[j][i].second], color );
                glColor3fv( color );
            }
            glNormal3fv( normals[faceIdx[j][i].second] );
            glVertex3fv( vertices[faceIdx[j][i].first] );
        }
    }

    glEnd();
    glPopMatrix();

    unsetMaterial();
}


void renderScene(void) {
        // clear buffers
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        // draw objects
        if (objectLoaded){
                drawObject();
        } else {
                drawSphere();
        }

        // update animation parameters
        updateAnimationParam();

        // display rendering
        glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////


void specialKey(int key, int x_, int y_) {

        switch (key) {
        case GLUT_KEY_LEFT :
                cameraAngle += 0.04f;
                setCamera();  break;
        case GLUT_KEY_RIGHT :
                cameraAngle -=0.04f;
                setCamera();  break;
        case GLUT_KEY_UP :
                distToOrigin -= 0.5f;
                if (distToOrigin<1.5f) distToOrigin = 1.5f;
                setCamera();  break;
        case GLUT_KEY_DOWN :
                distToOrigin += 0.5f;
                setCamera();  break;
        case GLUT_KEY_PAGE_UP :
                cameraHeight += 0.1f;
                setCamera();  break;
        case GLUT_KEY_PAGE_DOWN :
                cameraHeight -= 0.1f;
                setCamera();  break;
        case GLUT_KEY_F1 :
                detailMap=NONE; break;
        case GLUT_KEY_F2 :
                detailMap=DEPTH; break;
        case GLUT_KEY_F3 :
                detailMap=ORIENT; break;
        case GLUT_KEY_F4 :
                detailMap=REFLECT; break;
        }
}

void commonKey(unsigned char key, int x_, int y_) {

        switch (key) {
        case 'r':
                lightRotation = true; break;
        case ',':
                lightRotation = false;
                lightAngle += 0.1f;
                setLight(); break;
        case '.':
                lightRotation = false;
                lightAngle -= 0.1f;
                setLight(); break;
        case '1':
                materialModel = PHONG; break;
        case '2':
                materialModel = PHONG_TOON; break;
        case '3':
                materialModel = FULL_TOON; break;
        case '4':
                materialModel = X_TOON; break;
        case 'm':
                material = METAL; break;
        case 'p':
                material = PLASTIC; break;
        case '+':
                if( materialModel == X_TOON )
                {
                    if( detailMap == DEPTH )
                    {
                        xtoonDistance += 1.0;
                    }
                    if( detailMap == ORIENT )
                    {
                        xtoonOrient += 0.1;
                    }
                }
                else
                {
                    intervalles++;
                }
                break;
        case '-':
                if( materialModel == X_TOON )
                {
                    if( detailMap == DEPTH )
                    {
                        if( xtoonDistance > 1.0 )
                            xtoonDistance -= 1.0;
                    }
                    if( detailMap == ORIENT )
                    {
                        if( xtoonDistance > 0.1 )
                            xtoonOrient -= 0.1;
                    }
                }
                else
                {
                    if( intervalles > 2 )
                        intervalles--;
                }
                break;
        case '/':
                rotationSpeed += 0.1; break;
        case '*':
                rotationSpeed -= 0.1; break;
        }
}

////////////////////////////////////
//  Main method
////////////////////////////////////

void parseArgs(int argc, char **argv, char** objFileName, char** toonTexFileName){

        // default values
        *objFileName = "-";
        *toonTexFileName = "-";

        if (argc>1){
                int nbCar1 = strlen(argv[1]);

                if( !strncmp( &argv[1][nbCar1-3], "obj", 3 ) )
                {
                        *objFileName = argv[1];
                }
                else if( !strncmp( &argv[1][nbCar1-3], "bmp", 3 ) )
                {
                        *toonTexFileName = argv[1];
                }

                if (argc>2){
                        int nbCar2 = strlen(argv[2]);

                        if( !strncmp( &argv[2][nbCar2-3], "obj", 3 ) )
                        {
                                *objFileName = argv[2];
                        }
                        else if( !strncmp( &argv[2][nbCar2-3], "bmp", 3 ) )
                        {
                                *toonTexFileName = argv[2];
                        }
                }
        }
}

int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
    glutInitWindowPosition(100,100);
    glutInitWindowSize(800, 600);
    glutCreateWindow("Humanoid");

    char *objFileName, *toonTexFileName;
    parseArgs(argc, argv, &objFileName, &toonTexFileName);
    initScene(objFileName, toonTexFileName);

    glutSpecialFunc(specialKey);
    glutKeyboardFunc(commonKey);

    glutDisplayFunc(renderScene);
    glutIdleFunc(renderScene);

    glutReshapeFunc(changeSize);

    glutMainLoop();

    return(0);
}
