//////////////////////
// M. VIDAL Vincent //
//////////////////////

#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <float.h>
#include <string.h>
#include "Vec3.h"

using namespace std;

////////////////////////////////
// Object mesh methods
////////////////////////////////

float znear=1.0, zfar=100.0;
vector<Vec3> vertices, normals; // vecteurs et normales
vector< pair<int, int> > faceIdx1, faceIdx2, faceIdx3; // les 3 sommets du triangle de la face
bool objectLoaded = false;


bool loadObj(const char * filename, bool normalize) {
	
  ifstream in(filename);
	char type;
	char s[256];
		
  if (!in.is_open()) {
			cout << "Using default object: unit sphere"  << endl;
    return false;
	}

	vertices.clear();
	normals.clear();
	faceIdx1.clear();
	faceIdx2.clear();
	faceIdx3.clear();
	
	while (!in.eof()) {
		if (in.peek() == 'v') {
			in >> type;
			
			if (in.peek() == 'n'){
				in >> type;
			
				float x, y, z;
				in>> x; in>> y; in>> z;
				normals.push_back(Vec3(x,y,z));
				
			}	else if (in.peek() != 't') {
				float x, y, z;
				in>> x; in>> y; in>> z;
				vertices.push_back(Vec3(x,y,z));				
			}
			in.getline(s, 256);
			
		}	else if (in.peek() == 'f') {
			in >> type;
			
			int vertex1, vertex2, vertex3;
			int normal1, normal2, normal3;
			char tempByte;
			
			// vertex 1
			in>> vertex1; --vertex1;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 1
			in>> normal1; --normal1;
			
			// vertex 2
			in>> vertex2; --vertex2;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 2
			in>> normal2; --normal2;
			
			// vertex 3
			in>> vertex3; --vertex3;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 3
			in>> normal3; --normal3;
			
			// Store face
			pair<int, int> idx1 (vertex1, normal1);
			pair<int, int> idx2 (vertex2, normal2);
			pair<int, int> idx3 (vertex3, normal3);

			faceIdx1.push_back(idx1);
			faceIdx2.push_back(idx2);
			faceIdx3.push_back(idx3);
			
			in.getline(s, 256);
		} else {
			in.getline(s, 256);
		}
	}	
	in.close();

	
	unsigned int nbVertices = vertices.size();	
	cout << "Object loaded: " << endl;
	cout << "\t Filename = " << filename << endl;
	cout << "\t #vertices = " << nbVertices << endl;

	// normalize object scale.
	if (normalize) {
		float minX=FLT_MAX;
		float maxX=-FLT_MAX;
		float minY=FLT_MAX;;
		float maxY=-FLT_MAX;
		float minZ=FLT_MAX;
		float maxZ=-FLT_MAX;

		// find bounding box coords
		for (unsigned int i=0 ; i<nbVertices ; i++){
			minX=(minX>vertices[i].x)?vertices[i].x:minX;				
			maxX=(maxX<vertices[i].x)?vertices[i].x:maxX;
			minY=(minY>vertices[i].y)?vertices[i].y:minY;				
			maxY=(maxY<vertices[i].y)?vertices[i].y:maxY;
			minZ=(minZ>vertices[i].z)?vertices[i].z:minZ;				
			maxZ=(maxZ<vertices[i].z)?vertices[i].z:maxZ;
		}

		// modify object vertices so that the maximum distance is unit size...
		Vec3 center (0.5*(maxX+minX), 0.5*(maxY+minY), 0.5*(maxZ+minZ));
		float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
	  float scaling=1.0/norm;
	
	  for (unsigned int i=0 ; i<nbVertices ; i++) {		  
	    vertices[i].x =(vertices[i].x - center.x)*scaling;
	    vertices[i].y =(vertices[i].y - center.y)*scaling;
	    vertices[i].z =(vertices[i].z - center.z)*scaling;
	  }
	}


	return true;			
}





////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint	texture;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;


int loadImage(const char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;		
    if ((file = fopen(filename, "rb"))==NULL) {
				return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
				cout << "Error reading width from " << filename << endl;
				return 0;
    }    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
				cout << "Error reading height from " << filename << endl;
				return 0;
    }    
		unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
				cout << "Error reading planes from " << filename << endl;
				return 0;
    }
    if (planes != 1) {
				cout << "Planes from " << filename << "is not 1: " << planes << endl;
				return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) { 
				cout << "Error reading bpp from " << filename << endl;
				return 0;
    }
    if (bpp != 24) {
				cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
				return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
				cout << "Error allocating memory for color-corrected image data" << endl;
				return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
				cout << "Error reading image data from " << filename << endl;
				return 0;
    }

		// reverse all of the colors. (bgr -> rgb) 
		char temp;                         
		for (i=0;i<size;i+=3) { 
				temp = image->data[i];
				image->data[i] = image->data[i+2];
				image->data[i+2] = temp;
    }
		
    return 1;
}


GLvoid loadGLTexture(const char* textureName, GLuint* textureIdx) {	
    // Load Texture
    Image *image;
    
    // allocate space for cube texture
    image = (Image *) malloc(sizeof(Image));
    if (image == NULL) {
				printf("Error allocating space for image");
				exit(0);
    }
    if (!loadImage(textureName, image)){
				if (!loadImage("defaultToonTex.bmp", image)) {
						cerr << "Default texture fot found : defaultToonTex.bmp" << endl;
						exit(1);
				} else {
						cout << "Using default texture: defaultToonTex.bmp" << endl;
				}
    } else {
				cout << "Texture Loaded: " << endl;
				cout << "\t Filename: " << textureName << endl;
				cout << "\t Size: " << image->sizeX << endl;
		} 

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP); 
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
};






////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;
float lightRadius = 5.0;
float lightHeight = 1.0;
Vec3 light (lightRadius, lightRadius, 0.0);

void setLight(){
		light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));
		GLfloat lightPosition[]= {light.x, light.y, light.z, 1.0f};
		glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
		glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
		glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
		glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
		glEnable(GL_LIGHT0);
}





/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 0.0f;
Vec3 camera(0.0, cameraHeight, distToOrigin);

void setCamera() {

		float x = distToOrigin*cos(cameraAngle);
		float y = cameraHeight;
		float z = distToOrigin*sin(cameraAngle);
		camera = Vec3(x, y, z);
		
		glLoadIdentity();
		gluLookAt(x, y, z,
							-x, y, -z,
							0.0f,1.0f,0.0f);
		setLight();
}




////////////////////////////////////////
//  Material methods
////////////////////////////////////////
const int PHONG=1, PHONG_TOON=2, FULL_TOON=3, X_TOON=4;
int materialModel = PHONG;

int PLASTIC=0, METAL=1;
int material = PLASTIC;

const int NONE=0, DEPTH=1, ORIENT=2, REFLECT=3;
int detailMap = NONE;



// plastic material parameters 
float plastic_ambient [] = {0.0,0.1,0.06,1.0};
float plastic_diffuse [] = {0.0,0.50980392,0.50980392,1.0};
float plastic_specular [] = {0.94,0.94,0.94,1.0}; // couleur blanche
float plastic_shininess = 380; // grande shininess = petite tache => plastic



// metallic material parameters 
float metallic_ambient [] = {0.25,0.25,0.25,1.0};
float metallic_diffuse [] = {0.55,0.2,0.0,1.0};
float metallic_specular [] = {0.5,0.25,0.0,1.0}; // proche de la couleur diffuse
float metallic_shininess = 30;


// toon shading parameters

// nb d'intervalles pour segmenter la composante diffuse
unsigned int nb_intervalles = 1;
// on stoke la valeur de la fonction de diffusion
// sur chaque intervalle
vector <float> segmentation;
float ambient_null [] = {0.0,0.0,0.0,1.0};
float ambient_grey [] = {0.6,0.6,0.6,1.0};


// X-Toon shading parameters
int control_detail=0;
int flip = 0;

void MAJ_Segmentation()
{
	//printf("Entre dans MAJ_Segmentation\n");
	
	segmentation.clear(); // on supprime l'ancienne segmentation
	
	float cour=0,suiv;
	// creation de la nouvelle segmentation		
	
	for (unsigned int i = 1; i <= nb_intervalles; i++ )
	{
		suiv = i*1.0/nb_intervalles;
		
		segmentation.push_back( (suiv-cour)*0.5 + cour);
			
		//cout << "val " << segmentation.back() << endl;
		
		cour = suiv;
	}
	
	if (segmentation.size() != nb_intervalles) {
		printf("Erreur dans MAJ_Segmentation\n");
		exit(-1);
	}

}


// Si l'on souhaite controler la couleur spculaire, 
// il faut calculer et appliquer la composante spculaire 
// de l'clairage aprs le placage de la texture. Ceci 
// peut tre fait en utilisant la fonction 
// glLightModeli (GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR) 
// dans la fonction void afficheSphereTexOpenGL (...). 
// Le mode de calcul classique (par dfaut) de l'clairage est 
// ensuite restaur avec la fonction 
// glLightModeli (GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR).

void setMaterial(){		
		
		 if (material==PLASTIC)
		   {
			// modifiez la composante ambiante du matriau
			glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, plastic_ambient); 
		
			// modifiez la composante diffuse du matriau
			glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, plastic_diffuse);
		
			// modifiez la composante diffuse du matriau
			glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, plastic_specular);
                
			// modifiez la composante brillance du matriau
			glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, plastic_shininess);
		   } 
		   else // metal
		   {
			// modifiez la composante ambiante du matriau
			glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, metallic_ambient); 
		
			// modifiez la composante diffuse du matriau
			glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, metallic_diffuse);
		
			// modifiez la composante diffuse du matriau
			glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, metallic_specular);
                
			// modifiez la composante brillance du matriau
			glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, metallic_shininess);
				
		    }
		
		
		switch (materialModel) {
		
		case PHONG: break;
		
		case PHONG_TOON: 
		
			// pour manipuler la valeur d'une composante (diffuse ici)	
			glEnable(GL_COLOR_MATERIAL); 
		
			// il est conseill d'utiliser des coef ambiants nulls
		        glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, ambient_null); 
					
			// choix de la composante
			glColorMaterial( GL_FRONT_AND_BACK, GL_DIFFUSE);
			
			// calcul de la segmentation
			MAJ_Segmentation();
			
			
			break;
		
		case FULL_TOON: 
			
			glDisable(GL_LIGHTING);
			
			// il est conseill d'utiliser des coef ambiants nulls
		        glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, ambient_null);
			
			// calcul de la segmentation
			MAJ_Segmentation();
		
			break;
		
		case X_TOON: 
			glEnable(GL_TEXTURE_2D);
			
			
			//glLightModeli (GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
			
			break;		
		}
}


void unsetMaterial(){		
		
		switch (materialModel) {
		
		case PHONG: break;		
		
		case PHONG_TOON:
		
			glDisable(GL_COLOR_MATERIAL);
			
			break;
			
		case FULL_TOON: 
			
			glEnable(GL_LIGHTING);
						
			break;
		
		case X_TOON: 
			glDisable(GL_TEXTURE_2D);
			
			//glLightModeli (GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR);
			
			break;
		
		}
}




void getColor(const Vec3& vertex, const Vec3& normal, Vec3& color){		
	
	Vec3 LumiereDir = light - vertex;
		
	// on normalise la direction de la lumiere
	LumiereDir.normalize();	
	
	Vec3 NormSommet(normal);
	NormSommet.normalize();
	
	float diff;
	int num_inter;
	float valdiff;
	
	Vec3 V,H;
	float Spec;
	
	switch (materialModel) {
		
	case PHONG: break;		
		
	case PHONG_TOON:		
		
		// produit scalaire normale sommet et direction lumiere	
		// pour calculer la composante diffuse initiale		
		diff = dot(NormSommet, LumiereDir);    
		
		if (diff < 0) diff = 0;
		
		num_inter = (int) floor( diff * nb_intervalles );
		
		valdiff = segmentation[num_inter];
		
		//cout << valdiff << endl;
				
		//maj color
		if (material==PLASTIC) {
			color = Vec3(valdiff*plastic_diffuse[0],valdiff*plastic_diffuse[1],valdiff*plastic_diffuse[2]);
		}else {
			color = Vec3(valdiff*metallic_diffuse[0],valdiff*metallic_diffuse[1],valdiff*metallic_diffuse[2]);
		}
				
		break;
			
	case FULL_TOON: 
		
		V = camera - vertex;
		
		V.normalize();
				
		H = LumiereDir+V;
		H.normalize();
		
		// calcule de la composante spculaire
		Spec = dot(NormSommet, H);
								
		// calcul de la composante diffuse
		diff = dot(NormSommet, LumiereDir);
		
		if (diff < 0) diff = 0;
		num_inter = (int) floor( diff * nb_intervalles );
		
		valdiff = segmentation[num_inter];
		
		//maj color
		if (material==PLASTIC) {
			Spec = pow(Spec,plastic_shininess); // on peut changer la concentration du reflet
			
			if(Spec<0.5) Spec = 0.0;
			else Spec = 1.0;
			 
			color = Vec3(valdiff*plastic_diffuse[0],valdiff*plastic_diffuse[1],valdiff*plastic_diffuse[2]);
			
			color = color + Vec3(Spec*plastic_specular[0],Spec*plastic_specular[1],Spec*plastic_specular[2]);
			
		}else {
			Spec = pow(Spec,metallic_shininess); // on peut changer la concentration du reflet
			if(Spec<0.5) Spec = 0.0;
			else Spec = 1.0; 
			
			color = Vec3(valdiff*metallic_diffuse[0],valdiff*metallic_diffuse[1],valdiff*metallic_diffuse[2]);
			
			color = color + Vec3(Spec*metallic_specular[0],Spec*metallic_specular[1],Spec*metallic_specular[2]);
		}
		
		//cout << Spec << endl;
								
		break;
		
	case X_TOON: 
		break;
		
	}		
}


void getTexCoord(const Vec3& vertex, const Vec3& normal, float& toneCoord, float& detailCoord){
		
		
	Vec3 LumiereDir = light - vertex;
		
	// on normalise la direction de la lumiere
	LumiereDir.normalize();	
	
	Vec3 NormSommet(normal);
	NormSommet.normalize();
		
	// produit scalaire normale sommet et direction lumiere	
	// pour calculer la composante diffuse initiale		
	toneCoord = dot(NormSommet, LumiereDir);    
	if (toneCoord<0.0) toneCoord = 0.0;
	
	Vec3 VueDir = camera - vertex;
	Vec3 H;
	float dist, Spec,scal;
	
	switch(detailMap) {
	
	case NONE:
		detailCoord = 1;
		break;
	case DEPTH:
		dist = (camera - vertex).norm2();
		
		if(dist < znear) detailCoord = 1;
		else if (dist > znear + control_detail*(zfar-znear) ) detailCoord = 0;
		else detailCoord = 1-(dist-znear)/( control_detail*(zfar-znear) );
		
		//autres ides
		//detailCoord = 1.0*(control_detail+0.1)/(camera - vertex).norm2();
		//detailCoord = exp(-0.5*detailCoord);
		//if (detailCoord >1) detailCoord=1;
				
		break;
	
	case ORIENT:
		// il faut regarder l'angle
		// entre la normale au sommet
		// et la vue de la camra
		
		// notre objectif est de controler la silouette
		
		VueDir.normalize();
		
		scal = dot(normal, VueDir);
		
		if (scal <0) scal = 0;
		
		detailCoord = 2*acos( pow(scal,control_detail) )/3.14159;
				
		if (detailCoord > 1.0) detailCoord=1;
		
		//cout << detailCoord << endl;
		
		break;
	case REFLECT:
		VueDir.normalize();	
		H = LumiereDir+VueDir;
		H.normalize();
		
		// calcule de la composante spculaire
		Spec = dot(NormSommet, H);
		if ( Spec < 0) Spec = 0;
				
		detailCoord = pow(Spec,control_detail);
			
		//if (detailCoord > 1.0) detailCoord=1;
		
		//cout << detailCoord << endl;
		
		break;
	
	
	}
	
	// le flip est contrl par F5
	if (flip) detailCoord=1-detailCoord;
		
}



/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
float rotationSpeed = 1.0;
bool lightRotation = true;

void updateAnimationParam(){
		// compute a time parameter based on a 
		float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
		float t = newTime-oldTime;
		oldTime = newTime;

		// update a rotation parameter
		// A COMPLETER !!!!!!!!!
		
		// si la rotation de la lumire est active
		if (lightRotation)
		{			
			lightAngle += t*rotationSpeed;
					
			// maj de la lumire
			setLight(); 
		}
		
		
}

////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene(const char* objFile, const char* toonTexFile) {
		// init scanline
		glClearColor(0.3, 0.3, 0.3, 1.0f);				
		glClearDepth(1.0);
		glDepthFunc(GL_LESS);												
		glEnable(GL_DEPTH_TEST);										
		glShadeModel(GL_SMOOTH);										
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	
		glLineWidth(2.0f);													
		glPointSize(4.0f);

		// init lighting
		setLight();
		glEnable(GL_LIGHTING);

		// init textures
		loadGLTexture(toonTexFile, &texture);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		// init object
		glEnable(GL_NORMALIZE);
		objectLoaded = loadObj(objFile, true);
}


void changeSize(int w, int h)	{

		// Prevent a divide by zero, when window is too short
		// (you cant make a window of zero width).
		if(h == 0)
				h = 1;
		
		float ratio = 1.0f * w / h;
		// Reset the coordinate system before modifying
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		
		// Set the viewport to be the entire window
		glViewport(0, 0, w, h);
		
		// Set the clipping volume
		gluPerspective(45,ratio,znear,zfar);
		glMatrixMode(GL_MODELVIEW);
		setCamera();
}










///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////


void drawSphere(){
		if ( materialModel == X_TOON ) glBindTexture(GL_TEXTURE_2D,texture);
		
		setMaterial();
		
		glPushMatrix();
		
		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;
				
				// draw one strip
				Vec3 color1, color2;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;
						
						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						glColor3fv(color1);
						glNormal3fv(pos1);
						
						if ( materialModel == X_TOON ) {
							// les coordonnes de texture
							float diffu;
							float detail;
							getTexCoord(pos1, pos1, diffu, detail);
				
							// coord texture
							glTexCoord2f(diffu, detail);	
						
						}
						
						glVertex3fv(pos1);
						
						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						glColor3fv(color2);
						glNormal3fv(pos2);
						
						if ( materialModel == X_TOON ) {
							// les coordonnes de texture
							float diffu;
							float detail;
							getTexCoord(pos2, pos2, diffu, detail);
				
							// coord texture
							glTexCoord2f(diffu, detail);						
						}
						
						glVertex3fv(pos2);
				}
				glEnd();
		}
		glPopMatrix();

		unsetMaterial();

}

void drawObject(){
		
		if ( materialModel == X_TOON ) glBindTexture(GL_TEXTURE_2D,texture);
		
		setMaterial();
		
		glPushMatrix();
 		glScalef(10.0, 10.0, 10.0);
		
		glBegin(GL_TRIANGLES);

		
		// A COMPLETER !!!!!!!!
		
		// on va parcourir les faces triangules pour les afficher
		vector< pair<int, int> >::const_iterator S1faces;
		vector< pair<int, int> >::const_iterator S2faces;
		vector< pair<int, int> >::const_iterator S3faces;
		
		S1faces=faceIdx1.begin();
		S2faces=faceIdx2.begin();
		S3faces=faceIdx3.begin();
		
		Vec3 color1, color2, color3;
		
							
   		while (S1faces!=faceIdx1.end() && S2faces!=faceIdx2.end() && S1faces!=faceIdx2.end() ) {
			
			if (materialModel == PHONG_TOON || materialModel == FULL_TOON )
			{
				Vec3 ColorDiff;
				getColor(vertices[S1faces->first], normals[S1faces->second], ColorDiff);
				
				// maj de la composante diffuse
				glColor3f(ColorDiff.x,ColorDiff.y,ColorDiff.z);				
			}
			else if (materialModel == X_TOON )
			{
				// les coordonnes de texture
				float diffu;
				float detail;
				getTexCoord(vertices[S1faces->first],  normals[S1faces->second], diffu, detail);
				
				// coord texture
				glTexCoord2f(diffu, detail);
			}
			
			glNormal3fv(normals[S1faces->second]);
			glVertex3fv(vertices[S1faces->first]);
			
			
			if (materialModel == PHONG_TOON || materialModel == FULL_TOON )
			{
				Vec3 ColorDiff;
				getColor(vertices[S2faces->first], normals[S2faces->second], ColorDiff);
				
				// maj de la composante diffuse
				glColor3f(ColorDiff.x,ColorDiff.y,ColorDiff.z);					
			}
			else if (materialModel == X_TOON )
			{
				// les coordonnes de texture
				float diffu;
				float detail;
				getTexCoord(vertices[S2faces->first], normals[S2faces->second], diffu, detail);
				
				// coord texture
				glTexCoord2f(diffu, detail);	
			}
			
			
			glNormal3fv(normals[S2faces->second]);
			glVertex3fv(vertices[S2faces->first]);
			
			if (materialModel == PHONG_TOON || materialModel == FULL_TOON)
			{
				Vec3 ColorDiff;
				getColor(vertices[S3faces->first], normals[S3faces->second], ColorDiff);
				
				// maj de la composante diffuse
				glColor3f(ColorDiff.x,ColorDiff.y,ColorDiff.z);					
			}
			else if (materialModel == X_TOON )
			{
				// les coordonnes de texture
				float diffu;
				float detail;
				getTexCoord(vertices[S3faces->first], normals[S3faces->second], diffu, detail);
				
				// coord texture
				glTexCoord2f(diffu, detail);	
			}
			
			glNormal3fv(normals[S3faces->second]);
			glVertex3fv(vertices[S3faces->first]);		
			
			S1faces++;
			S2faces++;
			S3faces++;
   		   
   		}
		
		//faceIdx1, faceIdx2, faceIdx3;
				
		glEnd();
		
		glPopMatrix();

		unsetMaterial();
}


void renderScene(void) {
		// clear buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		// draw objects
		if (objectLoaded){
				drawObject();
		} else {
				drawSphere();
		}

		// update animation parameters
		updateAnimationParam();

		// display rendering
		glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////

    
void specialKey(int key, int x_, int y_) {
		
		switch (key) {
		case GLUT_KEY_LEFT :
				cameraAngle += 0.04f;
				setCamera();  break;
		case GLUT_KEY_RIGHT :
				cameraAngle -=0.04f;
				setCamera();  break;
		case GLUT_KEY_UP :
				distToOrigin -= 0.5f;
				if (distToOrigin<1.5f) distToOrigin = 1.5f;
				setCamera();  break;
		case GLUT_KEY_DOWN :
				distToOrigin += 0.5f;
				setCamera();  break;
		case GLUT_KEY_PAGE_UP :
				cameraHeight += 0.1f;
				setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
				cameraHeight -= 0.1f;
				setCamera();  break;
				
		case GLUT_KEY_F1 :
				detailMap=NONE; break;
		case GLUT_KEY_F2 :
				detailMap=DEPTH; break;
		case GLUT_KEY_F3 :
				detailMap=ORIENT; break;
		case GLUT_KEY_F4 :
				detailMap=REFLECT; break;
		case GLUT_KEY_F5 :
				flip = 1-flip; break;
		}
}




void commonKey(unsigned char key, int x_, int y_) {
		
		switch (key) {
		case 'r':
				lightRotation = true; break;
		case ',':
				lightRotation = false;
				lightAngle += 0.1f;
				setLight(); break;
		case '.':
				lightRotation = false;
				lightAngle -= 0.1f;
				setLight(); break;
		case '1':
				materialModel = PHONG; break;
		case '2':
				materialModel = PHONG_TOON; break;
		case '3':
				materialModel = FULL_TOON; break;
		case '4':
				materialModel = X_TOON; break;
		case 'm':
				material = METAL; break;
		case 'p':
				material = PLASTIC; break;
		case '+':
				if ( materialModel == X_TOON ) {
					control_detail++;
				}else {
					// on incrmente le nb d'intervalles
					nb_intervalles++;
				}
								
				break;
		case '-':
				if ( materialModel == X_TOON ) {
					if (control_detail>0) control_detail--;
				}else {
				
					if (nb_intervalles>1) nb_intervalles--;
				}
				break;
		case '/': 
				rotationSpeed += 0.1; break;
		case '*': 
				rotationSpeed -= 0.1; break;
		}
}





////////////////////////////////////
//  Main method
////////////////////////////////////

void parseArgs(int argc, char **argv, char** objFileName, char** toonTexFileName){

		// default values
		*objFileName = "-";
		*toonTexFileName = "-";

		if (argc>1){
				int nbCar1 = strlen(argv[1]);

				if (argv[1][nbCar1-3]=='o' && argv[1][nbCar1-2]=='b' && argv[1][nbCar1-1]=='j'){
						*objFileName = argv[1];
				} else if (argv[1][nbCar1-3]=='b' && argv[1][nbCar1-2]=='m' && argv[1][nbCar1-1]=='p'){
						*toonTexFileName = argv[1];
				}

				if (argc>2){
						int nbCar2 = strlen(argv[2]);
						
						if (argv[2][nbCar2-3]=='o' && argv[2][nbCar2-2]=='b' && argv[2][nbCar2-1]=='j'){
								*objFileName = argv[2];					
						} else if (argv[2][nbCar2-3]=='b' && argv[2][nbCar2-2]=='m' && argv[2][nbCar2-1]=='p'){
								*toonTexFileName = argv[2];					
						}
				}
		}
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800, 600);
	glutCreateWindow("Humanoid");

	char *objFileName, *toonTexFileName;
	parseArgs(argc, argv, &objFileName, &toonTexFileName);
	initScene(objFileName, toonTexFileName);

	glutSpecialFunc(specialKey);
	glutKeyboardFunc(commonKey);

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);

	glutReshapeFunc(changeSize);

	glutMainLoop();

	return(0);
}
