#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <float.h>
#include <string.h>
#include "Vec3.h"

using namespace std;

////////////////////////////////
// Object mesh methods
////////////////////////////////

vector<Vec3> vertices, normals;
vector< pair<int, int> > faceIdx1, faceIdx2, faceIdx3;
bool objectLoaded = false;
bool flip=false;
float adjustDetailCoord=0.0f;
float znear=1.0, zfar=100.0;
int num_intervals = 10;

bool loadObj(const char * filename, bool normalize)
{
    ifstream in(filename);
    char type;
    char s[256];

    if (!in.is_open())
    {
        cout << "Using default object: unit sphere"  << endl;
        return false;
    }

    vertices.clear();
    normals.clear();
    faceIdx1.clear();
    faceIdx2.clear();
    faceIdx3.clear();

    while (!in.eof())
    {
        if (in.peek() == 'v')
        {
            in >> type;

            if (in.peek() == 'n')
            {
                in >> type;

                float x, y, z;
                in>> x; in>> y; in>> z;
                normals.push_back(Vec3(x,y,z));

            }
            else if (in.peek() != 't')
            {
                float x, y, z;
                in>> x; in>> y; in>> z;
                vertices.push_back(Vec3(x,y,z));
            }
            in.getline(s, 256);

        }
        else if (in.peek() == 'f')
        {
            in >> type;

            int vertex1, vertex2, vertex3;
            int normal1, normal2, normal3;
            char tempByte;

            // vertex 1
            in>> vertex1; --vertex1;
            // separator
            in >> tempByte; in >> tempByte;
            // normal 1
            in>> normal1; --normal1;

            // vertex 2
            in>> vertex2; --vertex2;
            // separator
            in >> tempByte; in >> tempByte;
            // normal 2
            in>> normal2; --normal2;

            // vertex 3
            in>> vertex3; --vertex3;
            // separator
            in >> tempByte; in >> tempByte;
            // normal 3
            in>> normal3; --normal3;

            // Store face
            pair<int, int> idx1 (vertex1, normal1);
            pair<int, int> idx2 (vertex2, normal2);
            pair<int, int> idx3 (vertex3, normal3);

            faceIdx1.push_back(idx1);
            faceIdx2.push_back(idx2);
            faceIdx3.push_back(idx3);

            in.getline(s, 256);
        }
        else
        {
            in.getline(s, 256);
        }
    }
    in.close();

    unsigned int nbVertices = vertices.size();
    cout << "Object loaded: " << endl;
    cout << "\t Filename = " << filename << endl;
    cout << "\t #vertices = " << nbVertices << endl;

    // normalize object scale.
    if (normalize)
    {
        float minX=FLT_MAX;
        float maxX=-FLT_MAX;
        float minY=FLT_MAX;;
        float maxY=-FLT_MAX;
        float minZ=FLT_MAX;
        float maxZ=-FLT_MAX;

        // find bounding box coords
        for (unsigned int i=0 ; i<nbVertices ; i++)
        {
            minX=(minX>vertices[i].x)?vertices[i].x:minX;
            maxX=(maxX<vertices[i].x)?vertices[i].x:maxX;
            minY=(minY>vertices[i].y)?vertices[i].y:minY;
            maxY=(maxY<vertices[i].y)?vertices[i].y:maxY;
            minZ=(minZ>vertices[i].z)?vertices[i].z:minZ;
            maxZ=(maxZ<vertices[i].z)?vertices[i].z:maxZ;
        }

        // modify object vertices so that the maximum distance is unit size...
        Vec3 center (0.5*(maxX+minX), 0.5*(maxY+minY), 0.5*(maxZ+minZ));
        float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
        float scaling=1.0/norm;

        for (unsigned int i=0 ; i<nbVertices ; i++)
        {
            vertices[i].x =(vertices[i].x - center.x)*scaling;
            vertices[i].y =(vertices[i].y - center.y)*scaling;
            vertices[i].z =(vertices[i].z - center.z)*scaling;
        }
    }

    return true;
}

void printBitmapString(char* s)
{

   glRasterPos2f(-5.0f, 3.0f);

#if 0
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glLoadIdentity();
#endif

   glColor3f(1.0, 1.0, 1.0);
   glDisable(GL_LIGHTING);
   glDisable(GL_TEXTURE_2D);

   void* font = GLUT_BITMAP_HELVETICA_18;
   if (s && strlen(s))
   {
      while (*s)
      {
         glutBitmapCharacter(font, *s);
         s++;
      }
   }
#if 0
   glPopMatrix();
#endif
}


////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint  texture;

typedef struct
{
   unsigned long sizeX;
   unsigned long sizeY;
   char *data;
}Image;

int loadImage(const char *filename, Image *image)
{
   // make sure the file is there.
   FILE *file;
   if ((file = fopen(filename, "rb"))==NULL)
   {
      return 0;
   }

   // seek through the bmp header, up to the width/height:
   fseek(file, 18, SEEK_CUR);

   // read width and height, and calculate the size of the image
   unsigned long i;
   if ((i = fread(&image->sizeX, 4, 1, file)) != 1)
   {
      cout << "Error reading width from " << filename << endl;
      return 0;
   }
   if ((i = fread(&image->sizeY, 4, 1, file)) != 1)
   {
      cout << "Error reading height from " << filename << endl;
      return 0;
   }
   unsigned long size = image->sizeX * image->sizeY * 3;

   // read the planes
   unsigned short int planes;
   if ((fread(&planes, 2, 1, file)) != 1)
   {
      cout << "Error reading planes from " << filename << endl;
      return 0;
   }
   if (planes != 1)
   {
      cout << "Planes from " << filename << "is not 1: " << planes << endl;
      return 0;
   }

   // read the bpp
   unsigned short int bpp;
   if ((i = fread(&bpp, 2, 1, file)) != 1)
   {
      cout << "Error reading bpp from " << filename << endl;
      return 0;
   }
   if (bpp != 24)
   {
      cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
      return 0;
   }

   // seek past the rest of the bitmap header.
   fseek(file, 24, SEEK_CUR);

   // read the data.
   image->data = (char *) malloc(size);
   if (image->data == NULL)
   {
      cout << "Error allocating memory for color-corrected image data" << endl;
      return 0;
   }

   if ((i = fread(image->data, size, 1, file)) != 1)
   {
      cout << "Error reading image data from " << filename << endl;
      return 0;
   }

   // reverse all of the colors. (bgr -> rgb)
   char temp;
   for (i=0;i<size;i+=3)
   {
      temp = image->data[i];
      image->data[i] = image->data[i+2];
      image->data[i+2] = temp;
   }

   return 1;
}


GLvoid loadGLTexture(const char* textureName, GLuint* textureIdx)
{
   // Load Texture
   Image *image;

   // allocate space for cube texture
   image = (Image *) malloc(sizeof(Image));
   if (image == NULL)
   {
      printf("Error allocating space for image");
      exit(0);
   }
   if (!loadImage(textureName, image))
   {
      if (!loadImage("defaultToonTex.bmp", image))
      {
         cerr << "Default texture fot found : defaultToonTex.bmp" << endl;
         exit(1);
      }
      else
      {
         cout << "Using default texture: defaultToonTex.bmp" << endl;
      }
   }
   else
   {
      cout << "Texture Loaded: " << endl;
      cout << "\t Filename: " << textureName << endl;
      cout << "\t Size: " << image->sizeX << endl;
   }

   // Create Texture
   glGenTextures(1, textureIdx);
   glBindTexture(GL_TEXTURE_2D, *textureIdx);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
   glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
};

////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 0.3f, 0.3f, 0.3f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;
float lightRadius = 5.0;
float lightHeight = 1.0;
Vec3 light (lightRadius, lightRadius, 0.0);

void setLight()
{
   light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));
   GLfloat lightPosition[]= {light.x, light.y, light.z, 1.0f};
   glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
   glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
   glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
   glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
   glEnable(GL_LIGHT0);
}


/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 0.0f;
Vec3 camera(0.0, cameraHeight, distToOrigin);

void setCamera()
{
   float x = distToOrigin*cos(cameraAngle);
   float y = cameraHeight;
   float z = distToOrigin*sin(cameraAngle);
   camera = Vec3(x, y, z);

   glLoadIdentity();
   gluLookAt(x, y, z,
         -x, y, -z,
         0.0f,1.0f,0.0f);
   setLight();
}

GLfloat segmentation(GLfloat v)
{
   GLfloat step = 1.0f / GLfloat(num_intervals);
   int k = int(v / step);

   return (GLfloat(k)+0.5f) * step;
}

////////////////////////////////////////
//  Material methods
////////////////////////////////////////
enum
{
   MODEL_PHONG      = 1,
   MODEL_PHONGTOON  = 2,
   MODEL_FULLTOON   = 3,
   MODEL_XTOON      = 4
};

enum
{
   MATERIAL_PLASTIC = 0,
   MATERIAL_METAL   = 1,
};

enum
{
   DETAILMAP_NONE,
   DETAILMAP_DEPTH,
   DETAILMAP_ORIENT,
   DETAILMAP_REFLECT,
};

int materialModel = MODEL_XTOON;
int material = MATERIAL_PLASTIC;
int detailMap = DETAILMAP_ORIENT;


const char* getDetailMapString()
{
   switch(detailMap)
   {
      case DETAILMAP_REFLECT:
         return "reflect"; break;
      case DETAILMAP_ORIENT:
         return "orient"; break;
      case DETAILMAP_DEPTH:
         return "depth"; break;
      case DETAILMAP_NONE:
      default:
         return "none"; break;
   }
}

// plastic material parameters
const GLfloat diff_plastic[] = { 0.4, 0.4, 0.7, 1.0};
const GLfloat spec_plastic[] = { 1.0, 1.0, 1.0, 1.0};
const GLfloat shin_plastic = 100.0f;

// metallic material parameters
const GLfloat diff_metal[]   = { 0.2, 0.2, 0.2, 1.0};
const GLfloat spec_metal[]   = { 0.3, 0.3, 0.4, 1.0};
const GLfloat shin_metal = 10.0f;

// active material parameters
GLfloat diff_current[]   = { 0.2, 0.2, 0.2, 1.0};
GLfloat spec_current[]   = { 0.3, 0.3, 0.4, 1.0};
GLfloat shin_current = 10.0f;

// toon shading parameters
// A COMPLETER !!!!!!!!

// X-Toon shading parameters
// A COMPLETER !!!!!!!!
void setMaterial()
{
   switch(material)
   {
      case MATERIAL_PLASTIC:
         glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_plastic);
         glMaterialfv(GL_FRONT, GL_SPECULAR, spec_plastic);
         glMaterialf(GL_FRONT, GL_SHININESS, shin_plastic);

         memcpy(diff_current, diff_plastic, sizeof(GLfloat) * 4);
         memcpy(spec_current, spec_plastic, sizeof(GLfloat) * 4);
         shin_current = shin_plastic;
         break;

      case MATERIAL_METAL:
         glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_metal);
         glMaterialfv(GL_FRONT, GL_SPECULAR, spec_metal);
         glMaterialf(GL_FRONT, GL_SHININESS, shin_metal);

         memcpy(diff_current, diff_metal, sizeof(GLfloat) * 4);
         memcpy(spec_current, spec_metal, sizeof(GLfloat) * 4);
         shin_current = shin_metal;
         break;
   }
}


void unsetMaterial()
{
   glMaterialf(GL_FRONT, GL_DIFFUSE,   0.0f);
   glMaterialf(GL_FRONT, GL_SPECULAR,   0.0f);
   glMaterialf(GL_FRONT, GL_SHININESS,  0.0f);
   glEnable(GL_LIGHTING);
}


void getColor_PHONGTOON(const Vec3& vertex, const Vec3& normal, Vec3& color)
{
   color = Vec3(0, 0, 0);

   Vec3 L = light - vertex;
   Vec3 V = camera - vertex;
   L.normalize();
   V.normalize();

   float diffuse = dot(normal, L);
   if(diffuse < 0.0f)
      diffuse = 0.0f;

   diffuse = segmentation(diffuse);

   color.x += diffuse * diff_current[0] * lightDiffuse[0];
   color.y += diffuse * diff_current[1] * lightDiffuse[1];
   color.z += diffuse * diff_current[2] * lightDiffuse[2];
}

void getColor_FULLTOON(const Vec3& vertex, const Vec3& normal, Vec3& color)
{
   color = Vec3(0, 0, 0);

   Vec3 L = light - vertex;
   Vec3 V = camera - vertex;

   V.normalize();
   L.normalize();

   float diffuse = dot(normal, L);
   if(diffuse < 0.0f)
      diffuse = 0.0f;

   diffuse = segmentation(diffuse);

   color.x += diffuse * diff_current[0] * lightDiffuse[0];
   color.y += diffuse * diff_current[1] * lightDiffuse[1];
   color.z += diffuse * diff_current[2] * lightDiffuse[2];

   Vec3 H = L + V;
   H.normalize();

   float d = dot(normal, H);

   float specular = powf(d, shin_current);
   if(specular >= 0.5f)
      specular = 1.0f;
   else
      specular = 0.0f;

   color.x += specular * spec_current[0];
   color.y += specular * spec_current[1];
   color.z += specular * spec_current[2];
}


float alpha = 100.0f;

void getTexCoord(const Vec3& vertex, const Vec3& normal, float& toneCoord, float& detailCoord)
{
   Vec3 L = light  - vertex;
   Vec3 V = camera - vertex;
   Vec3 H = L + V;
   H.normalize();


   float dist = V.norm();
   float d = dot(normal, H);
   L.normalize();
   V.normalize();
   float cos = dot(normal, V);

   float specular;
   float diffuse = dot(normal, L);
   if(diffuse < 0.0f)
      diffuse = 0.0f;

   toneCoord = diffuse;
   detailCoord = 0.0f;

   switch(detailMap)
   {
      case DETAILMAP_NONE:
         detailCoord = 1.0;
         detailCoord += adjustDetailCoord;
         break;
      case DETAILMAP_DEPTH:
         detailCoord = dist / (alpha - znear);
         break;
      case DETAILMAP_ORIENT:
         detailCoord = cos;
         detailCoord += adjustDetailCoord;
         break;
      case DETAILMAP_REFLECT:
         specular = powf(d, shin_current);
         detailCoord = specular;
         detailCoord += adjustDetailCoord;
         break;
   }


   if(flip)
      detailCoord = 1.0f - detailCoord;
}


/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
float rotationSpeed = 1.0;
bool lightRotation = true;

void updateAnimationParam()
{
   // compute a time parameter based on a
   float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
   float t = newTime-oldTime;
   oldTime = newTime;

   // update a rotation parameter
   if(lightRotation)
   {
      lightAngle += t * rotationSpeed;
   }
   else
   {

   }
}


////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene(const char* objFile, const char* toonTexFile)
{
   // init scanline
   glClearColor(0.3, 0.3, 0.3, 1.0f);
   glClearDepth(1.0);
   glDepthFunc(GL_LESS);
   glEnable(GL_DEPTH_TEST);
   glShadeModel(GL_SMOOTH);
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
   //glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
   glLineWidth(2.0f);
   glPointSize(4.0f);

   // init lighting
   setLight();
   glEnable(GL_LIGHTING);

   // init textures
   loadGLTexture(toonTexFile, &texture);
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

   // init object
   glEnable(GL_NORMALIZE);
   objectLoaded = loadObj(objFile, true);
}


void changeSize(int w, int h)
{
   // Prevent a divide by zero, when window is too short
   // (you cant make a window of zero width).
   if(h == 0)
      h = 1;

   float ratio = 1.0f * w / h;
   // Reset the coordinate system before modifying
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();

   // Set the viewport to be the entire window
   glViewport(0, 0, w, h);

   // Set the clipping volume
   gluPerspective(45,ratio,znear,zfar);
   glMatrixMode(GL_MODELVIEW);
   setCamera();
}


///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////

void drawSphere()
{
   setMaterial();

   glPushMatrix();

   int nbStrips = 20;
   float stepVAngle = M_PI/nbStrips;

   int nbFacesPerStrip = 50;
   float stepHAngle = 2*M_PI/nbFacesPerStrip;

   for(int i=0 ; i<nbStrips ; i++)
   {
      float vAngle1 = i*stepVAngle - 0.5*M_PI;
      float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;

      // draw one strip
      Vec3 color1, color2;
      glBegin(GL_QUAD_STRIP);
      for (int j=0 ; j<=nbFacesPerStrip ; j++)
      {
         float hAngle = j*stepHAngle;

         float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
         glColor3fv(color1);
         glNormal3fv(pos1);
         glVertex3fv(pos1);

         float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
         glColor3fv(color2);
         glNormal3fv(pos2);
         glVertex3fv(pos2);
      }
      glEnd();
   }
   glPopMatrix();

   unsetMaterial();
}


void drawObject_PHONG()
{
   glDisable(GL_TEXTURE_2D);
   glDisable(GL_COLOR_MATERIAL);
   glEnable(GL_LIGHTING);

   setMaterial();

   glPushMatrix();
   glScalef(10.0, 10.0, 10.0);

   glBegin(GL_TRIANGLES);

   for(unsigned int i=0;i < faceIdx1.size();++i)
   {
      std::pair<int, int> p1 = faceIdx1[i];
      std::pair<int, int> p2 = faceIdx2[i];
      std::pair<int, int> p3 = faceIdx3[i];

      Vec3& v1 = vertices[p1.first];
      Vec3& v2 = vertices[p2.first];
      Vec3& v3 = vertices[p3.first];

      Vec3& n1 = normals[p1.second];
      Vec3& n2 = normals[p2.second];
      Vec3& n3 = normals[p3.second];

      glNormal3fv(n1);
      glVertex3fv(v1);

      glNormal3fv(n2);
      glVertex3fv(v2);

      glNormal3fv(n3);
      glVertex3fv(v3);
   }

   glEnd();

   glPopMatrix();

   unsetMaterial();

   printBitmapString("PHONG");
}


void drawObject_PHONGTOON()
{
   glDisable(GL_TEXTURE_2D);
   glColorMaterial(GL_FRONT, GL_DIFFUSE);
   glEnable(GL_COLOR_MATERIAL);
   glEnable(GL_LIGHTING);

   setMaterial();

   glPushMatrix();
   glScalef(10.0, 10.0, 10.0);

   glBegin(GL_TRIANGLES);

   for(unsigned int i=0;i < faceIdx1.size();++i)
   {
      std::pair<int, int> p1 = faceIdx1[i];
      std::pair<int, int> p2 = faceIdx2[i];
      std::pair<int, int> p3 = faceIdx3[i];

      Vec3& v1 = vertices[p1.first];
      Vec3& v2 = vertices[p2.first];
      Vec3& v3 = vertices[p3.first];

      Vec3& n1 = normals[p1.second];
      Vec3& n2 = normals[p2.second];
      Vec3& n3 = normals[p3.second];

      Vec3 diff1;
      Vec3 diff2;
      Vec3 diff3;

      getColor_PHONGTOON(v1, n1, diff1);
      getColor_PHONGTOON(v2, n2, diff2);
      getColor_PHONGTOON(v3, n3, diff3);

      glColor3fv(diff1);
      glNormal3fv(n1);
      glVertex3fv(v1);

      glColor3fv(diff2);
      glNormal3fv(n2);
      glVertex3fv(v2);

      glColor3fv(diff3);
      glNormal3fv(n3);
      glVertex3fv(v3);
   }

   glEnd();

   glPopMatrix();

   unsetMaterial();

   glRasterPos2f(-5.0f, 3.0f);
   printBitmapString("PHONGTOON");
}


void drawObject_FULLTOON()
{
   glDisable(GL_LIGHTING);
   glDisable(GL_TEXTURE_2D);

   setMaterial();

   glPushMatrix();
   glScalef(10.0, 10.0, 10.0);

   glBegin(GL_TRIANGLES);

   for(unsigned int i=0;i < faceIdx1.size();++i)
   {
      std::pair<int, int> p1 = faceIdx1[i];
      std::pair<int, int> p2 = faceIdx2[i];
      std::pair<int, int> p3 = faceIdx3[i];

      Vec3& v1 = vertices[p1.first];
      Vec3& v2 = vertices[p2.first];
      Vec3& v3 = vertices[p3.first];

      Vec3& n1 = normals[p1.second];
      Vec3& n2 = normals[p2.second];
      Vec3& n3 = normals[p3.second];

      Vec3 diff1;
      Vec3 diff2;
      Vec3 diff3;

      getColor_FULLTOON(v1, n1, diff1);
      getColor_FULLTOON(v2, n2, diff2);
      getColor_FULLTOON(v3, n3, diff3);

      glColor3fv(diff1);
      glVertex3fv(v1);

      glColor3fv(diff2);
      glVertex3fv(v2);

      glColor3fv(diff3);
      glVertex3fv(v3);
   }

   glEnd();

   glPopMatrix();

   unsetMaterial();

   glRasterPos2f(-5.0f, 3.0f);
   printBitmapString("FULLTOON");
}

void drawObject_XTOON()
{
   glDisable(GL_LIGHTING);
   glEnable(GL_TEXTURE_2D);

   setMaterial();

   glPushMatrix();
   glScalef(10.0, 10.0, 10.0);

   glBegin(GL_TRIANGLES);

   for(unsigned int i=0;i < faceIdx1.size();++i)
   {
      std::pair<int, int> p1 = faceIdx1[i];
      std::pair<int, int> p2 = faceIdx2[i];
      std::pair<int, int> p3 = faceIdx3[i];

      Vec3& v1 = vertices[p1.first];
      Vec3& v2 = vertices[p2.first];
      Vec3& v3 = vertices[p3.first];

      Vec3& n1 = normals[p1.second];
      Vec3& n2 = normals[p2.second];
      Vec3& n3 = normals[p3.second];

      float u, v;

      getTexCoord(v1, n1, u, v);
      glTexCoord2f(u, v);
      glVertex3fv(v1);

      getTexCoord(v2, n2, u, v);
      glTexCoord2f(u, v);
      glVertex3fv(v2);

      getTexCoord(v3, n3, u, v);
      glTexCoord2f(u, v);
      glVertex3fv(v3);
   }

   glEnd();

   glPopMatrix();

   unsetMaterial();

   glRasterPos2f(-5.0f, 3.0f);
   char buffer[256];
   sprintf(buffer, "XTOON (detail map:%s, %s)",
         getDetailMapString(),
         flip? "flipped" : "not flipped"
         );
   printBitmapString(buffer);
}

void renderScene(void)
{
   // clear buffers
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_CULL_FACE);

   // draw objects
   if (objectLoaded)
   {
      switch(materialModel)
      {
         case MODEL_PHONG:     drawObject_PHONG();     break;
         case MODEL_PHONGTOON: drawObject_PHONGTOON(); break;
         case MODEL_FULLTOON:  drawObject_FULLTOON();  break;
         case MODEL_XTOON:     drawObject_XTOON();     break;
      }
   }
   else
   {
      drawSphere();
   }

   // update animation parameters
   updateAnimationParam();
   setLight();

   // display rendering
   glutSwapBuffers();
}


///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////

void specialKey(int key, int x_, int y_)
{
   switch (key)
   {
      case GLUT_KEY_LEFT :
         cameraAngle += 0.04f;
         setCamera();  break;
      case GLUT_KEY_RIGHT :
         cameraAngle -=0.04f;
         setCamera();  break;
      case GLUT_KEY_UP :
         distToOrigin -= 0.5f;
         if (distToOrigin<1.5f) distToOrigin = 1.5f;
         setCamera();  break;
      case GLUT_KEY_DOWN :
         distToOrigin += 0.5f;
         setCamera();  break;
      case GLUT_KEY_PAGE_UP :
         cameraHeight += 0.1f;
         setCamera();  break;
      case GLUT_KEY_PAGE_DOWN :
         cameraHeight -= 0.1f;
         setCamera();  break;
      case GLUT_KEY_F1 :
         detailMap=DETAILMAP_NONE; break;
      case GLUT_KEY_F2 :
         detailMap=DETAILMAP_DEPTH; break;
      case GLUT_KEY_F3 :
         detailMap=DETAILMAP_ORIENT; break;
      case GLUT_KEY_F4 :
         detailMap=DETAILMAP_REFLECT; break;
   }
}


void commonKey(unsigned char key, int x_, int y_)
{
   switch (key)
   {
      case 'f':
         flip = flip? false : true; break;
      case 'r':
         lightRotation = true; break;
      case ',':
         lightRotation = false;
         lightAngle += 0.1f;
         setLight(); break;
      case '.':
         lightRotation = false;
         lightAngle -= 0.1f;
         setLight(); break;
      case '1':
         materialModel = MODEL_PHONG; break;
      case '2':
         materialModel = MODEL_PHONGTOON; break;
      case '3':
         materialModel = MODEL_FULLTOON; break;
      case '4':
         materialModel = MODEL_XTOON; break;
      case 'm':
         material = MATERIAL_METAL; break;
      case 'p':
         material = MATERIAL_PLASTIC; break;
      case '+':
         adjustDetailCoord+=0.1f;
         alpha += 1.0f;
         num_intervals ++; break;
      case '-':
         alpha -= 1.0f;
         adjustDetailCoord-=0.1f;
         num_intervals --; 
         if(num_intervals < 1)
            num_intervals = 1;
         break;
      case '/':
         rotationSpeed += 0.1; break;
      case '*':
         rotationSpeed -= 0.1; break;
   }
}


////////////////////////////////////
//  Main method
////////////////////////////////////

void parseArgs(int argc, char **argv, char** objFileName, char** toonTexFileName)
{
   // default values
   *objFileName = "-";
   *toonTexFileName = "-";

   if (argc>1)
   {
      int nbCar1 = strlen(argv[1]);

      if (argv[1][nbCar1-3]=='o' && argv[1][nbCar1-2]=='b' && argv[1][nbCar1-1]=='j')
      {
         *objFileName = argv[1];
      }
      else if (argv[1][nbCar1-3]=='b' && argv[1][nbCar1-2]=='m' && argv[1][nbCar1-1]=='p')
      {
         *toonTexFileName = argv[1];
      }

      if (argc>2)
      {
         int nbCar2 = strlen(argv[2]);

         if (argv[2][nbCar2-3]=='o' && argv[2][nbCar2-2]=='b' && argv[2][nbCar2-1]=='j')
         {
            *objFileName = argv[2];
         }
         else if (argv[2][nbCar2-3]=='b' && argv[2][nbCar2-2]=='m' && argv[2][nbCar2-1]=='p')
         {
            *toonTexFileName = argv[2];
         }
      }
   }
}


int main(int argc, char **argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
   glutInitWindowPosition(100,100);
   glutInitWindowSize(800, 600);
   glutCreateWindow("Humanoid");
   glutFullScreen();

   char *objFileName, *toonTexFileName;
   parseArgs(argc, argv, &objFileName, &toonTexFileName);
   initScene(objFileName, toonTexFileName);

   glutSpecialFunc(specialKey);
   glutKeyboardFunc(commonKey);

   glutDisplayFunc(renderScene);
   glutIdleFunc(renderScene);

   glutReshapeFunc(changeSize);

   glutMainLoop();

   return(0);
}
