#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <float.h>
#include <string.h>
#include "Vec3.h"

using namespace std;

////////////////////////////////
// Object mesh methods
////////////////////////////////

int segmentation = 1;
float znear=1.0, zfar=100.0;
vector<Vec3> vertices, normals;
vector< pair<int, int> > faceIdx1, faceIdx2, faceIdx3;
bool objectLoaded = false;
bool inverse = false;


bool loadObj(const char * filename, bool normalize) {
	
  ifstream in(filename);
	char type;
	char s[256];
		
  if (!in.is_open()) {
			cout << "Using default object: unit sphere"  << endl;
    return false;
	}

	vertices.clear();
	normals.clear();
	faceIdx1.clear();
	faceIdx2.clear();
	faceIdx3.clear();
	
	while (!in.eof()) {
		if (in.peek() == 'v') {
			in >> type;
			
			if (in.peek() == 'n'){
				in >> type;
			
				float x, y, z;
				in>> x; in>> y; in>> z;
				normals.push_back(Vec3(x,y,z));

			}	else if (in.peek() != 't') {
				float x, y, z;
				in>> x; in>> y; in>> z;
				vertices.push_back(Vec3(x,y,z));
			}
			in.getline(s, 256);
			
		}	else if (in.peek() == 'f') {
			in >> type;

			int vertex1, vertex2, vertex3;
			int normal1, normal2, normal3;
			char tempByte;
			
			// vertex 1
			in>> vertex1; --vertex1;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 1
			in>> normal1; --normal1;
			
			// vertex 2
			in>> vertex2; --vertex2;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 2
			in>> normal2; --normal2;
			
			// vertex 3
			in>> vertex3; --vertex3;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 3
			in>> normal3; --normal3;
			
			// Store face
			pair<int, int> idx1 (vertex1, normal1);
			pair<int, int> idx2 (vertex2, normal2);
			pair<int, int> idx3 (vertex3, normal3);

			faceIdx1.push_back(idx1);
			faceIdx2.push_back(idx2);
			faceIdx3.push_back(idx3);
			
			in.getline(s, 256);
		} else {
			in.getline(s, 256);
		}
	}
	in.close();

	
	unsigned int nbVertices = vertices.size();	
	cout << "Object loaded: " << endl;
	cout << "\t Filename = " << filename << endl;
	cout << "\t #vertices = " << nbVertices << endl;

	// normalize object scale.
	if (normalize) {
		float minX=FLT_MAX;
		float maxX=-FLT_MAX;
		float minY=FLT_MAX;;
		float maxY=-FLT_MAX;
		float minZ=FLT_MAX;
		float maxZ=-FLT_MAX;

		// find bounding box coords
		for (unsigned int i=0 ; i<nbVertices ; i++){
			minX=(minX>vertices[i].x)?vertices[i].x:minX;				
			maxX=(maxX<vertices[i].x)?vertices[i].x:maxX;
			minY=(minY>vertices[i].y)?vertices[i].y:minY;				
			maxY=(maxY<vertices[i].y)?vertices[i].y:maxY;
			minZ=(minZ>vertices[i].z)?vertices[i].z:minZ;				
			maxZ=(maxZ<vertices[i].z)?vertices[i].z:maxZ;
		}

		// modify object vertices so that the maximum distance is unit size...
		Vec3 center (0.5*(maxX+minX), 0.5*(maxY+minY), 0.5*(maxZ+minZ));
		float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
	  float scaling=1.0/norm;
	
	  for (unsigned int i=0 ; i<nbVertices ; i++) {		  
	    vertices[i].x =(vertices[i].x - center.x)*scaling;
	    vertices[i].y =(vertices[i].y - center.y)*scaling;
	    vertices[i].z =(vertices[i].z - center.z)*scaling;
	  }
	}


	return true;			
}










////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint	texture;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;


int loadImage(const char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;		
    if ((file = fopen(filename, "rb"))==NULL) {
				return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
				cout << "Error reading width from " << filename << endl;
				return 0;
    }    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
				cout << "Error reading height from " << filename << endl;
				return 0;
    }    
		unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
				cout << "Error reading planes from " << filename << endl;
				return 0;
    }
    if (planes != 1) {
				cout << "Planes from " << filename << "is not 1: " << planes << endl;
				return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) { 
				cout << "Error reading bpp from " << filename << endl;
				return 0;
    }
    if (bpp != 24) {
				cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
				return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
				cout << "Error allocating memory for color-corrected image data" << endl;
				return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
				cout << "Error reading image data from " << filename << endl;
				return 0;
    }

		// reverse all of the colors. (bgr -> rgb) 
		char temp;                         
		for (i=0;i<size;i+=3) { 
				temp = image->data[i];
				image->data[i] = image->data[i+2];
				image->data[i+2] = temp;
    }
		
    return 1;
}


GLvoid loadGLTexture(const char* textureName, GLuint* textureIdx) {	
    // Load Texture
    Image *image;
    
    // allocate space for cube texture
    image = (Image *) malloc(sizeof(Image));
    if (image == NULL) {
				printf("Error allocating space for image");
				exit(0);
    }
    if (!loadImage(textureName, image)){
				if (!loadImage("defaultToonTex.bmp", image)) {
						cerr << "Default texture fot found : defaultToonTex.bmp" << endl;
						exit(1);
				} else {
						cout << "Using default texture: defaultToonTex.bmp" << endl;
				}
    } else {
				cout << "Texture Loaded: " << endl;
				cout << "\t Filename: " << textureName << endl;
				cout << "\t Size: " << image->sizeX << endl;
		} 

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP); 
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
};






////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;
float lightRadius = 5.0;
float lightHeight = 1.0;
Vec3 light (lightRadius, lightRadius, 0.0);

void setLight(){

		light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));
		GLfloat lightPosition[]= {light.x, light.y, light.z, 1.0f};
		glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
		glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
		glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
		glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
		glEnable(GL_LIGHT0);
}





/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 0.0f;
Vec3 camera(0.0, cameraHeight, distToOrigin);

void setCamera() {

		float x = distToOrigin*cos(cameraAngle);
		float y = cameraHeight;
		float z = distToOrigin*sin(cameraAngle);
		camera = Vec3(x, y, z);
		
		glLoadIdentity();
		gluLookAt(x, y, z,
							-x, y, -z,
							0.0f,1.0f,0.0f);
		setLight();
}

///////////////////////////////////////
//  Material methods
////////////////////////////////////////
int PHONG=1, PHONG_TOON=2, FULL_TOON=3, X_TOON=4;
int materialModel = PHONG;

int PLASTIC=0, METAL=1;
int material = PLASTIC;

int NONE=0, DEPTH=1, ORIENT=2, REFLECT=3;
int detailMap = NONE;

// plastic material parameters
Vec3 plastAmbient (0.0,0.0,0.0);
Vec3 plastDiffuse (0.3,0.3,0.0);
Vec3 plastSpecular (0.7,0.7,0.7);
GLfloat plastshininess[] = { 100 };

// metallic material parameters
Vec3 metalAmbient (0.0,0.0,0.0);
Vec3 metalDiffuse (0.3,0.3,0.0);
Vec3 metalSpecular (0.3,0.3,0.0);
GLfloat metalshininess[] = { 5 };

Vec3 currentAmbient (0.0,0.0,0.0);
Vec3 currentDiffuse (0.0,0.0,0.0);
Vec3 currentSpecular (0.0,0.0,0.0);
GLfloat currentshininess[] = { 0 };

void setMaterial(){

	if (materialModel == PHONG_TOON){
		glEnable(GL_COLOR_MATERIAL);
		glColorMaterial(GL_FRONT,GL_DIFFUSE);
	}

	if (materialModel == FULL_TOON){
		glDisable(GL_LIGHTING);
	}

	if (materialModel == X_TOON){
		glDisable(GL_LIGHTING);
		glEnable(GL_TEXTURE_2D);
	}

	//////////////////////////////////////////////////////////////////////////////////////

	if (material == METAL){
		currentAmbient = metalAmbient;
		currentDiffuse = metalDiffuse;
		currentSpecular = metalSpecular;
		currentshininess[0] = metalshininess[0];
	}

	if (material == PLASTIC){
		currentAmbient = plastAmbient;
		currentDiffuse = plastDiffuse;
		currentSpecular = plastSpecular;
		currentshininess[0] = plastshininess[0];
	}

	glMaterialfv(GL_FRONT,GL_AMBIENT,currentAmbient);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,currentDiffuse);
	glMaterialfv(GL_FRONT,GL_SPECULAR,currentSpecular);
	glMaterialfv(GL_FRONT,GL_SHININESS,currentshininess);

}

void unsetMaterial(){
	if (materialModel == PHONG_TOON){
		glDisable(GL_COLOR_MATERIAL);
	}

	if (materialModel == FULL_TOON){
		glEnable(GL_LIGHTING);
	}

	if (materialModel == X_TOON){
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
	}

}

void getColor(const Vec3& vertex, const Vec3& normal, Vec3& color){
		float diff;
		float specVal;
		Vec3 lightDir (light.x-vertex.x,light.y-vertex.y,light.z-vertex.z);
		Vec3 camDir (camera.x-vertex.x,camera.y-vertex.y,camera.z-vertex.z);

		lightDir= lightDir.normalized();
		camDir= camDir.normalized();
		Vec3 vertexNorm = normal.normalized();

		diff = dot(lightDir, vertexNorm);
		if(diff<0){
			diff = 0;
		}
		float seg = ceil(diff*segmentation);
		float val = (seg-0.5)/segmentation;

		if (materialModel == PHONG_TOON){
			Vec3 colorT(val*currentDiffuse.x,val*currentDiffuse.y, val*currentDiffuse.z);
			color = colorT;
		}

		if (materialModel == FULL_TOON){
			Vec3 h = lightDir + camDir;
			h = h.normalized();
			specVal = dot(h, vertexNorm);
			if (specVal < 0){
				specVal = 0;
			}
			specVal = pow(specVal,(float)currentshininess[0]);
			if (specVal>0.5){
				specVal = 1.0;
			}else{
				specVal = 0.0;
			}
			Vec3 colorT((val*currentDiffuse.x+specVal*currentSpecular.x),(val*currentDiffuse.y+specVal*currentSpecular.y), (val*currentDiffuse.z+specVal*currentSpecular.z));
			color = colorT;
		}
}


void getTexCoord(const Vec3& vertex, const Vec3& normal, float& toneCoord, float& detailCoord){

	Vec3 orig (0.0,0.0,0.0);
	float diff;
	int distCam;

	Vec3 lightDir (light.x-vertex.x,light.y-vertex.y,light.z-vertex.z);
	lightDir= lightDir.normalized();

	Vec3 vertexNorm = normal.normalized();

	Vec3 camDir (camera.x-vertex.x,camera.y-vertex.y,camera.z-vertex.z);
	camDir= camDir.normalized();

	diff = dot(lightDir, vertexNorm);
	if(diff < 0){
		diff = 0;
	}

	toneCoord = diff;

	if (detailMap == NONE){
		detailCoord = 1.0;
	}

	if (detailMap == DEPTH){

		distCam = segmentation;

		if(distCam > zfar){
			distCam = (int)zfar;
		}
		if(distCam < znear){
			distCam = (int)znear;
		}

		distCam = 101 - distCam;

		float dista = dist(vertex, camera);

		if (dista > distCam){
			dista = (float)distCam;
		}
		detailCoord = dista/(float)distCam;
	}

	if (detailMap == ORIENT){
		float diffOrient;
		diffOrient = dot(camDir, vertexNorm);
		if(diffOrient < 0){
			diffOrient = 0;
		}
		diffOrient = pow(diffOrient,segmentation);
		detailCoord = 1 - diffOrient;
	}

	if (detailMap == REFLECT){
		float specVal;
		Vec3 h = lightDir + camDir;
		h = h.normalized();
		specVal = dot(h, vertexNorm);
		if(specVal < 0){
			specVal = 0;
		}
		specVal = pow(specVal,(float)currentshininess[0]);
		specVal = pow(specVal,1/(float)segmentation);
		detailCoord = specVal;
	}

	if (inverse == false){
		detailCoord = detailCoord;
	}else{
		detailCoord = 1.0-detailCoord;
	}
}



/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
float rotationSpeed = 1.0;
bool lightRotation = true;

void updateAnimationParam(){
		// compute a time parameter based on a
		float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
		float t = newTime-oldTime;
		oldTime = newTime;

		// update a rotation parameter
		if (lightRotation == true){
				lightAngle += rotationSpeed*t;
				setLight();
		}
}

////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene(const char* objFile, const char* toonTexFile) {
		// init scanline
		glClearColor(0.3, 0.3, 0.3, 1.0f);
		glClearDepth(1.0);
		glDepthFunc(GL_LESS);
		glEnable(GL_DEPTH_TEST);
		glShadeModel(GL_SMOOTH);
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		glLineWidth(2.0f);
		glPointSize(4.0f);

		// init lighting
		setLight();
		glEnable(GL_LIGHTING);

		// init textures
		loadGLTexture(toonTexFile, &texture);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		// init object
		glEnable(GL_NORMALIZE);
		objectLoaded = loadObj(objFile, true);
}


void changeSize(int w, int h)	{

		// Prevent a divide by zero, when window is too short
		// (you cant make a window of zero width).
		if(h == 0)
				h = 1;

		float ratio = 1.0f * w / h;
		// Reset the coordinate system before modifying
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		// Set the viewport to be the entire window
		glViewport(0, 0, w, h);

		// Set the clipping volume
		gluPerspective(45,ratio,znear,zfar);
		glMatrixMode(GL_MODELVIEW);
		setCamera();
}

///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////


void drawSphere(){

		setMaterial();

		glPushMatrix();

		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;

		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;

				// draw one strip
				Vec3 color1, color2;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;

						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						glColor3fv(color1);
						glNormal3fv(pos1);
						glVertex3fv(pos1);

						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						glColor3fv(color2);
						glNormal3fv(pos2);
						glVertex3fv(pos2);
				}
				glEnd();
		}
		glPopMatrix();

		unsetMaterial();

}

void drawObject(){

		setMaterial();

		Vec3 color1(1.0, 0.0, 0.0);
		float tex1 = 0.0;
		float tex2 = 0.0;

		glPushMatrix();
 		glScalef(10.0, 10.0, 10.0);

		int vertex1, vertex2, vertex3;
		int normal1, normal2, normal3;

		for (int i = 0; i < faceIdx1.size(); i++) {

		vertex1 = faceIdx1[i].first;
		vertex2 = faceIdx2[i].first;
		vertex3 = faceIdx3[i].first;
		//cout << vertex1 << endl;
		normal1 = faceIdx1[i].second;
		normal2 = faceIdx2[i].second;
		normal3 = faceIdx3[i].second;
		//cout << normal1 << endl;

		glBegin(GL_TRIANGLES);
		if (materialModel != X_TOON){
		getColor(vertices[vertex1],normals[normal1],color1);
		glColor3fv(color1);
		}else{
		getTexCoord(vertices[vertex1],normals[normal1],tex1,tex2);
		glTexCoord2f(tex1, tex2);
		}
		glNormal3fv(normals[normal1]);
		glVertex3fv(vertices[vertex1]);

		if (materialModel != X_TOON){
		getColor(vertices[vertex2],normals[normal2],color1);
		glColor3fv(color1);
		}else{
		getTexCoord(vertices[vertex2],normals[normal2],tex1,tex2);
		glTexCoord2f(tex1, tex2);
		}
		glNormal3fv(normals[normal2]);
		glVertex3fv(vertices[vertex2]);

		if (materialModel != X_TOON){
		getColor(vertices[vertex3],normals[normal3],color1);
		glColor3fv(color1);
		}else{
		getTexCoord(vertices[vertex3],normals[normal3],tex1,tex2);
		glTexCoord2f(tex1, tex2);
		}
		glNormal3fv(normals[normal3]);
		glVertex3fv(vertices[vertex3]);

		glEnd();
		}

		glPopMatrix();

		unsetMaterial();
}

void renderScene(void) {
		// clear buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		// draw objects
		if (objectLoaded){
				drawObject();
		} else {
				drawSphere();
		}

		// update animation parameters
		updateAnimationParam();

		// display rendering
		glutSwapBuffers();
}

///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////

    
void specialKey(int key, int x_, int y_) {
		
		switch (key) {
		case GLUT_KEY_LEFT :
				cameraAngle += 0.04f;
				setCamera();  break;
		case GLUT_KEY_RIGHT :
				cameraAngle -=0.04f;
				setCamera();  break;
		case GLUT_KEY_UP :
				distToOrigin -= 0.5f;
				if (distToOrigin<1.5f) distToOrigin = 1.5f;
				setCamera();  break;
		case GLUT_KEY_DOWN :
				distToOrigin += 0.5f;
				setCamera();  break;
		case GLUT_KEY_PAGE_UP :
				cameraHeight += 0.1f;
				setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
				cameraHeight -= 0.1f;
				setCamera();  break;
		case GLUT_KEY_F1 :
				detailMap=NONE; break;
		case GLUT_KEY_F2 :
				detailMap=DEPTH; break;
		case GLUT_KEY_F3 :
				detailMap=ORIENT; break;
		case GLUT_KEY_F4 :
				detailMap=REFLECT; break;
		}
}




void commonKey(unsigned char key, int x_, int y_) {
		
		switch (key) {
		case 'r':
				lightRotation = true; break;
		case ',':
				lightRotation = false;
				lightAngle += 0.1f;
				setLight(); break;
		case '.':
				lightRotation = false;
				lightAngle -= 0.1f;
				setLight(); break;
		case '1':
				materialModel = PHONG; break;
		case '2':
				materialModel = PHONG_TOON; break;
		case '3':
				materialModel = FULL_TOON; break;
		case '4':
				materialModel = X_TOON; break;
		case 'm':
				material = METAL; break;
		case 'p':
				material = PLASTIC; break;
		case '+':
				segmentation++;
				break;
		case '-':
				if (segmentation > 1){
					segmentation--;
				}
				break;
		case '/':
				rotationSpeed += 0.1; break;

		case '*':
				rotationSpeed -= 0.1; break;
		case 'f':
				if (inverse == false){
					inverse = true;
				}else{
					inverse = false;
				}
				break;
		}
}






////////////////////////////////////
//  Main method
////////////////////////////////////

void parseArgs(int argc, char **argv, char** objFileName, char** toonTexFileName){

		// default values
		*objFileName = "-";
		*toonTexFileName = "-";

		if (argc>1){
				int nbCar1 = strlen(argv[1]);

				if (argv[1][nbCar1-3]=='o' && argv[1][nbCar1-2]=='b' && argv[1][nbCar1-1]=='j'){
						*objFileName = argv[1];
				} else if (argv[1][nbCar1-3]=='b' && argv[1][nbCar1-2]=='m' && argv[1][nbCar1-1]=='p'){
						*toonTexFileName = argv[1];
				}

				if (argc>2){
						int nbCar2 = strlen(argv[2]);
						
						if (argv[2][nbCar2-3]=='o' && argv[2][nbCar2-2]=='b' && argv[2][nbCar2-1]=='j'){
								*objFileName = argv[2];					
						} else if (argv[2][nbCar2-3]=='b' && argv[2][nbCar2-2]=='m' && argv[2][nbCar2-1]=='p'){
								*toonTexFileName = argv[2];					
						}
				}
		}
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800, 600);
	glutCreateWindow("Humanoid");

	char *objFileName, *toonTexFileName;
	parseArgs(argc, argv, &objFileName, &toonTexFileName);
	initScene(objFileName, toonTexFileName);

	glutSpecialFunc(specialKey);
	glutKeyboardFunc(commonKey);

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);

	glutReshapeFunc(changeSize);

	glutMainLoop();

	return(0);
}
