#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <float.h>
#include <string.h>
#include "Vec3.h"

using namespace std;

////////////////////////////////
// Object mesh methods
////////////////////////////////

float znear=1.0, zfar=100.0;
vector<Vec3> vertices, normals;
vector< pair<int, int> > faceIdx1, faceIdx2, faceIdx3;
bool objectLoaded = false;


bool loadObj(const char * filename, bool normalize) {
	
  ifstream in(filename);
	char type;
	char s[256];
		
  if (!in.is_open()) {
			cout << "Using default object: unit sphere"  << endl;
    return false;
	}

	vertices.clear();
	normals.clear();
	faceIdx1.clear();
	faceIdx2.clear();
	faceIdx3.clear();
	
	while (!in.eof()) {
		if (in.peek() == 'v') {
			in >> type;
			
			if (in.peek() == 'n'){
				in >> type;
			
				float x, y, z;
				in>> x; in>> y; in>> z;
				normals.push_back(Vec3(x,y,z));
				
			}	else if (in.peek() != 't') {
				float x, y, z;
				in>> x; in>> y; in>> z;
				vertices.push_back(Vec3(x,y,z));				
			}
			in.getline(s, 256);
			
		}	else if (in.peek() == 'f') {
			in >> type;
			
			int vertex1, vertex2, vertex3;
			int normal1, normal2, normal3;
			char tempByte;
			
			// vertex 1
			in>> vertex1; --vertex1;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 1
			in>> normal1; --normal1;
			
			// vertex 2
			in>> vertex2; --vertex2;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 2
			in>> normal2; --normal2;
			
			// vertex 3
			in>> vertex3; --vertex3;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 3
			in>> normal3; --normal3;
			
			// Store face
			pair<int, int> idx1 (vertex1, normal1);
			pair<int, int> idx2 (vertex2, normal2);
			pair<int, int> idx3 (vertex3, normal3);

			faceIdx1.push_back(idx1);
			faceIdx2.push_back(idx2);
			faceIdx3.push_back(idx3);
			
			in.getline(s, 256);
		} else {
			in.getline(s, 256);
		}
	}	
	in.close();

	
	unsigned int nbVertices = vertices.size();	
	cout << "Object loaded: " << endl;
	cout << "\t Filename = " << filename << endl;
	cout << "\t #vertices = " << nbVertices << endl;

	// normalize object scale.
	if (normalize) {
		float minX=FLT_MAX;
		float maxX=-FLT_MAX;
		float minY=FLT_MAX;;
		float maxY=-FLT_MAX;
		float minZ=FLT_MAX;
		float maxZ=-FLT_MAX;

		// find bounding box coords
		for (unsigned int i=0 ; i<nbVertices ; i++){
			minX=(minX>vertices[i].x)?vertices[i].x:minX;				
			maxX=(maxX<vertices[i].x)?vertices[i].x:maxX;
			minY=(minY>vertices[i].y)?vertices[i].y:minY;				
			maxY=(maxY<vertices[i].y)?vertices[i].y:maxY;
			minZ=(minZ>vertices[i].z)?vertices[i].z:minZ;				
			maxZ=(maxZ<vertices[i].z)?vertices[i].z:maxZ;
		}

		// modify object vertices so that the maximum distance is unit size...
		Vec3 center (0.5*(maxX+minX), 0.5*(maxY+minY), 0.5*(maxZ+minZ));
		float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
	  float scaling=1.0/norm;
	
	  for (unsigned int i=0 ; i<nbVertices ; i++) {		  
	    vertices[i].x =(vertices[i].x - center.x)*scaling;
	    vertices[i].y =(vertices[i].y - center.y)*scaling;
	    vertices[i].z =(vertices[i].z - center.z)*scaling;
	  }
	}


	return true;			
}










////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint	texture;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;


int loadImage(const char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;		
    if ((file = fopen(filename, "rb"))==NULL) {
				return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
				cout << "Error reading width from " << filename << endl;
				return 0;
    }    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
				cout << "Error reading height from " << filename << endl;
				return 0;
    }    
		unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
				cout << "Error reading planes from " << filename << endl;
				return 0;
    }
    if (planes != 1) {
				cout << "Planes from " << filename << "is not 1: " << planes << endl;
				return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) { 
				cout << "Error reading bpp from " << filename << endl;
				return 0;
    }
    if (bpp != 24) {
				cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
				return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
				cout << "Error allocating memory for color-corrected image data" << endl;
				return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
				cout << "Error reading image data from " << filename << endl;
				return 0;
    }

		// reverse all of the colors. (bgr -> rgb) 
		char temp;                         
		for (i=0;i<size;i+=3) { 
				temp = image->data[i];
				image->data[i] = image->data[i+2];
				image->data[i+2] = temp;
    }
		
    return 1;
}


GLvoid loadGLTexture(const char* textureName, GLuint* textureIdx) {	
    // Load Texture
    Image *image;
    
    // allocate space for cube texture
    image = (Image *) malloc(sizeof(Image));
    if (image == NULL) {
				printf("Error allocating space for image");
				exit(0);
    }
    if (!loadImage(textureName, image)){
				if (!loadImage("defaultToonTex.bmp", image)) {
						cerr << "Default texture fot found : defaultToonTex.bmp" << endl;
						exit(1);
				} else {
						cout << "Using default texture: defaultToonTex.bmp" << endl;
				}
    } else {
				cout << "Texture Loaded: " << endl;
				cout << "\t Filename: " << textureName << endl;
				cout << "\t Size: " << image->sizeX << endl;
		} 

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP); 
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
};






////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;
float lightRadius = 5.0;
float lightHeight = 1.0;
Vec3 light (lightRadius, lightRadius, 0.0);

void setLight(){
		light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));
		GLfloat lightPosition[]= {light.x, light.y, light.z, 1.0f};
		glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
		glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
		glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
		glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
		glEnable(GL_LIGHT0);
}





/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 0.0f;
Vec3 camera(0.0, cameraHeight, distToOrigin);

void setCamera() {

		float x = distToOrigin*cos(cameraAngle);
		float y = cameraHeight;
		float z = distToOrigin*sin(cameraAngle);
		camera = Vec3(x, y, z);
		
		glLoadIdentity();
		gluLookAt(x, y, z,
							-x, y, -z,
							0.0f,1.0f,0.0f);
		setLight();
}




////////////////////////////////////////
//  Material methods
////////////////////////////////////////
const int PHONG=1, PHONG_TOON=2, FULL_TOON=3, X_TOON=4;
int materialModel = PHONG;

const int PLASTIC=0, METAL=1;
int material = PLASTIC;

const int NONE=0, DEPTH=1, ORIENT=2, REFLECT=3;
int detailMap = NONE;



// plastic material parameters
  GLfloat plastic_ambient[] = { 0.15F,0.0F,0.3F,1.0F };
  GLfloat plastic_diffuse[] = { 0.1F,0.1F,0.3F,1.0F };
  GLfloat plastic_specular[] = { 0.9F,0.9F,1.0F,1.0F };
  GLfloat plastic_shininess[] = { 250.0F };
  GLfloat plastic_emission[] = {0.0F,0.0F,0.0F,0.0F};
  Vec3 plastic_shine= Vec3(0.9F,0.9F,1.0F);
  int S_plastic=250;


// metallic material parameters
  GLfloat metal_ambient[] = { 0.15F,0.0F,0.3F,1.0F };
  GLfloat metal_diffuse[] = { 0.1F,0.1F,0.3F,1.0F };
  GLfloat metal_specular[] = { 0.3F,0.1F,0.6F,1.0F };
  GLfloat metal_shininess[] = { 20.0F };
  GLfloat metal_emission[] = {0.0F,0.0F,0.0F,0.0F};
  Vec3 metal_shine= Vec3(0.2F,0.2F,0.6F);
  int S_metal=20;
  
// metallic material parameters
  GLfloat none_ambient[] = { 0.0F,0.0F,0.0F,1.0F };
  GLfloat none_diffuse[] = { 0.0F,0.0F,0.0F,1.0F };
  GLfloat none_specular[] = { 0.0F,0.0F,0.0F,1.0F };
  GLfloat none_shininess[] = { 2.0F };
  GLfloat none_emission[] = {0.0F,0.0F,0.0F,0.0F};
  Vec3 none_shine=Vec3(0.1F,0.0F,0.2F);
  int S=2;
    


// toon shading parameters
  Vec3 color_init = Vec3(0.3F,0.1F,0.6F);
  Vec3 Shine=Vec3(0.1F,0.0F,0.2F);
  int nb_interval=2;
  bool new_shine=false;

  
// X-Toon shading parameters
  float max_sommet_cam=10;
  bool flip=false;


void setMaterial(){
  
switch(material){
  	  case PLASTIC:
 		glMaterialfv(GL_FRONT,GL_AMBIENT,plastic_ambient);
  		glMaterialfv(GL_FRONT,GL_DIFFUSE,plastic_diffuse);
	  	glMaterialfv(GL_FRONT,GL_SPECULAR,plastic_specular);
 		glMaterialfv(GL_FRONT,GL_SHININESS,plastic_shininess);
	//	glMaterialfv(GL_FRONT,GL_EMISSION,plastic_emission);
		S=S_plastic;
		Shine=plastic_shine;
		
	
		break;
 	   case METAL:
 		glMaterialfv(GL_FRONT,GL_AMBIENT,metal_ambient);
 	 	glMaterialfv(GL_FRONT,GL_DIFFUSE,metal_diffuse);
 	 	glMaterialfv(GL_FRONT,GL_SPECULAR,metal_specular);
 		glMaterialfv(GL_FRONT,GL_SHININESS,metal_shininess);
	//	glMaterialfv(GL_FRONT,GL_EMISSION,metal_emission);
		S=S_metal;
		Shine=metal_shine;
			
		break;
	    default:
 		glMaterialfv(GL_FRONT,GL_AMBIENT,none_ambient);
  		glMaterialfv(GL_FRONT,GL_DIFFUSE,none_diffuse);
	  	glMaterialfv(GL_FRONT,GL_SPECULAR,none_specular);
 		glMaterialfv(GL_FRONT,GL_SHININESS,none_shininess);
	//	glMaterialfv(GL_FRONT,GL_EMISSION,none_emission);
		Shine=none_shine;
		S=2;
 	 }




  switch(materialModel){
  case PHONG:
  	glDisable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);

  	break;
	 

  case PHONG_TOON:
    	//glDisable(GL_COLOR_MATERIAL);


 	glMaterialf(GL_FRONT,GL_AMBIENT,0.0f);
  	glMaterialf(GL_FRONT,GL_DIFFUSE,0.0f);
	 
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glColorMaterial(GL_FRONT,GL_DIFFUSE);
	break;
  
	 
  case FULL_TOON:

	 
	 glEnable(GL_COLOR_MATERIAL);
	 glDisable(GL_LIGHTING);
	 glDisable(GL_TEXTURE_2D);
	

	 break;
	 
  case X_TOON:
  	glEnable(GL_LIGHTING);
	glDisable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
   	glBindTexture(GL_TEXTURE_2D, texture);
  
        break;
  }
  
  

	
}

void unsetMaterial(){
 		glMaterialfv(GL_FRONT,GL_AMBIENT,none_ambient);
  		glMaterialfv(GL_FRONT,GL_DIFFUSE,none_diffuse);
	  	glMaterialfv(GL_FRONT,GL_SPECULAR,none_specular);
 		glMaterialfv(GL_FRONT,GL_SHININESS,none_shininess);
	//	glMaterialfv(GL_FRONT,GL_EMISSION,none_emission);	
		
}


void getColor(const Vec3& vertex, const Vec3& normal, Vec3& color){		
		
	float diff=(light-vertex).normalized()|normal;
	color=Vec3(0,0,0);
	if (diff<0)
	  diff=0;
	  
   	float Spec=0;
	if (materialModel==FULL_TOON)
	{
	  Vec3 H=((camera-vertex).normalized()+(light-vertex).normalized()).normalized();
	  Spec=pow((normal|H),S);
	  if (Spec>0.6)
	  {
	    Spec=1;
	    if (new_shine) color+=Shine;
	  }
	  else Spec=0;	  
	  
	  
	  
	}

	  
	int zog=(int)((diff+Spec)*nb_interval);
	
	color+=((float(zog+1))/nb_interval)*color_init;	
	//std::cout<<diff<<" "<<Spec<<" "<<zog<<endl;


		
		
		
}


void getTexCoord(const Vec3& vertex, const Vec3& normal, float& toneCoord, float& detailCoord){

        float diff=(light-vertex).normalized()|normal;
	
	if (diff<0)
	  diff=0;
	toneCoord=diff;
	
	switch(detailMap)
	{
	  case DEPTH:
	  {
	    float dist_a=(camera-vertex).norm();
	    if (flip) dist_a=1-dist_a;
	    detailCoord=dist_a/max_sommet_cam;
	    break;
	  }
	  case ORIENT:
	  {
	    float dist_a=(camera-vertex).normalized()|normal;
	    if (flip) dist_a=1-dist_a;
	     
	    detailCoord=dist_a/max_sommet_cam;
	    break;
	  }
	    break;
	  case REFLECT:
	  {
	     Vec3 H=((camera-vertex).normalized()+(light-vertex).normalized()).normalized();
	     float Spec=pow((normal|H),nb_interval); 
	     //std::cout<<Spec<<endl;
	     if (flip) detailCoord=1-Spec;
	     else detailCoord=Spec;
	  } break;
	
  	  case NONE:default:
	    detailCoord=0.0f;
	    if (flip) detailCoord=1.0f;
	}


  		
		
}



/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
float rotationSpeed = 1.0;
bool lightRotation = true;

void updateAnimationParam(){
		// compute a time parameter based on a 
		float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
		float t = newTime-oldTime;
		oldTime = newTime;
		
		if(lightRotation)
		{
		  lightAngle+=t*rotationSpeed;
		  setLight();
		}
		

		
}

////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene(const char* objFile, const char* toonTexFile) {
		// init scanline
		glClearColor(0.3, 0.3, 0.3, 1.0f);				
		glClearDepth(1.0);
		glDepthFunc(GL_LESS);												
		glEnable(GL_DEPTH_TEST);										
		glShadeModel(GL_SMOOTH);										
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	
		glLineWidth(2.0f);													
		glPointSize(4.0f);

		// init lighting
		setLight();
		glEnable(GL_LIGHTING);

		// init textures
		loadGLTexture(toonTexFile, &texture);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		// init object
		glEnable(GL_NORMALIZE);
		objectLoaded = loadObj(objFile, true);
}


void changeSize(int w, int h)	{

		// Prevent a divide by zero, when window is too short
		// (you cant make a window of zero width).
		if(h == 0)
				h = 1;
		
		float ratio = 1.0f * w / h;
		// Reset the coordinate system before modifying
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		
		// Set the viewport to be the entire window
		glViewport(0, 0, w, h);
		
		// Set the clipping volume
		gluPerspective(45,ratio,znear,zfar);
		glMatrixMode(GL_MODELVIEW);
		setCamera();
}










///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////


void drawSphere(){

		setMaterial();
		
		glPushMatrix();
		
		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;
				
				// draw one strip
				Vec3 color1, color2;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;
						
						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						glColor3fv(color1);
						glNormal3fv(pos1);
						glVertex3fv(pos1);
						
						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						glColor3fv(color2);
						glNormal3fv(pos2);
						glVertex3fv(pos2);
				}
				glEnd();
		}
		glPopMatrix();

		unsetMaterial();

}


void Do_switch(pair<int, int> norm_vert)
{


	Vec3 color;
	Vec3 normal=normals[norm_vert.second].normalized();
		  


	switch(materialModel){
  	  case PHONG_TOON: 
	    glNormal3fv(normal);
	  case FULL_TOON:
	    getColor(vertices[norm_vert.first],normal,color);
	    //std::cout<<color.x<<" "<<color.y<<" "<<color.z<<endl;
	    glColor3fv(color);
	    break;
	  case X_TOON:
	    float x_coord, y_coord;  
	    getTexCoord(vertices[norm_vert.first],normal,x_coord, y_coord);
	    glTexCoord2f(x_coord, y_coord);
	  default:
	    glNormal3fv(normal);
	    break;
	}
	glVertex3fv(vertices[norm_vert.first]);
}


void drawObject(){

		setMaterial();
		
		glPushMatrix();
 		glScalef(10.0, 10.0, 10.0);
		
		Vec3 color;
		
		glBegin(GL_TRIANGLES);
		for (unsigned int i=0 ; i<faceIdx1.size() ; i++) {
		  //triangle i
		  Do_switch(faceIdx1[i]);
		  Do_switch(faceIdx2[i]);
		  Do_switch(faceIdx3[i]);

		}
		
		
		glEnd();
		
		glPopMatrix();

		unsetMaterial();
}


void renderScene(void) {
		// clear buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		// draw objects
		if (objectLoaded){
				drawObject();
		} else {
				drawSphere();
		}

		// update animation parameters
		updateAnimationParam();

		// display rendering
		glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////

    
void specialKey(int key, int x_, int y_) {
		
		switch (key) {
		case GLUT_KEY_LEFT :
				cameraAngle += 0.04f;
				setCamera();  break;
		case GLUT_KEY_RIGHT :
				cameraAngle -=0.04f;
				setCamera();  break;
		case GLUT_KEY_UP :
				distToOrigin -= 0.5f;
				if (distToOrigin<1.5f) distToOrigin = 1.5f;
				setCamera();  break;
		case GLUT_KEY_DOWN :
				distToOrigin += 0.5f;
				setCamera();  break;
		case GLUT_KEY_PAGE_UP :
				cameraHeight += 0.1f;
				setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
				cameraHeight -= 0.1f;
				setCamera();  break;
		case GLUT_KEY_F1 :
				detailMap=NONE; break;
		case GLUT_KEY_F2 :
				detailMap=DEPTH; break;
		case GLUT_KEY_F3 :
				detailMap=ORIENT; break;
		case GLUT_KEY_F4 :
				detailMap=REFLECT; break;
		}
}




void commonKey(unsigned char key, int x_, int y_) {
		
		switch (key) {
		case 'r':
				lightRotation = !lightRotation; break;
		case '<':
				lightRotation = false;
				lightAngle += 0.1f;
				setLight(); break;
		case '>':
				lightRotation = false;
				lightAngle -= 0.1f;
				setLight(); break;
		case '1':
				materialModel = PHONG; break;
		case '2':
				materialModel = PHONG_TOON; break;
		case '3':
				materialModel = FULL_TOON; break;
		case '4':
				materialModel = X_TOON; break;
		case 'm':
				material = METAL;//std::cout<<"METAL"<<endl; 
				break;
		case 'p':
				material = PLASTIC;//std::cout<<"PLASTIC"<<endl;  
				break;		
		case 'o':
				material = 5;//std::cout<<"PLASTIC"<<endl;  
				break;		
		case 'l':
				new_shine=!new_shine; std::cout<<"new_shine "<<new_shine<<endl;
				break;
		case 'f':
				flip=!flip;
				break;
		case '+':	
				nb_interval++;   
				max_sommet_cam*=1.3;
				std::cout<<"max_sommet_cam: "<<max_sommet_cam<<" nb_interval: "<<nb_interval<<endl;
				
				break;
		case '-':
				nb_interval--;
				if (nb_interval==0)nb_interval++;
				//std::cout<<"nb_interval: "<<nb_interval<<endl;
				max_sommet_cam/=2;
				if(max_sommet_cam<0.1f)max_sommet_cam=0.1f;
				std::cout<<"max_sommet_cam: "<<max_sommet_cam<<" nb_interval: "<<nb_interval<<endl;
				
				break;
		case '/': 
				rotationSpeed += 0.1; break;
		case '*': 
				rotationSpeed -= 0.1; break;
		case '0': 
				rotationSpeed = 0; break;		
		}
}






////////////////////////////////////
//  Main method
////////////////////////////////////

void parseArgs(int argc, char **argv, char** objFileName, char** toonTexFileName){

		// default values
		*objFileName = "-";
		*toonTexFileName = "-";

		if (argc>1){
				int nbCar1 = strlen(argv[1]);

				if (argv[1][nbCar1-3]=='o' && argv[1][nbCar1-2]=='b' && argv[1][nbCar1-1]=='j'){
						*objFileName = argv[1];
				} else if (argv[1][nbCar1-3]=='b' && argv[1][nbCar1-2]=='m' && argv[1][nbCar1-1]=='p'){
						*toonTexFileName = argv[1];
				}

				if (argc>2){
						int nbCar2 = strlen(argv[2]);
						
						if (argv[2][nbCar2-3]=='o' && argv[2][nbCar2-2]=='b' && argv[2][nbCar2-1]=='j'){
								*objFileName = argv[2];					
						} else if (argv[2][nbCar2-3]=='b' && argv[2][nbCar2-2]=='m' && argv[2][nbCar2-1]=='p'){
								*toonTexFileName = argv[2];					
						}
				}
		}
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800, 600);
	glutCreateWindow("Humanoid");

	char *objFileName, *toonTexFileName;
	parseArgs(argc, argv, &objFileName, &toonTexFileName);
	initScene(objFileName, toonTexFileName);

	glutSpecialFunc(specialKey);
	glutKeyboardFunc(commonKey);

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);

	glutReshapeFunc(changeSize);

	glutMainLoop();

	return(0);
}
