#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <float.h>
#include <string.h>
#include "Vec3.h"

using namespace std;

////////////////////////////////
// Object mesh methods
////////////////////////////////

float znear=1.0, zfar=100.0;
vector<Vec3> vertices, normals;
typedef vector< pair<int, int> > FaceIDX;

FaceIDX faceIdx1, faceIdx2, faceIdx3;



bool objectLoaded = false;


bool loadObj(const char * filename, bool normalize) {
	
  ifstream in(filename);
	char type;
	char s[256];
		
  if (!in.is_open()) {
    cout << "Using default object: unit sphere"  << endl;
    return false;
  }

	vertices.clear();
	normals.clear();
	faceIdx1.clear();
	faceIdx2.clear();
	faceIdx3.clear();
	
	while (!in.eof()) {
		if (in.peek() == 'v') {
			in >> type;
			
			if (in.peek() == 'n'){
				in >> type;
			
				float x, y, z;
				in>> x; in>> y; in>> z;
				normals.push_back(Vec3(x,y,z));
				
			}	else if (in.peek() != 't') {
				float x, y, z;
				in>> x; in>> y; in>> z;
				vertices.push_back(Vec3(x,y,z));				
			}
			in.getline(s, 256);
			
		}	else if (in.peek() == 'f') {
			in >> type;
			
			int vertex1, vertex2, vertex3;
			int normal1, normal2, normal3;
			char tempByte;
			
			// vertex 1
			in>> vertex1; --vertex1;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 1
			in>> normal1; --normal1;
			
			// vertex 2
			in>> vertex2; --vertex2;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 2
			in>> normal2; --normal2;
			
			// vertex 3
			in>> vertex3; --vertex3;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 3
			in>> normal3; --normal3;
			
			// Store face
			pair<int, int> idx1 (vertex1, normal1);
			pair<int, int> idx2 (vertex2, normal2);
			pair<int, int> idx3 (vertex3, normal3);

			faceIdx1.push_back(idx1);
			faceIdx2.push_back(idx2);
			faceIdx3.push_back(idx3);
			
			in.getline(s, 256);
		} else {
			in.getline(s, 256);
		}
	}	
	in.close();

	
	unsigned int nbVertices = vertices.size();	
	cout << "Object loaded: " << endl;
	cout << "\t Filename = " << filename << endl;
	cout << "\t #vertices = " << nbVertices << endl;

	// normalize object scale.
	if (normalize) {
		float minX=FLT_MAX;
		float maxX=-FLT_MAX;
		float minY=FLT_MAX;;
		float maxY=-FLT_MAX;
		float minZ=FLT_MAX;
		float maxZ=-FLT_MAX;

		// find bounding box coords
		for (unsigned int i=0 ; i<nbVertices ; i++){
			minX=(minX>vertices[i].x)?vertices[i].x:minX;				
			maxX=(maxX<vertices[i].x)?vertices[i].x:maxX;
			minY=(minY>vertices[i].y)?vertices[i].y:minY;				
			maxY=(maxY<vertices[i].y)?vertices[i].y:maxY;
			minZ=(minZ>vertices[i].z)?vertices[i].z:minZ;				
			maxZ=(maxZ<vertices[i].z)?vertices[i].z:maxZ;
		}

		// modify object vertices so that the maximum distance is unit size...
		Vec3 center (0.5*(maxX+minX), 0.5*(maxY+minY), 0.5*(maxZ+minZ));
		float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
	  float scaling=1.0/norm;
	
	  for (unsigned int i=0 ; i<nbVertices ; i++) {		  
	    vertices[i].x =(vertices[i].x - center.x)*scaling;
	    vertices[i].y =(vertices[i].y - center.y)*scaling;
	    vertices[i].z =(vertices[i].z - center.z)*scaling;
	  }
	}


	return true;			
}










////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint	texture;
bool flip = false;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;


int loadImage(const char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;		
    if ((file = fopen(filename, "rb"))==NULL) {
				return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
				cout << "Error reading width from " << filename << endl;
				return 0;
    }    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
				cout << "Error reading height from " << filename << endl;
				return 0;
    }    
		unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
				cout << "Error reading planes from " << filename << endl;
				return 0;
    }
    if (planes != 1) {
				cout << "Planes from " << filename << "is not 1: " << planes << endl;
				return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) { 
				cout << "Error reading bpp from " << filename << endl;
				return 0;
    }
    if (bpp != 24) {
				cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
				return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
				cout << "Error allocating memory for color-corrected image data" << endl;
				return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
				cout << "Error reading image data from " << filename << endl;
				return 0;
    }

		// reverse all of the colors. (bgr -> rgb) 
		char temp;                         
		for (i=0;i<size;i+=3) { 
				temp = image->data[i];
				image->data[i] = image->data[i+2];
				image->data[i+2] = temp;
    }
		
    return 1;
}


GLvoid loadGLTexture(const char* textureName, GLuint* textureIdx) {	
    // Load Texture
    Image *image;
    
    // allocate space for cube texture
    image = (Image *) malloc(sizeof(Image));
    if (image == NULL) {
				printf("Error allocating space for image");
				exit(0);
    }
    if (!loadImage(textureName, image)){
				if (!loadImage("defaultToonTex.bmp", image)) {
						cerr << "Default texture fot found : defaultToonTex.bmp" << endl;
						exit(1);
				} else {
						cout << "Using default texture: defaultToonTex.bmp" << endl;
				}
    } else {
				cout << "Texture Loaded: " << endl;
				cout << "\t Filename: " << textureName << endl;
				cout << "\t Size: " << image->sizeX << endl;
		} 

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP); 
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
};






////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };

GLfloat lightAmbientMaterial[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuseMaterial[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecularMaterial[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLint   lightShineMaterial[]    = { 50 };

float lightAngle = 0.0f;
float lightRadius = 5.0;
float lightHeight = 1.0;
Vec3 light (lightRadius, lightRadius, 0.0);

void setLight(){
		light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));
		GLfloat lightPosition[]= {light.x, light.y, light.z, 1.0f};
 		glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient); 
 		glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse); 
 		glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular); 

 		glMaterialfv(GL_FRONT, GL_AMBIENT, lightAmbientMaterial); 
 		glMaterialfv(GL_FRONT, GL_DIFFUSE, lightDiffuseMaterial); 
 		glMaterialfv(GL_FRONT, GL_SPECULAR, lightSpecularMaterial); 
 		glMaterialiv(GL_FRONT, GL_SHININESS, lightShineMaterial); 

 		glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
		glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
		glEnable(GL_LIGHT0);
}





/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 0.0f;
Vec3 camera(0.0, cameraHeight, distToOrigin);

void setCamera() {

		float x = distToOrigin*cos(cameraAngle);
		float y = cameraHeight;
		float z = distToOrigin*sin(cameraAngle);
		camera = Vec3(x, y, z);
		
		glLoadIdentity();
		gluLookAt(x, y, z,			-x, y, -z,
							0.0f,1.0f,0.0f);
		setLight();
}




////////////////////////////////////////
//  Material methods
////////////////////////////////////////
const int PHONG=1, PHONG_TOON=2, FULL_TOON=3, X_TOON=4;
int materialModel = PHONG;

const int PLASTIC=0, METAL=1;
int material = PLASTIC;

const int NONE=0, DEPTH=1, ORIENT=2, REFLECT=3;
int detailMap = NONE;

// plastic material parameters
GLfloat PLASTIC_lightAmbient[4]  = { 0.1f, 0.1f, 0.1f, 1.0f };
GLfloat PLASTIC_lightDiffuse[4]  = { 0.8f, 1.0f, 0.8f, 1.0f };
GLfloat PLASTIC_lightSpecular[4] = { 0.8f, 0.8f, 0.8f, 1.0f };
GLint PLASTIC_shininess[]     = { 100  }; // reflection de material (Question 3)

// metallic material parameters
GLfloat METAL_lightAmbient[4]  = { 0.1f, 0.1f, 0.1f, 1.0f };
GLfloat METAL_lightDiffuse[4]  = { 0.4f, 0.0f, 0.8f, 1.0f };
GLfloat METAL_lightSpecular[4] = { 0.5f, 0.0f, 0.8f, 1.0f };
GLint METAL_shininess[]     = { 10 }; // reflection de material (Question 3)


unsigned int noColors = 3; // nombre des couleurs utilisee (Question 4)


// toon shading parameters
float shininess = 10;  // exponant pour influencer le reflet (Question 5, 9)

// X-Toon shading parameters
float vitesse = 1.0; // change la vitesse par rapport de l'orientation (Question 8)
int minDist   = 3;   // distance minimal dont la detail est 1 (Question 7)
int maxDist   = 13;  // distance maximal dont la detail est 0 (Question 7)


void setMaterial(){	

  switch(materialModel) {
    case PHONG_TOON:
      glColorMaterial ( GL_FRONT_AND_BACK, GL_DIFFUSE ) ;
      glEnable ( GL_COLOR_MATERIAL ) ;
      break;
     
  case FULL_TOON:
      glDisable(GL_LIGHTING);
      break;
  case X_TOON:
      glEnable(GL_TEXTURE_2D);
      break;
  }

  switch(material) {
  case PLASTIC:
    memcpy(lightAmbientMaterial,  PLASTIC_lightAmbient, sizeof(GLfloat)*4);
    memcpy(lightSpecularMaterial, PLASTIC_lightSpecular,sizeof(GLfloat)*4);
    memcpy(lightDiffuseMaterial,  PLASTIC_lightDiffuse, sizeof(GLfloat)*4);
    memcpy(lightShineMaterial,    PLASTIC_shininess,    sizeof(GLint));

    break;
  case METAL:
    memcpy(lightAmbientMaterial,  METAL_lightAmbient, sizeof(GLfloat)*4);
    memcpy(lightSpecularMaterial, METAL_lightSpecular,sizeof(GLfloat)*4);
    memcpy(lightDiffuseMaterial,  METAL_lightDiffuse, sizeof(GLfloat)*4);
    memcpy(lightShineMaterial,    METAL_shininess,    sizeof(GLint));
    break;
  }

}

void unsetMaterial(){		
  glDisable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHTING);
  glDisable(GL_TEXTURE_2D);
}


inline
float SegmentIt(float toSegment) {
  return (floor(toSegment * noColors)+0.5) / noColors;  // classifier l'intensite par rapport de nombre couleurs (Question 4,5)
}

float LinearNormalize(float value) {
  if (value >= maxDist) // toutes valeurs superieur a maxDist sont associe a un detail 0
    return 0.0;
  if (value <= minDist) // toutes valeurs inferieur a minDist sont associe a un detail 1
    return 1.0;
  
  return (maxDist - value)/(maxDist - minDist); // fonction lineaire entre minDist et maxDist
}


// Question 4, 5
void getColor(const Vec3& vertex, const Vec3& normal, Vec3& color){		

  Vec3 direction( (light - vertex).normalized() ); // vector de la source  l'objet courant
  Vec3 eye( (camera - vertex).normalized() );      // vector de la camera  l'objet courant

  float intensite = dot(normal, direction); // default
  if (intensite < 0) intensite = 0;
  intensite = SegmentIt(intensite);
  
  float spec      = 0; // default

  if (materialModel == FULL_TOON) {                                // Question 5
    Vec3 h = (direction + eye).normalized();
    spec = pow(dot(normal,h),shininess);
    if (spec < 0.3) // tronque  0.5
      spec = 0;
  }

  
  switch(material) {
  case PLASTIC:
    color.x = PLASTIC_lightDiffuse[0] * intensite + PLASTIC_lightSpecular[0] * spec;
    color.y = PLASTIC_lightDiffuse[1] * intensite + PLASTIC_lightSpecular[1] * spec;
    color.z = PLASTIC_lightDiffuse[2] * intensite + PLASTIC_lightSpecular[2] * spec;
    break;
    
  case METAL:
    color.x = METAL_lightSpecular[0] * intensite + METAL_lightSpecular[0] * spec;
    color.y = METAL_lightSpecular[1] * intensite + METAL_lightSpecular[1] * spec;
    color.z = METAL_lightSpecular[2] * intensite + METAL_lightSpecular[2] * spec;
    break;
  }
  
}


// Question 6 - 9 "X_TOON"
void getTexCoord(const Vec3& vertex, const Vec3& normal, float& toneCoord, float& detailCoord){
  Vec3 direction( (light - vertex).normalized() );
  Vec3 eye( (camera - vertex).normalized() );

  toneCoord = dot(normal.normalized(), direction);

  switch(detailMap){
  case NONE:                   // Question 6
    detailCoord = 1.0;
    break;
  case DEPTH:                  // Question 7
    detailCoord = LinearNormalize((camera - vertex).norm()); // distance entre l'objet et la camera
    break;
  case ORIENT:                 // Question 8
    detailCoord = dot(  eye ,normal.normalized()  ) * vitesse ; // = cos (angle(norm, eye))
    break;
  case REFLECT:                // Question 9
    Vec3 h = (direction + eye).normalized();
    detailCoord = pow( dot( normal,h ),shininess );
    break;
  }

  if (flip) detailCoord= 1 - detailCoord;
}



/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
float rotationSpeed = 1.0;
bool lightRotation = true;

void updateAnimationParam(){
		// compute a time parameter based on a 
		float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
		float t = newTime-oldTime;
		oldTime = newTime;
		
		// update a rotation parameter
		if (lightRotation)
		  lightAngle += t * rotationSpeed;
		setLight();
		
}

////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene(const char* objFile, const char* toonTexFile) {
		// init scanline
		glClearColor(0.3, 0.3, 0.3, 1.0f);				
		glClearDepth(1.0);
		glDepthFunc(GL_LESS);												
		glEnable(GL_DEPTH_TEST);										
		glShadeModel(GL_SMOOTH);										
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	
		glLineWidth(2.0f);													
		glPointSize(4.0f);

		// init lighting
		setLight();
		glEnable(GL_LIGHTING);

		// init textures
		loadGLTexture(toonTexFile, &texture);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		// init object
		glEnable(GL_NORMALIZE);
		objectLoaded = loadObj(objFile, true);
}


void changeSize(int w, int h)	{

		// Prevent a divide by zero, when window is too short
		// (you cant make a window of zero width).
		if(h == 0)
				h = 1;
		
		float ratio = 1.0f * w / h;
		// Reset the coordinate system before modifying
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		
		// Set the viewport to be the entire window
		glViewport(0, 0, w, h);
		
		// Set the clipping volume
		gluPerspective(45,ratio,znear,zfar);
		glMatrixMode(GL_MODELVIEW);
		setCamera();
}










///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////

void drawSphere(){

		setMaterial();
		
		glPushMatrix();
		
		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;
				
				// draw one strip
				Vec3 color1, color2;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;
						
						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						glColor3fv(color1);
						glNormal3fv(pos1);
						glVertex3fv(pos1);
						
						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						glColor3fv(color2);
						glNormal3fv(pos2);
						glVertex3fv(pos2);
				}
				glEnd();
		}
		glPopMatrix();

		unsetMaterial();

}


void drawObject(){

		setMaterial();
		
		glPushMatrix();
 		glScalef(10.0, 10.0, 10.0);
		
		glBegin(GL_TRIANGLES);

		FaceIDX* faceIdxA[3] = {&faceIdx1, &faceIdx2, &faceIdx3};

		// Load each vertex and normal for every single triangle
		for(int i = 0; i < faceIdx1.size(); ++i) { // using faceIdx1 instead of vertices.size for speedup ;)

		  Vec3 color;
		  float detail,tone;

		  for (int idxNo = 0; idxNo < 3; idxNo++) {   // pour faceIdx1 - faceIdx3

		    FaceIDX* face = faceIdxA[idxNo];
		    
		    switch(materialModel) {
		    case PHONG_TOON:
		    case FULL_TOON:
		      // on utilise notre algorithme de coloration pour calculer le couleur d'objet
		      getColor(vertices[(*face)[i].first], normals[(*face)[i].second],color);
		      glColor3fv(color);
		      break;
		    case X_TOON:
		      // on utilise notre algoritme pour plaquer le texture
		      getTexCoord(vertices[(*face)[i].first], normals[(*face)[i].second],tone,detail);
		      glTexCoord2f(tone,detail);
		      break;
		    }
		    normals[(*face)[i].second].glNormal();
		    vertices[(*face)[i].first].glVertex();
		    
		  }
		}
		

		glEnd();
		
		glPopMatrix();

		unsetMaterial();
}


void renderScene(void) {
		// clear buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		// draw objects
		if (objectLoaded){
				drawObject();
		} else {
				drawSphere();
		}

		// update animation parameters
		updateAnimationParam();

		// display rendering
		glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////

    
void specialKey(int key, int x_, int y_) {
		
		switch (key) {
		case GLUT_KEY_LEFT :
				cameraAngle += 0.04f;
				setCamera();  break;
		case GLUT_KEY_RIGHT :
				cameraAngle -=0.04f;
				setCamera();  break;
		case GLUT_KEY_UP :
				distToOrigin -= 0.5f;
				if (distToOrigin<1.5f) distToOrigin = 1.5f;
				setCamera();  break;
		case GLUT_KEY_DOWN :
				distToOrigin += 0.5f;
				setCamera();  break;
		case GLUT_KEY_PAGE_UP :
				cameraHeight += 0.1f;
				setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
				cameraHeight -= 0.1f;
				setCamera();  break;
		case GLUT_KEY_F1 :
        		        std::cout << " -> NONE\n";
				detailMap=NONE; break;
		case GLUT_KEY_F2 :
        		        std::cout << " -> DEPTH\n";
				detailMap=DEPTH; break;
		case GLUT_KEY_F3 :
        		        std::cout << " -> ORIENT\n";
				detailMap=ORIENT; break;
		case GLUT_KEY_F4 :
        		        std::cout << " -> REFLECT\n";
				detailMap=REFLECT; break;
		}
}




void commonKey(unsigned char key, int x_, int y_) {
		
		switch (key) {
		case 'r':
				lightRotation = true; break;
		case ',':
				lightRotation = false;
				lightAngle += 0.1f;
				setLight(); break;
		case '.':
				lightRotation = false;
				lightAngle -= 0.1f;
				setLight(); break;
		case '1':
        		        std::cout << "PHONG\n";
				materialModel = PHONG; break;
		case '2':
        		        std::cout << "PHONG_TOON\n";
				materialModel = PHONG_TOON; break;
		case '3':
        		        std::cout << "FULL_TOON\n";
				materialModel = FULL_TOON; break;
		case '4':
        		        std::cout << "X_TOON\n";
				materialModel = X_TOON; break;
		case 'm':
				material = METAL; break;
		case 'p':
				material = PLASTIC; break;
		case 's':       
				shininess+=0.1; 
				std::cout << "shininess = " << shininess << "\n";
				break;
		       
		case 'S':       
		                shininess-=0.1;
				std::cout << "shininess = " << shininess << "\n";
				break;

		case 'd':
		                unsetMaterial(); break;
		case 'f':       flip = !flip; break;
		case '+':
		                switch(materialModel) {
				case X_TOON:
				              switch(detailMap) {                   // nested switch, je suis desolee...
					      case DEPTH:
						std::cout << "Distance maximale = " << ++maxDist << "\n";
						break;
					      case ORIENT:
						vitesse = vitesse + vitesse/100;
						std::cout << "Vitesse = " << vitesse << "\n";
						break;
					      case REFLECT:
						shininess+=0.1;
						std::cout << "Shininess = " << shininess << "\n";
						break;
					      case NONE: /* rien particuliere */
						break;
					      }

				  break;
				case PHONG_TOON:
				case FULL_TOON:
				  std::cout << "Nombre couleurs = " << ++noColors <<"\n";
				  break;
				}
				break;

		case '-':
		                switch(materialModel) {
				case X_TOON:
				              switch(detailMap) {
					      case DEPTH:
						if (maxDist > minDist)
						  --maxDist;
						std::cout << "Distance maximale = " << maxDist << "\n";
						break;
					      case ORIENT:
						vitesse = vitesse - vitesse/111.1111;
						std::cout << "Vitesse = " << vitesse << "\n";
						break;
					      case REFLECT:
						if (shininess > 0.1)
						  shininess-=0.1;
						std::cout << "Shininess = " << shininess << "\n";
						break;
					      case NONE: /* rien particuliere */
						break;
					      }

				  break;
				case PHONG_TOON:
				case FULL_TOON:
				  std::cout << "Nombre couleurs = " << --noColors <<"\n";
				  break;
				}

				break;
		case '/': 
				rotationSpeed += 0.1; break;
		case '*': 
				rotationSpeed -= 0.1; break;
		}
}






////////////////////////////////////
//  Main method
////////////////////////////////////

void parseArgs(int argc, char **argv, char** objFileName, char** toonTexFileName){

		// default values
		*objFileName = "-";
		*toonTexFileName = "-";

		if (argc>1){
				int nbCar1 = strlen(argv[1]);

				if (argv[1][nbCar1-3]=='o' && argv[1][nbCar1-2]=='b' && argv[1][nbCar1-1]=='j'){
						*objFileName = argv[1];
				} else if (argv[1][nbCar1-3]=='b' && argv[1][nbCar1-2]=='m' && argv[1][nbCar1-1]=='p'){
						*toonTexFileName = argv[1];
				}

				if (argc>2){
						int nbCar2 = strlen(argv[2]);
						
						if (argv[2][nbCar2-3]=='o' && argv[2][nbCar2-2]=='b' && argv[2][nbCar2-1]=='j'){
								*objFileName = argv[2];					
						} else if (argv[2][nbCar2-3]=='b' && argv[2][nbCar2-2]=='m' && argv[2][nbCar2-1]=='p'){
								*toonTexFileName = argv[2];					
						}
				}
		}
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800, 600);
	glutCreateWindow("Humanoid");

	char *objFileName, *toonTexFileName;
	parseArgs(argc, argv, &objFileName, &toonTexFileName);
	initScene(objFileName, toonTexFileName);

	glutSpecialFunc(specialKey);
	glutKeyboardFunc(commonKey);

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);

	glutReshapeFunc(changeSize);

	glutMainLoop();

	return(0);
}
