#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <float.h>
#include <string.h>
#include "Vec3.h"

using namespace std;

////////////////////////////////
// Object mesh methods
////////////////////////////////

float znear=1.0, zfar=100.0;
vector<Vec3> vertices, normals;
vector< pair<int, int> > faceIdx1, faceIdx2, faceIdx3;
bool objectLoaded = false;


bool loadObj(const char * filename, bool normalize) {
	
  ifstream in(filename);
	char type;
	char s[256];
		
  if (!in.is_open()) {
			cout << "Using default object: unit sphere"  << endl;
    return false;
	}

	vertices.clear();
	normals.clear();
	faceIdx1.clear();
	faceIdx2.clear();
	faceIdx3.clear();
	
	while (!in.eof()) {
		if (in.peek() == 'v') {
			in >> type;
			
			if (in.peek() == 'n'){
				in >> type;
			
				float x, y, z;
				in>> x; in>> y; in>> z;
				normals.push_back(Vec3(x,y,z));
				
			}	else if (in.peek() != 't') {
				float x, y, z;
				in>> x; in>> y; in>> z;
				vertices.push_back(Vec3(x,y,z));				
			}
			in.getline(s, 256);
			
		}	else if (in.peek() == 'f') {
			in >> type;
			
			int vertex1, vertex2, vertex3;
			int normal1, normal2, normal3;
			char tempByte;
			
			// vertex 1
			in>> vertex1; --vertex1;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 1
			in>> normal1; --normal1;
			
			// vertex 2
			in>> vertex2; --vertex2;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 2
			in>> normal2; --normal2;
			
			// vertex 3
			in>> vertex3; --vertex3;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 3
			in>> normal3; --normal3;
			
			// Store face
			pair<int, int> idx1 (vertex1, normal1);
			pair<int, int> idx2 (vertex2, normal2);
			pair<int, int> idx3 (vertex3, normal3);

			faceIdx1.push_back(idx1);
			faceIdx2.push_back(idx2);
			faceIdx3.push_back(idx3);
			
			in.getline(s, 256);
		} else {
			in.getline(s, 256);
		}
	}	
	in.close();

	
	unsigned int nbVertices = vertices.size();	
	cout << "Object loaded: " << endl;
	cout << "\t Filename = " << filename << endl;
	cout << "\t #vertices = " << nbVertices << endl;

	// normalize object scale.
	if (normalize) {
		float minX=FLT_MAX;
		float maxX=-FLT_MAX;
		float minY=FLT_MAX;;
		float maxY=-FLT_MAX;
		float minZ=FLT_MAX;
		float maxZ=-FLT_MAX;

		// find bounding box coords
		for (unsigned int i=0 ; i<nbVertices ; i++){
			minX=(minX>vertices[i].x)?vertices[i].x:minX;				
			maxX=(maxX<vertices[i].x)?vertices[i].x:maxX;
			minY=(minY>vertices[i].y)?vertices[i].y:minY;				
			maxY=(maxY<vertices[i].y)?vertices[i].y:maxY;
			minZ=(minZ>vertices[i].z)?vertices[i].z:minZ;				
			maxZ=(maxZ<vertices[i].z)?vertices[i].z:maxZ;
		}

		// modify object vertices so that the maximum distance is unit size...
		Vec3 center (0.5*(maxX+minX), 0.5*(maxY+minY), 0.5*(maxZ+minZ));
		float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
	  float scaling=1.0/norm;
	
	  for (unsigned int i=0 ; i<nbVertices ; i++) {		  
	    vertices[i].x =(vertices[i].x - center.x)*scaling;
	    vertices[i].y =(vertices[i].y - center.y)*scaling;
	    vertices[i].z =(vertices[i].z - center.z)*scaling;
	  }
	}


	return true;			
}










////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint	texture;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;


int loadImage(const char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;		
    if ((file = fopen(filename, "rb"))==NULL) {
				return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
				cout << "Error reading width from " << filename << endl;
				return 0;
    }    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
				cout << "Error reading height from " << filename << endl;
				return 0;
    }    
		unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
				cout << "Error reading planes from " << filename << endl;
				return 0;
    }
    if (planes != 1) {
				cout << "Planes from " << filename << "is not 1: " << planes << endl;
				return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) { 
				cout << "Error reading bpp from " << filename << endl;
				return 0;
    }
    if (bpp != 24) {
				cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
				return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
				cout << "Error allocating memory for color-corrected image data" << endl;
				return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
				cout << "Error reading image data from " << filename << endl;
				return 0;
    }

		// reverse all of the colors. (bgr -> rgb) 
		char temp;                         
		for (i=0;i<size;i+=3) { 
				temp = image->data[i];
				image->data[i] = image->data[i+2];
				image->data[i+2] = temp;
    }
		
    return 1;
}


GLvoid loadGLTexture(const char* textureName, GLuint* textureIdx) {	
    // Load Texture
    Image *image;
    
    // allocate space for cube texture
    image = (Image *) malloc(sizeof(Image));
    if (image == NULL) {
				printf("Error allocating space for image");
				exit(0);
    }
    if (!loadImage(textureName, image)){
				if (!loadImage("defaultToonTex.bmp", image)) {
						cerr << "Default texture fot found : defaultToonTex.bmp" << endl;
						exit(1);
				} else {
						cout << "Using default texture: defaultToonTex.bmp" << endl;
				}
    } else {
				cout << "Texture Loaded: " << endl;
				cout << "\t Filename: " << textureName << endl;
				cout << "\t Size: " << image->sizeX << endl;
		} 

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP); 
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
};






////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;
float lightRadius = 5.0;
float lightHeight = 1.0;
Vec3 light (lightRadius, lightRadius, 0.0);

void setLight(){
		light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));
		GLfloat lightPosition[]= {light.x, light.y, light.z, 1.0f};
		glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
		glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
		glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
		glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
		glEnable(GL_LIGHT0);
}





/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 0.0f;
Vec3 camera(0.0, cameraHeight, distToOrigin);

void setCamera() {

		float x = distToOrigin*cos(cameraAngle);
		float y = cameraHeight;
		float z = distToOrigin*sin(cameraAngle);
		camera = Vec3(x, y, z);
		
		glLoadIdentity();
		gluLookAt(x, y, z,
							-x, y, -z,
							0.0f,1.0f,0.0f);
		setLight();
}




////////////////////////////////////////
//  Material methods
////////////////////////////////////////
const int PHONG=1, PHONG_TOON=2, FULL_TOON=3, X_TOON=4;
int materialModel = PHONG;

const int PLASTIC=0, METAL=1;
int material = PLASTIC;

int nbMatIntervals = 4;

const int NONE=0, DEPTH=1, ORIENT=2, REFLECT=3;
int detailMap = NONE;

// plastic material parameters
  GLfloat plastic_ambient[] = { 0.0F,0.0F,0.0F,1.0F };
  GLfloat plastic_diffuse[] = { 0.3F,0.5F,0.7F,1.0F };
  GLfloat plastic_specular[] = { 0.6F, 0.6F, 0.6F,1.0F };
  GLfloat plastic_shininess[] = {100.0F };
  GLfloat plastic_emission[] = {0.2F,0.2F,0.2F,0.0F};

// metal material parameters
  GLfloat metal_ambient[] = { 0.0F,0.0F,0.0F,1.0F };
  GLfloat metal_diffuse[] = { 0.2F,0.3F,0.5F,1.0F };
  GLfloat metal_specular[] = { 0.3F,0.4F,0.6F,1.0F };
  GLfloat metal_shininess[] = {5.0F };
  GLfloat metal_emission[] = {0.2F,0.2F,0.2F,0.0F};

// toon shading parameters
float fullToonS = 50. ;
GLfloat phongToonDiffuse[] = { 0.0F, 0.0F, 0.0F };
GLfloat fullToonDiffuse[] = { 0.0F, 0.0F, 0.0F };
GLfloat fullToonSpecular[] = { 0.0F, 0.0F, 0.0F };

// X-Toon shading parameters
// A COMPLETER !!!!!!!!
float Flip = 1.0;


void unsetMaterial(){		

      glDisable(GL_COLOR_MATERIAL);
      glDisable(GL_LIGHTING);
      glDisable(GL_TEXTURE_2D);
      glDisable(GL_COLOR_MATERIAL);
      
}


void setMaterial(){

  unsetMaterial();

  switch(materialModel) {
    case PHONG:
      glEnable(GL_LIGHTING);

      if (material == PLASTIC) {
         glMaterialfv(GL_FRONT,GL_AMBIENT, plastic_ambient);
         glMaterialfv(GL_FRONT,GL_DIFFUSE, plastic_diffuse);
         glMaterialfv(GL_FRONT,GL_SPECULAR, plastic_specular);
         glMaterialfv(GL_FRONT,GL_SHININESS, plastic_shininess);
         glMaterialfv(GL_FRONT,GL_EMISSION, plastic_emission);
      } else if (material == METAL) {
         glMaterialfv(GL_FRONT,GL_AMBIENT, metal_ambient);
         glMaterialfv(GL_FRONT,GL_DIFFUSE, metal_diffuse);
         glMaterialfv(GL_FRONT,GL_SPECULAR, metal_specular);
         glMaterialfv(GL_FRONT,GL_SHININESS, metal_shininess);
         glMaterialfv(GL_FRONT,GL_EMISSION, metal_emission);
      }

      break;

    case PHONG_TOON:
      glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
      glEnable(GL_COLOR_MATERIAL);
      glEnable(GL_LIGHTING);

      if (material == PLASTIC) {
         phongToonDiffuse[0] = plastic_diffuse[0];
         phongToonDiffuse[1] = plastic_diffuse[1];
         phongToonDiffuse[2] = plastic_diffuse[2];
         glMaterialfv(GL_FRONT,GL_AMBIENT, plastic_ambient);
         glMaterialfv(GL_FRONT,GL_DIFFUSE, plastic_diffuse);
         glMaterialfv(GL_FRONT,GL_SPECULAR, plastic_specular);
         glMaterialfv(GL_FRONT,GL_SHININESS, plastic_shininess);
         glMaterialfv(GL_FRONT,GL_EMISSION, plastic_emission);
      } else if (material == METAL) {
         phongToonDiffuse[0] = metal_diffuse[0];
         phongToonDiffuse[1] = metal_diffuse[1];
         phongToonDiffuse[2] = metal_diffuse[2];
         glMaterialfv(GL_FRONT,GL_AMBIENT, metal_ambient);
         glMaterialfv(GL_FRONT,GL_DIFFUSE, metal_diffuse);
         glMaterialfv(GL_FRONT,GL_SPECULAR, metal_specular);
         glMaterialfv(GL_FRONT,GL_SHININESS, metal_shininess);
         glMaterialfv(GL_FRONT,GL_EMISSION, metal_emission);
      }

      break;
      
    case FULL_TOON:

      if (material == PLASTIC) {
         fullToonS = plastic_shininess[0];
         fullToonDiffuse[0] = plastic_diffuse[0];
         fullToonDiffuse[1] = plastic_diffuse[1];
         fullToonDiffuse[2] = plastic_diffuse[2];
         fullToonSpecular[0] = plastic_specular[0];
         fullToonSpecular[1] = plastic_specular[1];
         fullToonSpecular[2] = plastic_specular[2];
      } else if (material == METAL) {
         fullToonS = metal_shininess[0];
         fullToonDiffuse[0] = metal_diffuse[0];
         fullToonDiffuse[1] = metal_diffuse[1];
         fullToonDiffuse[2] = metal_diffuse[2];
         fullToonSpecular[0] = metal_specular[0];
         fullToonSpecular[1] = metal_specular[1];
         fullToonSpecular[2] = metal_specular[2];
      }

      break;
      
    case X_TOON:
      glEnable(GL_TEXTURE_2D);
      glEnable(GL_LIGHTING);
      glBindTexture(GL_TEXTURE_2D, texture);
      
      break;
  }  

  setLight();
}


void getColor(const Vec3& vertex, const Vec3& normal, Vec3& color){
  
  Vec3 h, l, v;
  GLfloat diffuse, Spec, res;
  
  switch(materialModel) {
   case PHONG_TOON:	
		l = light - vertex;
      l.normalize();
      diffuse = dot(l, normal);

      diffuse = ((int) (diffuse * nbMatIntervals) * 1.0/nbMatIntervals) + 0.5/nbMatIntervals ;

      color.x = diffuse * phongToonDiffuse[0];
      color.y = diffuse * phongToonDiffuse[1];
      color.z = diffuse * phongToonDiffuse[2];
      break;
   case FULL_TOON:

		l = light - vertex;
      l.normalize();
      diffuse = dot(l, normal);
      
      v = (camera - vertex).normalized();
		h = l + v;
      h.normalize();
      
      Spec = pow( (float) ((dot(h, normal) <0) ? (float) 0. : dot(h, normal)) , fullToonS );

      ((int) ((diffuse+Spec) * nbMatIntervals)) * (1.0 / nbMatIntervals) + 0.5/nbMatIntervals;

      //color.x = ((int) (( (diffuse*fullToonDiffuse[0]) + (Spec*fullToonSpecular[0]) )/2 * nbMatIntervals)) * (1.0 / nbMatIntervals) + 0.5/nbMatIntervals;
      //color.y = ((int) (( (diffuse*fullToonDiffuse[1]) + (Spec*fullToonSpecular[1]) )/2 * nbMatIntervals)) * (1.0 / nbMatIntervals) + 0.5/nbMatIntervals;
      //color.z = ((int) (( (diffuse*fullToonDiffuse[2]) + (Spec*fullToonSpecular[2]) )/2 * nbMatIntervals)) * (1.0 / nbMatIntervals) + 0.5/nbMatIntervals;

      
      color.x = ( ((int) ( diffuse * nbMatIntervals) )* (1.0 / nbMatIntervals) + 0.5/nbMatIntervals ) * fullToonDiffuse[0] +
      ( ((int) ( Spec * nbMatIntervals)) * (1.0 / nbMatIntervals) + 0.5/nbMatIntervals ) * fullToonSpecular[0];
      color.y = ( ((int) ( diffuse * nbMatIntervals) )* (1.0 / nbMatIntervals) + 0.5/nbMatIntervals ) * fullToonDiffuse[1] + 
      ( ((int) ( Spec * nbMatIntervals)) * (1.0 / nbMatIntervals) + 0.5/nbMatIntervals ) * fullToonSpecular[1];
      color.z = ( ((int) ( diffuse * nbMatIntervals) )* (1.0 / nbMatIntervals) + 0.5/nbMatIntervals ) * fullToonDiffuse[2] + 
      ( ((int) ( Spec * nbMatIntervals)) * (1.0 / nbMatIntervals) + 0.5/nbMatIntervals ) * fullToonSpecular[2];
      

      break;
   default:
      break;
   }
}


void getTexCoord(const Vec3& vertex, const Vec3& normal, float& toneCoord, float& detailCoord){
      Vec3 l, h, v;
      float D;
      switch (detailMap) {
        case NONE:
          D = 1.0;
          break;
        case DEPTH:
          D = ((camera-vertex).norm() - (zfar - znear)*(nbMatIntervals/100.0)) / 30.;
          break;
        case ORIENT:
          D = dot((camera-vertex).normalized(), normal) * exp(-0.05 * nbMatIntervals);
          break;
        case REFLECT:
          l = light - vertex;
          l.normalize();
          v = (camera - vertex).normalized();
          h = l + v;
          h.normalize();
          D = pow( (float) ((dot(h, normal) < 0) ? (float) 0. : dot(h, normal)) , nbMatIntervals );
          break;

     }

      if (D>1.0) D=1.0;
      if (D<0.0) D=0.0;
		l = light - vertex;
      l.normalize();
      toneCoord = dot(l, normal);
      if (toneCoord<0.) toneCoord = 0.;

      detailCoord = (1. + Flip*D>1.) ? 1 - D : D;
}



/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
float rotationSpeed = 1.0;
bool lightRotation = true;

void updateAnimationParam(){
		// compute a time parameter based on a 
		float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
		float t = newTime-oldTime;
		oldTime = newTime;

		// update a rotation parameter
      if (lightRotation) {
         lightAngle += rotationSpeed * t;
         setLight();
      }
}

////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene(const char* objFile, const char* toonTexFile) {
		// init scanline
		glClearColor(0.3, 0.3, 0.3, 1.0f);				
		glClearDepth(1.0);
		glDepthFunc(GL_LESS);												
		glEnable(GL_DEPTH_TEST);										
		glShadeModel(GL_SMOOTH);										
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	
		glLineWidth(2.0f);													
		glPointSize(4.0f);

		// init lighting
		setLight();
		glEnable(GL_LIGHTING);

		// init textures
		loadGLTexture(toonTexFile, &texture);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		// init object
		glEnable(GL_NORMALIZE);
		objectLoaded = loadObj(objFile, true);
}


void changeSize(int w, int h)	{

		// Prevent a divide by zero, when window is too short
		// (you cant make a window of zero width).
		if(h == 0)
				h = 1;
		
		float ratio = 1.0f * w / h;
		// Reset the coordinate system before modifying
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		
		// Set the viewport to be the entire window
		glViewport(0, 0, w, h);
		
		// Set the clipping volume
		gluPerspective(45,ratio,znear,zfar);
		glMatrixMode(GL_MODELVIEW);
		setCamera();
}










///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////


void drawSphere(){

		setMaterial();
		
		glPushMatrix();
		
		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;
				
				// draw one strip
				Vec3 color1, color2;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;
						
						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						glColor3fv(color1);
						glNormal3fv(pos1);
						glVertex3fv(pos1);
						
						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						glColor3fv(color2);
						glNormal3fv(pos2);
						glVertex3fv(pos2);
				}
				glEnd();
		}
		glPopMatrix();

		unsetMaterial();

}

void drawObject(){

   	setMaterial();
      Vec3 color;
      float texd, texc;

		glPushMatrix();
 		glScalef(10.0, 10.0, 10.0);

      glBegin(GL_TRIANGLES);
      
		for (unsigned int i=0; i<faceIdx1.size(); i++) {
         // parcours des triangles

         switch(materialModel) {
            case PHONG:
		         glNormal3fv(normals[faceIdx1[i].second]);
		         glVertex3fv(vertices[faceIdx1[i].first]);

		         glNormal3fv(normals[faceIdx2[i].second]);
		         glVertex3fv(vertices[faceIdx2[i].first]);

		         glNormal3fv(normals[faceIdx3[i].second]);
		         glVertex3fv(vertices[faceIdx3[i].first]);
               break;
               
            case PHONG_TOON:
               getColor(vertices[faceIdx1[i].first], normals[faceIdx1[i].second], color);
		         glColor3fv(color);
		         glNormal3fv(normals[faceIdx1[i].second]);
		         glVertex3fv(vertices[faceIdx1[i].first]);

               getColor(vertices[faceIdx2[i].first], normals[faceIdx2[i].second], color);
		         glColor3fv(color);
		         glNormal3fv(normals[faceIdx2[i].second]);
		         glVertex3fv(vertices[faceIdx2[i].first]);

               getColor(vertices[faceIdx3[i].first], normals[faceIdx3[i].second], color);
		         glColor3fv(color);
		         glNormal3fv(normals[faceIdx3[i].second]);
		         glVertex3fv(vertices[faceIdx3[i].first]);
               break;
            
            case FULL_TOON:
               getColor(vertices[faceIdx1[i].first], normals[faceIdx1[i].second], color);
		         glColor3fv(color);
		         glVertex3fv(vertices[faceIdx1[i].first]);

               getColor(vertices[faceIdx2[i].first], normals[faceIdx2[i].second], color);
		         glColor3fv(color);
		         glVertex3fv(vertices[faceIdx2[i].first]);

               getColor(vertices[faceIdx3[i].first], normals[faceIdx3[i].second], color);
		         glColor3fv(color);
		         glVertex3fv(vertices[faceIdx3[i].first]);
               break;

            case X_TOON:
               getTexCoord(vertices[faceIdx1[i].first], normals[faceIdx1[i].second], texc, texd);
               glTexCoord2f(texc,texd);
		         glVertex3fv(vertices[faceIdx1[i].first]);
               
               getTexCoord(vertices[faceIdx2[i].first], normals[faceIdx2[i].second], texc, texd);
               glTexCoord2f(texc,texd);
		         glVertex3fv(vertices[faceIdx2[i].first]); 
            
               getTexCoord(vertices[faceIdx3[i].first], normals[faceIdx3[i].second], texc, texd);
               glTexCoord2f(texc,texd);
		         glVertex3fv(vertices[faceIdx3[i].first]);
               break;

            default:
               break;
         }
      }
         
         
      
      
		glEnd();
		
		glPopMatrix();

		unsetMaterial();
}


void renderScene(void) {
		// clear buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		// draw objects
		if (objectLoaded){
				drawObject();
		} else {
				drawSphere();
		}

		// update animation parameters
		updateAnimationParam();

		// display rendering
		glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////

    
void specialKey(int key, int x_, int y_) {
		
		switch (key) {
		case GLUT_KEY_LEFT :
				cameraAngle += 0.04f;
				setCamera();  break;
		case GLUT_KEY_RIGHT :
				cameraAngle -=0.04f;
				setCamera();  break;
		case GLUT_KEY_UP :
				distToOrigin -= 0.5f;
				if (distToOrigin<1.5f) distToOrigin = 1.5f;
				setCamera();  break;
		case GLUT_KEY_DOWN :
				distToOrigin += 0.5f;
				setCamera();  break;
		case GLUT_KEY_PAGE_UP :
				cameraHeight += 0.1f;
				setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
				cameraHeight -= 0.1f;
				setCamera();  break;
		case GLUT_KEY_F1 :
				detailMap=NONE; break;
		case GLUT_KEY_F2 :
				detailMap=DEPTH; break;
		case GLUT_KEY_F3 :
				detailMap=ORIENT; break;
		case GLUT_KEY_F4 :
				detailMap=REFLECT; break;
		}
}




void commonKey(unsigned char key, int x_, int y_) {
  float D;
		switch (key) {
		case 'r':
				lightRotation = true; break;
		case ',':
				lightRotation = false;
				lightAngle += 0.1f;
				setLight(); break;
		case '.':
				lightRotation = false;
				lightAngle -= 0.1f;
				setLight(); break;
		case '1':
				materialModel = PHONG;
            cerr<<"Modele : PHONG"<<endl;
            break;
		case '2':
				materialModel = PHONG_TOON; 
            cerr<<"Modele : PHONG_TOON"<<endl;
            break;
		case '3':
				materialModel = FULL_TOON; 
            cerr<<"Modele : FULL_TOON"<<endl;
            break;
		case '4':
				materialModel = X_TOON; 
            cerr<<"Modele : X_TOON"<<endl;
            break;
		case 'm':
				material = METAL; 
            cerr<<"material : METAL"<<endl;
            break;
		case 'p':
				material = PLASTIC; 
            cerr<<"material : PLASTIC"<<endl;
            break;
		case 'f':
				Flip *= -1.0;
            cerr<<"Flip-Mode : "<< Flip << endl;
            break;
    case '+':
				nbMatIntervals++;
            cerr<<"nbMatIntervals : "<<nbMatIntervals<<endl;
            D = (nbMatIntervals/10.>1.0)?1.0:(nbMatIntervals/10.);
            cerr<<"D : "<<D<<endl;
				break;
		case '-':
				if (nbMatIntervals>0) nbMatIntervals--;
            cerr<<"nbMatIntervals : "<<nbMatIntervals<<endl;
            D = (nbMatIntervals/10.>1.0)?1.0:(nbMatIntervals/10.);
            cerr<<"D : "<<D<<endl;
				break;
		case '/': 
				rotationSpeed += 0.1; break;
		case '*': 
				rotationSpeed -= 0.1; break;
		}
}






////////////////////////////////////
//  Main method
////////////////////////////////////

void parseArgs(int argc, char **argv, char** objFileName, char** toonTexFileName){

		// default values
		*objFileName = "-";
		*toonTexFileName = "-";

		if (argc>1){
				int nbCar1 = strlen(argv[1]);

				if (argv[1][nbCar1-3]=='o' && argv[1][nbCar1-2]=='b' && argv[1][nbCar1-1]=='j'){
						*objFileName = argv[1];
				} else if (argv[1][nbCar1-3]=='b' && argv[1][nbCar1-2]=='m' && argv[1][nbCar1-1]=='p'){
						*toonTexFileName = argv[1];
				}

				if (argc>2){
						int nbCar2 = strlen(argv[2]);
						
						if (argv[2][nbCar2-3]=='o' && argv[2][nbCar2-2]=='b' && argv[2][nbCar2-1]=='j'){
								*objFileName = argv[2];					
						} else if (argv[2][nbCar2-3]=='b' && argv[2][nbCar2-2]=='m' && argv[2][nbCar2-1]=='p'){
								*toonTexFileName = argv[2];					
						}
				}
		}
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800, 600);
	glutCreateWindow("Humanoid");

	char *objFileName, *toonTexFileName;
	parseArgs(argc, argv, &objFileName, &toonTexFileName);
	initScene(objFileName, toonTexFileName);

	glutSpecialFunc(specialKey);
	glutKeyboardFunc(commonKey);

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);

	glutReshapeFunc(changeSize);

	glutMainLoop();

	return(0);
}
