#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <float.h>
#include <string.h>
#include "Vec3.h"

#define PHONG 1
#define PHONG_TOON 2
#define FULL_TOON 3
#define X_TOON 4

#define NONE 0
#define DEPTH 1
#define ORIENT 2
#define REFLECT 3

using namespace std;

////////////////////////////////
// Object mesh methods
////////////////////////////////

float znear=1.0, zfar=100.0;
vector<Vec3> vertices, normals;
vector< pair<int, int> > faceIdx1, faceIdx2, faceIdx3;
bool objectLoaded = false;
int materialModel = PHONG;
bool flip = false;
float min_dist_p2cam =  9000000.0f;
float max_dist_p2cam = -9000000.0f;
float distance_milieu = 0.0f;
float cone = 1.0f;
float x_shininess = 10.0f;


bool loadObj(const char * filename, bool normalize) {

	ifstream in(filename);
	char type;
	char s[256];

	if (!in.is_open()) {
		cout << "Using default object: unit sphere"  << endl;
		return false;
	}

	vertices.clear();
	normals.clear();
	faceIdx1.clear();
	faceIdx2.clear();
	faceIdx3.clear();

	while (!in.eof()) {
		if (in.peek() == 'v') {
			in >> type;

			if (in.peek() == 'n'){
				in >> type;

				float x, y, z;
				in>> x; in>> y; in>> z;
				normals.push_back(Vec3(x,y,z));

			}	else if (in.peek() != 't') {
				float x, y, z;
				in>> x; in>> y; in>> z;
				vertices.push_back(Vec3(x,y,z));
			}
			in.getline(s, 256);

		}	else if (in.peek() == 'f') {
			in >> type;

			int vertex1, vertex2, vertex3;
			int normal1, normal2, normal3;
			char tempByte;

			// vertex 1
			in>> vertex1; --vertex1;
			// separator
			in >> tempByte; in >> tempByte;
			// normal 1
			in>> normal1; --normal1;

			// vertex 2
			in>> vertex2; --vertex2;
			// separator
			in >> tempByte; in >> tempByte;
			// normal 2
			in>> normal2; --normal2;

			// vertex 3
			in>> vertex3; --vertex3;
			// separator
			in >> tempByte; in >> tempByte;
			// normal 3
			in>> normal3; --normal3;

			// Store face
			pair<int, int> idx1 (vertex1, normal1);
			pair<int, int> idx2 (vertex2, normal2);
			pair<int, int> idx3 (vertex3, normal3);

			faceIdx1.push_back(idx1);
			faceIdx2.push_back(idx2);
			faceIdx3.push_back(idx3);

			in.getline(s, 256);
		} else {
			in.getline(s, 256);
		}
	}
	in.close();


	unsigned int nbVertices = vertices.size();	
	cout << "Object loaded: " << endl;
	cout << "\t Filename = " << filename << endl;
	cout << "\t #vertices = " << nbVertices << endl;

	// normalize object scale.
	if (normalize) {
		float minX=FLT_MAX;
		float maxX=-FLT_MAX;
		float minY=FLT_MAX;;
		float maxY=-FLT_MAX;
		float minZ=FLT_MAX;
		float maxZ=-FLT_MAX;

		// find bounding box coords
		for (unsigned int i=0 ; i<nbVertices ; i++){
			minX=(minX>vertices[i].x)?vertices[i].x:minX;				
			maxX=(maxX<vertices[i].x)?vertices[i].x:maxX;
			minY=(minY>vertices[i].y)?vertices[i].y:minY;				
			maxY=(maxY<vertices[i].y)?vertices[i].y:maxY;
			minZ=(minZ>vertices[i].z)?vertices[i].z:minZ;				
			maxZ=(maxZ<vertices[i].z)?vertices[i].z:maxZ;
		}

		// modify object vertices so that the maximum distance is unit size...
		Vec3 center (0.5*(maxX+minX), 0.5*(maxY+minY), 0.5*(maxZ+minZ));
		float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
		float scaling=1.0/norm;

		for (unsigned int i=0 ; i<nbVertices ; i++) {		  
			vertices[i].x =(vertices[i].x - center.x)*scaling;
			vertices[i].y =(vertices[i].y - center.y)*scaling;
			vertices[i].z =(vertices[i].z - center.z)*scaling;
		}
	}


	return true;			
}










////////////////////////////////////////
//  Texture methods
////////////////////////////////////////

GLuint	texture;

typedef struct {
	unsigned long sizeX;
	unsigned long sizeY;
	char *data;
}Image;


int loadImage(const char *filename, Image *image) {

	// make sure the file is there.
	FILE *file;		
	if ((file = fopen(filename, "rb"))==NULL) {
		return 0;
	}

	// seek through the bmp header, up to the width/height:
	fseek(file, 18, SEEK_CUR);

	// read width and height, and calculate the size of the image
	unsigned long i;
	if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
		cout << "Error reading width from " << filename << endl;
		return 0;
	}    
	if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
		cout << "Error reading height from " << filename << endl;
		return 0;
	}    
	unsigned long size = image->sizeX * image->sizeY * 3;

	// read the planes
	unsigned short int planes;
	if ((fread(&planes, 2, 1, file)) != 1) {
		cout << "Error reading planes from " << filename << endl;
		return 0;
	}
	if (planes != 1) {
		cout << "Planes from " << filename << "is not 1: " << planes << endl;
		return 0;
	}

	// read the bpp
	unsigned short int bpp;
	if ((i = fread(&bpp, 2, 1, file)) != 1) { 
		cout << "Error reading bpp from " << filename << endl;
		return 0;
	}
	if (bpp != 24) {
		cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
		return 0;
	}

	// seek past the rest of the bitmap header.
	fseek(file, 24, SEEK_CUR);

	// read the data. 
	image->data = (char *) malloc(size);
	if (image->data == NULL) {
		cout << "Error allocating memory for color-corrected image data" << endl;
		return 0;	
	}

	if ((i = fread(image->data, size, 1, file)) != 1) {
		cout << "Error reading image data from " << filename << endl;
		return 0;
	}

	// reverse all of the colors. (bgr -> rgb) 
	char temp;                         
	for (i=0;i<size;i+=3) { 
		temp = image->data[i];
		image->data[i] = image->data[i+2];
		image->data[i+2] = temp;
	}

	return 1;
}


GLvoid loadGLTexture(const char* textureName, GLuint* textureIdx) {	
	// Load Texture
	Image *image;

	// allocate space for cube texture
	image = (Image *) malloc(sizeof(Image));
	if (image == NULL) {
		printf("Error allocating space for image");
		exit(0);
	}
	if (!loadImage(textureName, image)){
		if (!loadImage("defaultToonTex.bmp", image)) {
			cerr << "Default texture fot found : defaultToonTex.bmp" << endl;
			exit(1);
		} else {
			cout << "Using default texture: defaultToonTex.bmp" << endl;
		}
	} else {
		cout << "Texture Loaded: " << endl;
		cout << "\t Filename: " << textureName << endl;
		cout << "\t Size: " << image->sizeX << endl;
	} 

	// Create Texture
	glGenTextures(1, textureIdx);
	glBindTexture(GL_TEXTURE_2D, *textureIdx);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); 
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); 
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP); 
	glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
};






////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 0.0f, 0.0f, 0.0f, 0.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;
float lightRadius = 5.0;
float lightHeight = 1.0;
Vec3 light (lightRadius, lightRadius, 0.0);

void setLight(){
	light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));
	GLfloat lightPosition[]= {light.x, light.y, light.z, 1.0f};
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
	glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);

	switch(materialModel)
	{
		case PHONG:
			glEnable(GL_LIGHT0);
			glEnable(GL_LIGHTING);
			break;
		case PHONG_TOON:
			glEnable(GL_LIGHT0);
			glEnable(GL_LIGHTING);
			break;
		case FULL_TOON:
			glDisable(GL_LIGHT0);
			glDisable(GL_LIGHTING);
			break;
		case X_TOON:
			glDisable(GL_LIGHT0);
			glDisable(GL_LIGHTING);
			break;
		default:
			glDisable(GL_LIGHT0);
			glDisable(GL_LIGHTING);
	}
}





/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 0.0f;
Vec3 camera(0.0, cameraHeight, distToOrigin);

void setCamera() {

	float x = distToOrigin*cos(cameraAngle);
	float y = cameraHeight;
	float z = distToOrigin*sin(cameraAngle);
	camera = Vec3(x, y, z);

	glLoadIdentity();
	gluLookAt(x, y, z,
			-x, y, -z,
			0.0f,1.0f,0.0f);
	setLight();
}




////////////////////////////////////////
//  Material methods
////////////////////////////////////////



int PLASTIC=0, METAL=1;
int material = PLASTIC;

int detailMap = NONE;



// plastic material parameters
GLfloat p_lightDiffuse[]= { 0.0f, 0.6f, 1.0f, 1.0f };
GLfloat p_lightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float p_shininess = 100.0f;

// metallic material parameters
GLfloat m_lightDiffuse[]= { 1.0f, 0.6f, 0.6f, 1.0f };
GLfloat m_lightSpecular[]= { 1.0f, 0.6f, 0.6f, 1.0f };
float m_shininess = 10.0f;

GLfloat Diffuse0[] = { 0.0f, 0.0f, 0.0f, 1.0f };

// toon shading parameters
int nb_intervalles = 3;

// X-Toon shading parameters
// A COMPLETER !!!!!!!!



void setMaterial(){

	switch(materialModel)
	{
		case PHONG:
			if (material == METAL)
			{
				glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,   lightAmbient);
				glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,   m_lightDiffuse);
				glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,  m_lightSpecular);
				glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, m_shininess);

			}
			else if (material == PLASTIC)
			{
				glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,   lightAmbient);
				glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,   p_lightDiffuse);
				glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,  p_lightSpecular);
				glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, p_shininess);
			}
			glDisable(GL_COLOR_MATERIAL);
			break;
		case PHONG_TOON:
			glEnable(GL_COLOR_MATERIAL);
			glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);

			glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,   Diffuse0);

			if (material == METAL)
			{
				glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,   lightAmbient);
				glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,  m_lightSpecular);
				glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, m_shininess);

			}
			else if (material == PLASTIC)
			{
				glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,   lightAmbient);
				glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,  p_lightSpecular);
				glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, p_shininess);
			}
			break;
		case FULL_TOON:
			glDisable(GL_COLOR_MATERIAL);
			break;
		case X_TOON:
			glDisable(GL_COLOR_MATERIAL);
			glEnable(GL_TEXTURE_2D);
			break;
		default:
			glDisable(GL_COLOR_MATERIAL);
	}
}

	void unsetMaterial(){		
		switch(materialModel)
		{
			case PHONG_TOON:
				glDisable(GL_COLOR_MATERIAL);
				break;
			case X_TOON:
				glDisable(GL_TEXTURE_2D);
				break;
			default:;
		}
	}


void getColor(const Vec3& vertex, const Vec3& normal, Vec3& color){
	Vec3 light = Vec3(lightRadius*cos(lightAngle), lightHeight, lightRadius*sin(lightAngle));

	switch(materialModel)
	{
		case PHONG_TOON:
			{
				Vec3 li = light-vertex;
				float d = dot((light-vertex).normalized(),normal.normalized());
				if (d < 0.0f) {
					color.x=lightAmbient[0];
					color.y=lightAmbient[1];
					color.z=lightAmbient[2];
				} else {
					float l = ( (float)floor(d*(float)nb_intervalles) + 0.5f/((float)nb_intervalles)) / ((float)nb_intervalles);
					//float l = d;
					color.x=l;
					color.y=l;
					color.z=l;
				}
				if (material == METAL)
				{
					color.x *= m_lightDiffuse[0]*lightDiffuse[0];
					color.y *= m_lightDiffuse[1]*lightDiffuse[1];
					color.z *= m_lightDiffuse[2]*lightDiffuse[2];
					glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,  m_lightSpecular);
					glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, m_shininess);
				}
				else if (material == PLASTIC)
				{
					color.x *= p_lightDiffuse[0]*lightDiffuse[0];
					color.y *= p_lightDiffuse[1]*lightDiffuse[1];
					color.z *= p_lightDiffuse[2]*lightDiffuse[2];
					glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,  p_lightSpecular);
					glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, p_shininess);
				}
			}
			break;

		case FULL_TOON:
			{
				//\\ Composante diffuse //\\
				
				float d = dot((light-vertex).normalized(),normal.normalized());
				if (d < 0.0f) {
					color.x=lightAmbient[0];
					color.y=lightAmbient[1];
					color.z=lightAmbient[2];
				} else {
					float l = ( (float)floor(d*(float)nb_intervalles) + 0.5f/((float)nb_intervalles)) / ((float)nb_intervalles);
					color.x=l;
					color.y=l;
					color.z=l;
				}
				if (material == METAL)
				{
					color.x *= m_lightDiffuse[0]*lightDiffuse[0];
					color.y *= m_lightDiffuse[1]*lightDiffuse[1];
					color.z *= m_lightDiffuse[2]*lightDiffuse[2];
				}
				else if (material == PLASTIC)
				{
					color.x *= p_lightDiffuse[0]*lightDiffuse[0];
					color.y *= p_lightDiffuse[1]*lightDiffuse[1];
					color.z *= p_lightDiffuse[2]*lightDiffuse[2];
				}


				//\\ composante specular \\//

				Vec3 n = normal;
				Vec3 l =light;
				Vec3 v (distToOrigin*cos(cameraAngle), cameraHeight, distToOrigin*sin(cameraAngle));
				v = v-vertex;
				Vec3 h = l+v;
				float s;
				if (material == PLASTIC)
					s = p_shininess;
				else
					s = m_shininess;
				float spec = pow( dot(n.normalized(),h.normalized()), s );
				if (spec > 0.5f && dot(n,l) > 0.0f ) spec = 1.0f; else spec = 0.0f;
				if (material == METAL)
				{
					color.x += spec*m_lightSpecular[0]*lightSpecular[0];
					color.y += spec*m_lightSpecular[1]*lightSpecular[1];
					color.z += spec*m_lightSpecular[2]*lightSpecular[2];
				}
				else if (material == PLASTIC)
				{
					color.x += spec*p_lightSpecular[0]*lightSpecular[0];
					color.y += spec*p_lightSpecular[1]*lightSpecular[1];
					color.z += spec*p_lightSpecular[2]*lightSpecular[2];
				}
			}

			break;
	}

}


void getTexCoord(const Vec3& vertex, const Vec3& normal, float& toneCoord, float& detailCoord)
{
	float d = dot((light-vertex).normalized(),normal.normalized());
	if (d < 0.0f) {
		d=0.0f;
	}
	toneCoord = d;

	switch(detailMap)
	{
		case NONE:
			detailCoord = 1.0f;
			break;
		case DEPTH:
			{
			float d = (vertex-camera).norm2();
			if (d < min_dist_p2cam) min_dist_p2cam = d;
			if (d > max_dist_p2cam) max_dist_p2cam = d;
			detailCoord = (d-min_dist_p2cam)/(max_dist_p2cam-min_dist_p2cam) + distance_milieu;
			}
			break;
		case ORIENT:
			{
			float cos_angle = dot(normal.normalized(), (vertex-camera).normalized());
			detailCoord = ( (cos_angle/2.0f)*cone + cone*0.5f);
			}
			break;
		case REFLECT:
			Vec3 n = normal;
			Vec3 l =light;
			Vec3 v (distToOrigin*cos(cameraAngle), cameraHeight, distToOrigin*sin(cameraAngle));
			v = v-vertex;
			Vec3 h = l+v;
			float spec = pow( dot(n.normalized(),h.normalized()), x_shininess );
			detailCoord = spec;
			break;
	}

	if (flip) detailCoord = 1.0f - detailCoord;
}



/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
float rotationSpeed = 1.0;
bool lightRotation = true;

void updateAnimationParam(){
	// compute a time parameter based on a
	float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
	float t = newTime-oldTime;
	oldTime = newTime;

	// update a rotation parameter

	if(lightRotation)
	{
		lightAngle += t*rotationSpeed;
		setLight();
	}

}

////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene(const char* objFile, const char* toonTexFile) {
	// init scanline
	glClearColor(0.3, 0.3, 0.3, 1.0f);
	glClearDepth(1.0);
	glDepthFunc(GL_LESS);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glLineWidth(2.0f);
	glPointSize(4.0f);

	// init lighting
	setLight();
	glEnable(GL_LIGHTING);

	// init textures
	loadGLTexture(toonTexFile, &texture);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	// init object
	glEnable(GL_NORMALIZE);
	objectLoaded = loadObj(objFile, true);
}


void changeSize(int w, int h)	{

	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	float ratio = 1.0f * w / h;
	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Set the viewport to be the entire window
	glViewport(0, 0, w, h);

	// Set the clipping volume
	gluPerspective(45,ratio,znear,zfar);
	glMatrixMode(GL_MODELVIEW);
	setCamera();
}










///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////


void drawSphere(){

	setMaterial();

	glPushMatrix();

	int nbStrips = 20;
	float stepVAngle = M_PI/nbStrips;

	int nbFacesPerStrip = 50;
	float stepHAngle = 2*M_PI/nbFacesPerStrip;

	for(int i=0 ; i<nbStrips ; i++) {
		float vAngle1 = i*stepVAngle - 0.5*M_PI;
		float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;

		// draw one strip
		Vec3 color1, color2;
		glBegin(GL_QUAD_STRIP);
		for (int j=0 ; j<=nbFacesPerStrip ; j++){
			float hAngle = j*stepHAngle;

			float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
			glColor3fv(color1);
			glNormal3fv(pos1);
			glVertex3fv(pos1);

			float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
			glColor3fv(color2);
			glNormal3fv(pos2);
			glVertex3fv(pos2);
		}
		glEnd();
	}
	glPopMatrix();

	unsetMaterial();

}

void drawObject(){

	setMaterial();

	glPushMatrix();
	glScalef(10.0, 10.0, 10.0);

	glBegin(GL_TRIANGLES);

	for (int f=0; f<faceIdx1.size(); f++)
	{
		Vec3 color;
		if (materialModel == PHONG_TOON)
		{
			glNormal3fv(normals[faceIdx1[f].second].normalized());
			getColor(vertices[faceIdx1[f].first],normals[faceIdx1[f].second],color);
			glColor3fv(color);
		} else if (materialModel == FULL_TOON) {
			getColor(vertices[faceIdx1[f].first],normals[faceIdx1[f].second],color);
			glColor3fv(color);
		} else if (materialModel == X_TOON) {
			float toneCoord;
			float detailCoord;
			getTexCoord(vertices[faceIdx1[f].first], normals[faceIdx1[f].second], toneCoord, detailCoord);
			glTexCoord2f(toneCoord, detailCoord);
			glColor3f(1.0f, 1.0f, 1.0f);
		} else {
			glNormal3fv(normals[faceIdx1[f].second].normalized());
		}
		glVertex3fv(vertices[faceIdx1[f].first]);


		if (materialModel == PHONG_TOON)
		{
			glNormal3fv(normals[faceIdx2[f].second].normalized());
			getColor(vertices[faceIdx2[f].first],normals[faceIdx2[f].second],color);
			glColor3fv(color);
		} else if (materialModel == FULL_TOON)
		{
			getColor(vertices[faceIdx2[f].first],normals[faceIdx2[f].second],color);
			glColor3fv(color);
		}  else if (materialModel == X_TOON) {

			float toneCoord;
			float detailCoord;
			getTexCoord(vertices[faceIdx2[f].first], normals[faceIdx2[f].second], toneCoord, detailCoord);
			glTexCoord2f(toneCoord, detailCoord);
			glColor3f(1.0f, 1.0f, 1.0f);

		} else {
			glNormal3fv(normals[faceIdx2[f].second].normalized());
		}
		glVertex3fv(vertices[faceIdx2[f].first]);


		if (materialModel == PHONG_TOON)
		{
			glNormal3fv(normals[faceIdx3[f].second].normalized());
			getColor(vertices[faceIdx3[f].first],normals[faceIdx3[f].second],color);
			glColor3fv(color);
		} else if (materialModel == FULL_TOON)
		{
			getColor(vertices[faceIdx3[f].first],normals[faceIdx3[f].second],color);
			glColor3fv(color);
		} else if (materialModel == X_TOON) {

			float toneCoord;
			float detailCoord;
			getTexCoord(vertices[faceIdx3[f].first], normals[faceIdx3[f].second], toneCoord, detailCoord);
			glTexCoord2f(toneCoord, detailCoord);
			glColor3f(1.0f, 1.0f, 1.0f);

		} else {
			glNormal3fv(normals[faceIdx3[f].second].normalized());
		}
		glVertex3fv(vertices[faceIdx3[f].first]);

	}

	glEnd();

	glPopMatrix();

	unsetMaterial();
}


void renderScene(void) {
	// clear buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// draw objects
	if (objectLoaded){
		drawObject();
	} else {
		drawSphere();
	}

	// update animation parameters
	updateAnimationParam();

	// display rendering
	glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////


void specialKey(int key, int x_, int y_) {

	switch (key) {
		case GLUT_KEY_LEFT :
			cameraAngle += 0.04f;
			setCamera();  break;
		case GLUT_KEY_RIGHT :
			cameraAngle -=0.04f;
			setCamera();  break;
		case GLUT_KEY_UP :
			distToOrigin -= 0.5f;
			if (distToOrigin<1.5f) distToOrigin = 1.5f;
			setCamera();  break;
		case GLUT_KEY_DOWN :
			distToOrigin += 0.5f;
			setCamera();  break;
		case GLUT_KEY_PAGE_UP :
			cameraHeight += 0.1f;
			setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
			cameraHeight -= 0.1f;
			setCamera();  break;
		case GLUT_KEY_F1 :
			detailMap=NONE; break;
		case GLUT_KEY_F2 :
			detailMap=DEPTH; break;
		case GLUT_KEY_F3 :
			detailMap=ORIENT; break;
		case GLUT_KEY_F4 :
			detailMap=REFLECT; break;
	}
}




void commonKey(unsigned char key, int x_, int y_) {

	switch (key) {
		case 'r':
			lightRotation = true; break;
		case ',':
			lightRotation = false;
			lightAngle += 0.1f;
			setLight(); break;
		case '.':
			lightRotation = false;
			lightAngle -= 0.1f;
			setLight(); break;
		case '1':
			materialModel = PHONG; setLight(); break;
		case '2':
			materialModel = PHONG_TOON; setLight(); break;
		case '3':
			materialModel = FULL_TOON; setLight(); break;
		case '4':
			materialModel = X_TOON; setLight(); break;
		case 'm':
			material = METAL; break;
		case 'p':
			material = PLASTIC; break;
		case '+':
			if (materialModel == X_TOON)
			{
				switch(detailMap)
				{
					case NONE:
						break;
					case DEPTH:
						{
						distance_milieu += 0.1f;
						}
						break;
					case ORIENT:
						cone += 0.1f;
						break;
					case REFLECT:
						x_shininess *= 2.0f;
						break;
				}
			} else {
				nb_intervalles++;
			}
			break;
		case '-':
			if (materialModel == X_TOON)
			{
				switch(detailMap)
				{
					case NONE:
						break;
					case DEPTH:
						{
						distance_milieu -= 0.1f;
						}
						break;
					case ORIENT:
						cone -= 0.1f;
						break;
					case REFLECT:
						x_shininess /= 2.0f;
						break;
				}
			} else {
				if (nb_intervalles > 1) nb_intervalles--;
			}
			break;
		case '/':
			rotationSpeed += 0.1; break;
		case '*': 
			rotationSpeed -= 0.1; break;
		case 'f':
			flip = !flip; break;
	}
}






////////////////////////////////////
//  Main method
////////////////////////////////////

void parseArgs(int argc, char **argv, char** objFileName, char** toonTexFileName){

	// default values
	*objFileName = "-";
	*toonTexFileName = "-";

	if (argc>1){
		int nbCar1 = strlen(argv[1]);

		if (argv[1][nbCar1-3]=='o' && argv[1][nbCar1-2]=='b' && argv[1][nbCar1-1]=='j'){
			*objFileName = argv[1];
		} else if (argv[1][nbCar1-3]=='b' && argv[1][nbCar1-2]=='m' && argv[1][nbCar1-1]=='p'){
			*toonTexFileName = argv[1];
		}

		if (argc>2){
			int nbCar2 = strlen(argv[2]);

			if (argv[2][nbCar2-3]=='o' && argv[2][nbCar2-2]=='b' && argv[2][nbCar2-1]=='j'){
				*objFileName = argv[2];					
			} else if (argv[2][nbCar2-3]=='b' && argv[2][nbCar2-2]=='m' && argv[2][nbCar2-1]=='p'){
				*toonTexFileName = argv[2];					
			}
		}
	}
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800, 600);
	glutCreateWindow("Humanoid");

	char *objFileName, *toonTexFileName;
	parseArgs(argc, argv, &objFileName, &toonTexFileName);
	initScene(objFileName, toonTexFileName);

	glutSpecialFunc(specialKey);
	glutKeyboardFunc(commonKey);

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);

	glutReshapeFunc(changeSize);

	glutMainLoop();

	return(0);
}
