#include <qfiledialog.h>
#include <float.h>

#include "viewer.h"

using namespace std;
using namespace qglviewer;
//-----------------------------------------------------------------
// CONSTRUCTOR & DESTRUCTOR
//-------------------------
Viewer::Viewer(): meshOK_(false),
		  toogleShadowmap_(false)
{
  mesh_ = new Mesh();
  lightFrame_ = new ManipulatedFrame();
  shadowmapSize_ = 512;
}

Viewer::~Viewer() {
  delete mesh_;
  delete lightFrame_;
}


//-----------------------------------------------------------------
// INITIALIZATION
//---------------
void Viewer::init(){
  
  // Background color
  //-----------------
  setBackgroundColor(QColor(255,255,255));
  glClearColor(1.,1.,1.,1.);

  // Key description
  //----------------
  setKeyDescription(Qt::Key_L,"Load new .obj file");
  setKeyDescription(Qt::CTRL+Qt::Key_I,"Initialize light position");

  // Light frame control (Ctrl button)
  //--------------------
  setHandlerKeyboardModifiers(QGLViewer::FRAME, Qt::ControlButton);
  setManipulatedFrame(lightFrame_);
  initShadowmap();


  // Check for useful OpenGL extensions
  //-----------------------------------
  // Shadow Mapping
  if (!checkExtension("GL_ARB_depth_texture")) 
    cerr<<"Warning! GL_ARB_depth_texture unsupported (needed for shadow mapping)!"<<endl; 
  if (!checkExtension("GL_ARB_shadow")) 
    cerr<<"Warning! GL_ARB_shadow unsupported (needed for shadow mapping)!"<<endl;

  restoreStateFromFile(); // Read data from .qglviewer.xml
  help();                 // Help window
}

void Viewer::initShadowmap(){
  glGenTextures(1,&shadowmap_);
  glBindTexture(GL_TEXTURE_2D, shadowmap_);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, shadowmapSize_, shadowmapSize_, 0,
	       GL_DEPTH_COMPONENT, GL_UNSIGNED_BYTE, NULL);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_CLAMP);

}

const bool Viewer::loadMesh(const QString& filename) {
  if (filename.isEmpty()) return false;

  bool res = mesh_->load(filename);
  if (res) {
    setSceneCenter(qglviewer::Vec(mesh_->center()[0],mesh_->center()[1],mesh_->center()[2]));
    setSceneRadius(mesh_->radius()); 
    showEntireScene();
  }
  return res;
}

//-----------------------------------------------------------------
// RENDERING
//----------
void Viewer::draw(){
  glClearColor(1.,1.,1.,1.);
  glClear(GL_COLOR_BUFFER_BIT);

  if (!meshOK_) return;

  // PREPROCESS
  //-----------------

  // Shadow mapping preprocess
  //-------------------------- 
  qglviewer::Vec lightDir = sceneCenter()-lightFrame_->position(); 

  float lightProjectionMatrix [16];
  float lightViewMatrix       [16];
  if (toogleShadowmap_) {

    // Save camera matrices
    glMatrixMode(GL_PROJECTION); glPushMatrix();
    glMatrixMode(GL_MODELVIEW);  glPushMatrix();
    
    // Compute light matrices
    float lightDist = lightDir.normalize();
    
    float fov   = (lightDist<=sceneRadius())? 90.: atan2(sceneRadius(),lightDist-sceneRadius())*180./(float)M_PI;
    float zNear = (lightDist<=sceneRadius())? 1. : lightDist - sceneRadius();
    float zFar  = lightDist + sceneRadius();
    
    glLoadIdentity();
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    gluPerspective(fov, 1., zNear, zFar);
    glGetFloatv(GL_MODELVIEW_MATRIX, lightProjectionMatrix);
    
    glLoadIdentity();
    gluLookAt(lightFrame_->position().x, lightFrame_->position().y, lightFrame_->position().z,
	      sceneCenter().x, sceneCenter().y, sceneCenter().z,
	      0., 1., 0.);
    glGetFloatv(GL_MODELVIEW_MATRIX, lightViewMatrix);
  }

  //---------------  

  // Lighting configuration
  //-----------------------
  float fullLight[4] = {1.,1.,1.,1.};
  float dimLight[4]  = {.8,.8,.8,1.};
  glLightfv(GL_LIGHT0,GL_POSITION,      lightFrame_->position());
  glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,lightDir);
  glLightf (GL_LIGHT0,GL_SPOT_CUTOFF,   45.); 

  // RENDERING
  //----------------- 
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Scene rendering
  //----------------
  glPushAttrib(GL_ENABLE_BIT);

  // Rendering w/ hard shadows
  //--------------------------
  if (toogleShadowmap_) {
    //---------------------------------------------------------
    // 1. Shadow Map computation
    //--------------------------
    glDisable(GL_LIGHT0);
    glDisable(GL_LIGHTING);
    
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(lightProjectionMatrix);
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(lightViewMatrix);
    
    glViewport(0, 0, shadowmapSize_, shadowmapSize_);
    
    
    glCullFace(GL_FRONT);    // Draw back faces only to prevent Z-fighting
    glShadeModel(GL_FLAT);   // Speed up rendering
    glColorMask(0, 0, 0, 0);
    
    glEnable(GL_POLYGON_OFFSET_FILL); // Offset the rendering to prevent Z-fighting
    glPolygonOffset(16.,1.);
    
    //------------------
    drawMesh();
    glFinish();
    glFlush();
    //-----------------
    
    // Depth buffer -> Shadow Map
    glBindTexture(GL_TEXTURE_2D, shadowmap_);
    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, shadowmapSize_, shadowmapSize_);
    
    glCullFace(GL_BACK);    // Restore OpenGL states
    glShadeModel(GL_SMOOTH);
    glColorMask(1, 1, 1, 1);
    
    glDisable(GL_POLYGON_OFFSET_FILL);
    
    
    //---------------------------------------------------------
    // 2. Shaded scene rendering
    //--------------------------
    glClear(GL_DEPTH_BUFFER_BIT);
    
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHTING);
    
    glMatrixMode(GL_MODELVIEW);  glPopMatrix(); // Restore camera modelview matrix
    glMatrixMode(GL_PROJECTION); glPopMatrix(); // Restore camera projection matrix
    
    glViewport(0, 0, width(), height());
    
    glLightfv(GL_LIGHT0, GL_DIFFUSE, dimLight);
    //------------------
    drawMesh();
    glFinish();
    glFlush();
    //------------------
    
    
    //---------------------------------------------------------
    // 3. Lighted scene rendering
    //--------------------------- 
    glDepthFunc(GL_LEQUAL);
    
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    
    // Texture matrix computation
    glLoadIdentity();
    glTranslatef(.5,.5,.5);
    glScalef(.5,.5,.5);
    glMultMatrixf(lightProjectionMatrix);
    glMultMatrixf(lightViewMatrix);
    
    float textureMatrix[16];
    glGetFloatv(GL_MODELVIEW_MATRIX, textureMatrix);
    glPopMatrix();
    
    float row[4];
    row[0]=textureMatrix[0]; row[1]=textureMatrix[4]; row[2]=textureMatrix[8]; row[3]=textureMatrix[12];
    glTexGenfv(GL_S, GL_EYE_PLANE, row);
    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glEnable(GL_TEXTURE_GEN_S);
    
    row[0]=textureMatrix[1]; row[1]=textureMatrix[5]; row[2]=textureMatrix[9]; row[3]=textureMatrix[13];
    glTexGenfv(GL_T, GL_EYE_PLANE, row);
    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glEnable(GL_TEXTURE_GEN_T);
    
    row[0]=textureMatrix[2]; row[1]=textureMatrix[6]; row[2]=textureMatrix[10]; row[3]=textureMatrix[14];
    glTexGenfv(GL_R, GL_EYE_PLANE, row);
    glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glEnable(GL_TEXTURE_GEN_R);
    
    row[0]=textureMatrix[3]; row[1]=textureMatrix[7]; row[2]=textureMatrix[11]; row[3]=textureMatrix[15];
    glTexGenfv(GL_Q, GL_EYE_PLANE, row);
    glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glEnable(GL_TEXTURE_GEN_Q);
    
    // Shadow comparison
    glBindTexture(GL_TEXTURE_2D, shadowmap_);
    glEnable(GL_TEXTURE_2D);
    glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE_ARB, GL_INTENSITY);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE); // Enable shadow comparison
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);               // not in shadow <=> r<=shadow map
    
    //Set alpha test to discard false comparisons
    glAlphaFunc(GL_GEQUAL, 0.99f);
    glEnable(GL_ALPHA_TEST);
    glEnable(GL_BLEND);
    
    glLightfv(GL_LIGHT0, GL_DIFFUSE, fullLight);
    //---------------------
    drawMesh();
    glFinish();
    glFlush();
    //---------------------
  }

  // Rendering w/out shadows
  //------------------------
  else {
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHTING);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, fullLight);
    //---------------------
    drawMesh();
    glFinish();
    glFlush();
    //---------------------
  }

  glPopAttrib();

  
  // ADDITIONNAL RENDERING
  //----------------------
  drawLight();   // Draw a dot to symbolize the light
  
}

void Viewer::drawLight() const {
  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);

  glPointSize(5);
  glBegin(GL_POINTS);
  glColor4f(1.,0.,0.,1.);
  glVertex3fv(lightFrame_->position());
  glColor4f(0.,.5,0.,1.);
  glVertex3fv(sceneCenter());
  glEnd();

  glPopAttrib();
}

void Viewer::drawMesh() const {
  glPushAttrib(GL_ENABLE_BIT);
  glEnable(GL_LIGHTING);
  glEnable(GL_COLOR_MATERIAL);

  mesh_->draw();

  glPopAttrib();
}
//--------------------------------------------------------------------
// USER INTERFACE
//---------------
void Viewer::keyPressEvent(QKeyEvent *e){

  const Qt::ButtonState modifiers = (Qt::ButtonState)(e->state() & Qt::KeyButtonMask);
  if ((e->key()==Qt::Key_L) && (modifiers==Qt::NoButton)) {
    // L = Load new .obj file
    meshOK_ = loadMesh( QFileDialog::getOpenFileName("../../..",
						     "3D Meshes (*.obj *.OBJ)",
						     this,
						     "open file dialog",
						     "Choose a mesh to open"));
  }
  else if ((e->key()==Qt::Key_I) && (modifiers==Qt::ControlButton)) {
    // Ctrl+I = Initialize light position
    lightFrame_->setPosition(sceneCenter());
  }
  else if ((e->key()==Qt::Key_S) && (modifiers==Qt::ControlButton)) {
    // Ctrl+S = Toogle shadow mapping
    toogleShadowmap_ = !toogleShadowmap_;
  }

  updateGL();
}


QString Viewer::helpString() const{

  QString text("<h2>Basic Viewer</h2>");
  return text;
}

const bool Viewer::checkExtension(const char* extName) const{
  int extNameLength = strlen(extName);

  char* p = (char*)glGetString(GL_EXTENSIONS);
  char* end = p+strlen(p);
  
  while (p<end) {
    int n = strcspn(p," ");
    if ((n==extNameLength) && (strncmp(extName,p,n)==0)) {
      return true;
    }
    p += (n+1);
  }
  return false;
}
