#include "Vec3.h"
#include <GL/gl.h>
#include <math.h>

Vec3::Vec3(float x, float y, float z) : x(x), y(y), z(z) {}
Vec3::Vec3(const float *v) : x(v[0]), y(v[1]), z(v[2]) {}

/*********************************************************************************************/

float Vec3::norm2()      const { return x*x + y*y + z*z; }
float Vec3::norm()       const { return sqrt(norm2()); }
Vec3  Vec3::normalized() const { return (*this) / norm(); }
void  Vec3::normalize()        { (*this) /= norm(); }

Vec3 vec(const Vec3 &a, const Vec3 &b) {
    return Vec3(
        a.y * b.z - a.z * b.y,
        a.z * b.x - a.x * b.z,
        a.x * b.y - a.y * b.x
        );
    }

float dot(const Vec3 &a, const Vec3 &b) { return a.x * b.x + a.y * b.y + a.z * b.z; }

float dist(Vec3 v1, Vec3 v2) { return (v2 - v1).norm(); }

Vec3 triangleNormal(const Vec3 &a, const Vec3 &b, const Vec3 &c) { return ((b - a) ^ (c - a)).normalized(); }

/*********************************************************************************************/

Vec3 operator-(const Vec3 &a) { return Vec3(-a.x, -a.y, -a.z); }
    
/*********************************************************************************************/
    
Vec3 operator+(const Vec3 &a, const Vec3 &b) { return Vec3(a.x + b.x, a.y + b.y, a.z + b.z); }
Vec3 operator-(const Vec3 &a, const Vec3 &b) { return Vec3(a.x - b.x, a.y - b.y, a.z - b.z); }
Vec3 operator*(const Vec3 &a, const Vec3 &b) { return Vec3(a.x * b.x, a.y * b.y, a.z * b.z); }
Vec3 operator/(const Vec3 &a, const Vec3 &b) { return Vec3(a.x / b.x, a.y / b.y, a.z / b.z); }

/*********************************************************************************************/

Vec3 operator*(float s, const Vec3 &a) { return Vec3(s * a.x, s * a.y, s * a.z); }
Vec3 operator*(const Vec3 &a, float s) { return s * a; }
Vec3 operator/(const Vec3 &a, float s) { return Vec3(a.x / s, a.y / s, a.z / s); }
Vec3 operator/(float s, const Vec3 &a) { return Vec3(s / a.x, s / a.y, s / a.z); }

/*********************************************************************************************/

Vec3& Vec3::operator+=(const Vec3 &a) { x+=a.x; y+=a.y; z+=a.z; return *this; }
Vec3& Vec3::operator-=(const Vec3 &a) { x-=a.x; y-=a.y; z-=a.z; return *this; }
Vec3& Vec3::operator*=(const Vec3 &a) { x*=a.x; y*=a.y; z*=a.z; return *this; }
Vec3& Vec3::operator/=(const Vec3 &a) { x/=a.x; y/=a.y; z/=a.z; return *this; }
Vec3& Vec3::operator*=(float s) { x*=s; y*=s; z*=s; return *this; }
Vec3& Vec3::operator/=(float s) { x/=s; y/=s; z/=s; return *this; }

/*********************************************************************************************/

bool operator==(const Vec3 &a, const Vec3 &b) { return fabsf(b.x - a.x) < 1e-10 && fabsf(b.y - a.y) < 1e-10 && fabsf(b.z - a.z) < 1e-10;}
bool operator!=(const Vec3 &a, const Vec3 &b) { return fabsf(b.x - a.x) > 1e-10 && fabsf(b.y - a.y) > 1e-10 && fabsf(b.z - a.z) > 1e-10;}
bool operator>=(const Vec3 &a, const Vec3 &b) { return a.x >= b.x && a.y >= b.y && a.z >= b.z; }
bool operator<=(const Vec3 &a, const Vec3 &b) { return a.x <= b.x && a.y <= b.y && a.z <= b.z; }
bool operator> (const Vec3 &a, const Vec3 &b) { return a.x >  b.x && a.y >  b.y && a.z >  b.z; }
bool operator< (const Vec3 &a, const Vec3 &b) { return a.x <  b.x && a.y <  b.y && a.z <  b.z; }
bool operator>=(const Vec3 &a, float  b) { return a.x >= b && a.y >= b && a.z >= b; }
bool operator<=(const Vec3 &a, float  b) { return a.x <= b && a.y <= b && a.z <= b; }
bool operator> (const Vec3 &a, float  b) { return a.x >  b && a.y >  b && a.z >  b; }
bool operator< (const Vec3 &a, float  b) { return a.x <  b && a.y <  b && a.z <  b; }

/*********************************************************************************************/

// dfinition de fonctions de base :
namespace {
    float sign(float a) { return a > 0 ? 1 : (a < 0 ? -1 : 0); }
    float fract(float a) { return a - floorf(a); }
    float step(float e, float x) { return x < e ? 0 : 1; }
    float mix(float x, float y, float a) { return (1 - a) * x + a * y; }
    }
    
Vec3 abs(const Vec3 &a)   { return Vec3(fabsf(a.x),  fabsf(a.y),  fabsf(a.z));  }
Vec3 sign(const Vec3 &a)  { return Vec3(sign(a.x),   sign(a.y),   sign(a.z));   }
Vec3 floor(const Vec3 &a) { return Vec3(floorf(a.x), floorf(a.y), floorf(a.z)); }
Vec3 ceil(const Vec3 &a)  { return Vec3(ceilf(a.x),  ceilf(a.y),  ceilf(a.z));  }
Vec3 fract(const Vec3 &a) { return Vec3(fract(a.x),  fract(a.y),  fract(a.z));  }

Vec3 min(const Vec3 &a, const Vec3 &b) { return Vec3(fminf(a.x, b.x), fminf(a.y, b.y), fminf(a.z, b.z)); }
Vec3 min(const Vec3 &a, float b)       { return Vec3(fminf(a.x, b),   fminf(a.y, b),   fminf(a.z, b));   }
Vec3 max(const Vec3 &a, const Vec3 &b) { return Vec3(fmaxf(a.x, b.x), fmaxf(a.y, b.y), fmaxf(a.z, b.z)); }
Vec3 max(const Vec3 &a, float b)       { return Vec3(fmaxf(a.x, b),   fmaxf(a.y, b),   fmaxf(a.z, b));   }
Vec3 clamp(const Vec3 &a, const Vec3 &vMin, const Vec3 &vMax) { return min(max(a, vMin), vMax); }
Vec3 clamp(const Vec3 &a, float vMin, float vMax)             { return min(max(a, vMin), vMax); }
Vec3 mix(const Vec3 &a, const Vec3 &b, const Vec3 &alpha) { return Vec3(mix(a.x, b.x, alpha.x), mix(a.y, b.y, alpha.y), mix(a.z, b.z, alpha.z)); }
Vec3 mix(const Vec3 &a, const Vec3 &b, float alpha)       { return Vec3(mix(a.x, b.x, alpha),   mix(a.y, b.y, alpha),   mix(a.z, b.z, alpha)); }
Vec3 step(const Vec3 &e, const Vec3 &a) { return Vec3(step(e.x, a.x), step(e.y, a.y), step(e.z, a.z)); }
Vec3 step(float e, const Vec3 &a) { return Vec3(step(e, a.x), step(e, a.y), step(e, a.z)); }

float min(const Vec3 &a) { return fminf(fminf(a.x, a.y), a.z); }
float max(const Vec3 &a) { return fmaxf(fmaxf(a.x, a.y), a.z); }

/*********************************************************************************************/

void Vec3::glVertex() const { glVertex3fv(v); }
void Vec3::glNormal() const { glNormal3fv(v); }


