#include <OpenMesh/Core/IO/MeshIO.hh>
#include <OpenMesh/Core/Mesh/Types/TriMesh_ArrayKernelT.hh>
#include <qdom.h>
#include <QGLViewer/qglviewer.h>
#include <GL/gl.h>

#include <TREES/octree.h>

class Mesh_AmbientOcclusion_FormFactor {
 public :
  Mesh_AmbientOcclusion_FormFactor(){};

  void draw() const;

  void setArea(const float& area)                         { area_=area; radius_=sqrt(area/M_PI); }
  void setCenter(const OpenMesh::Vec3f& center)           { center_=center; }
  void setOrientation(const OpenMesh::Vec3f& orientation) { orientation_=orientation; }
  
  float area()   const { return area_; }
  float radius() const { return radius_; }
  const OpenMesh::Vec3f& center()      const { return center_; }
  const OpenMesh::Vec3f& orientation() const { return orientation_; }

 private :
  float area_,radius_;
  OpenMesh::Vec3f center_;
  OpenMesh::Vec3f orientation_;
};
//-----------------------------------------------------------------------------

typedef OpenMesh::TriMesh_ArrayKernelT<> TriMesh;
class Mesh : public TriMesh {

 public :
  Mesh();
  ~Mesh();

  const OpenMesh::Vec3f   center() const { return center_; }
  float                   radius() const { return radius_; }

  void boundingbox(float& xmin, float& ymin, float& zmin,
		   float& xmax, float& ymax, float& zmax) const {
    xmin=boundingboxMin_[0]; ymin=boundingboxMin_[1]; zmin=boundingboxMin_[2];
    xmax=boundingboxMax_[0]; ymax=boundingboxMax_[1]; zmax=boundingboxMax_[2];
  }

  //------------------------------------
  const bool load(const char* fileName);
  //------------------------------------

  // Rendering
  //----------
  void draw() const;
  void draw(const int& R, const int& G, const int& B, const int& A) const;
  void smoothDraw() const;

  // Curvature computation and smoothing 
  //------------------------------------
  //   (two steps of smoothing : first, geometric smoothing; second, tensor relaxation using RBF )
  //   NB : for the geometric smoothing step, scaleFactor1 and scaleFactor2 must have opposite signs (band-pass filter effect)
  void computeCurvatures(const int&   geometrySmoothingIterations=0,
			 const float& scaleFactor1=+.5, 
			 const float& scaleFactor2=-.75,
			 const int&   curvatureSmoothingIteration=0, 
			 const float& smoothParameter=2000.);

  // Ambient occlusion term computation
  //-----------------------------------
  //   The octree used to store the tree has yet to be exploited... quite long computation times right now...
  const bool buildOctree(const unsigned int& size_min, const unsigned int& depth_max);
  void computeAmbientOcclusion();

  const Octree<VertexHandle>* octree() const { return (octreeOK_)? octree_ : NULL; }

protected:
  inline float minimalCurvature (const VertexHandle& vertex) const;
  inline float maximalCurvature (const VertexHandle& vertex) const;
  inline float meanCurvature    (const VertexHandle& vertex) const;
  inline float gaussianCurvature(const VertexHandle& vertex) const;
  inline float radialCurvature  (const VertexHandle& vertex, 
				 const float& eyeX, const float& eyeY, const float& eyeZ) const;
  inline OpenMesh::Vec3f minimalCurvatureDirection(const VertexHandle& vertex) const;
  inline OpenMesh::Vec3f maximalCurvatureDirection(const VertexHandle& vertex) const;  

 private:
  bool geometryOK_;

  float radius_;
  OpenMesh::Vec3f center_, boundingboxMin_, boundingboxMax_;

  void smoothNormals();
  void updateBoundingVolume();


  //--------------------------------------------------
  // Octree & Ambient occlusion computation
  //---------------------------------------
  bool octreeOK_;
  Octree<VertexHandle>* octree_;
  OpenMesh::VPropHandleT<float> occlusions_;

  //inline void  computeAmbientOcclusion();


  //--------------------------------------------------
  // Surface curvature computation
  //------------------------------
  bool curvatureOK_;
  OpenMesh::VPropHandleT<OpenMesh::Vec3f> vertices_; // Stores the smoothed geometry vertices
  OpenMesh::VPropHandleT<float>           maximalCurvatures_;
  OpenMesh::VPropHandleT<float>           minimalCurvatures_;
  OpenMesh::VPropHandleT<OpenMesh::Vec3f> maximalCurvatureDirections_;
  OpenMesh::VPropHandleT<OpenMesh::Vec3f> minimalCurvatureDirections_;
  
  inline void smoothGeometry(const float& scaleFactor1, const float& scaleFactor2);
  inline void computeCurvatureTensor();
  inline void smoothCurvatures(const float& smoothParameter);
  //----------------------
  inline void curvatureExtrema(float& min, float& max, 
			       const float& eyeX, const float& eyeY, const float& eyeZ) const;
  //------------------------------------------------------------------


  // Tools for rendering and computation
  //------------------------------------
  inline const float sqrNorm (const OpenMesh::Vec3f& p) const;
  inline const float faceArea(const OpenMesh::FaceHandle& face) const;
  inline const float angle   (const OpenMesh::Vec3f& v1, const OpenMesh::Vec3f& v2) const;

  inline double M3x3_determinant(double matrix3x3[9]) const;
  inline void   M4x4_subMatrix(double matrix4x4[16],double matrix3x3[9],int i,int j) const;
  inline double M4x4_determinant(double matrix4x4[16]) const;
  inline void   M4x4_inverse(double matrix4x4[16],double inverse4x4[16]) const;

  inline OpenMesh::Vec3f getEyePosition() const;
  inline bool isNan(OpenMesh::Vec3f vec) const {return ( isnan(vec[0]) || isnan(vec[1]) || isnan(vec[2]) );}
};
